@@ -23,6 +23,10 @@ class TokenRe:
 
 
 def extract_file_tokens(filepath):
+    """
+    Tokenize a C++ file and 'yield' its tokens. For correct
+    line number computation, this also includes comments.
+    """
     # in the following, it's important to use
     # non-capturing groups (?: .... )
     token_types = [
@@ -51,6 +55,14 @@ def extract_file_tokens(filepath):
                 yield t
 
 
+def strip_comment_tokens(tokens):
+    for t in tokens:
+        if t.startswith('//') or t.startswith('/*'):
+            continue
+        else:
+            yield t
+
+
 def pretty_print_token_list(tokens, indent='  '):
     """print all the tokens and auto-indent based on { }"""
     number_nested_braces = 0
@@ -235,7 +247,11 @@ class TokenStream:
                 if t == self.pos:
                     nextline += '^ here'
                 else:
-                    nextline += ' ' * len(self.tokens[t])
+                    if isinstance(self.tokens[t], list):
+                        length = len(self.tokens[t])
+                    else:
+                        length = len(str(self.tokens[t]))
+                    nextline += ' ' * length
             text += '\n' + nextline
             if msg != '':
                 text += "\n'{}'".format(msg)
@@ -381,6 +397,9 @@ class ObjectInformation:
         def set_default_value(self, cpp_token):
             if TokenGroup.IsTokenGroup(cpp_token):
                 # drop surrounding '{' ... '}' if its an initializer list
+                if len(cpp_token.enclosed_tokens) == 0:
+                    cpp_token = None
+                    return
                 cpp_token = cpp_token.enclosed_tokens[0]
             if cpp_token[0:1] == ['-']:
                 # this is most probably a signed number
@@ -390,6 +409,8 @@ class ObjectInformation:
                 # assume that the next token in the list 'cpp_token' is a
                 # token group
                 cpp_token = cpp_token[4].enclosed_tokens[0]
+            if cpp_token[0:3] == ['LayoutAlgorithm', ':', ':']:
+                cpp_token = cpp_token[3]
             if cpp_token == 'WINDOW_MANAGER_NAME':
                 cpp_token = 'herbstluftwm'
             if cpp_token[0:1] == '"':
@@ -401,6 +422,7 @@ class ObjectInformation:
         def __init__(self, cpp_name):
             self.cpp_name = cpp_name
             self.user_name = None  # what the user sees
+            self.user_name_pattern = None  # if no concrete user_name can be given
             self.child_class = None  # whether this is a Link_ or a Child_
             self.type = None  # the template argument to Link_ or Child_
             self.constructor_args = None
@@ -450,6 +472,10 @@ class ObjectInformation:
             if clsname == 'Settings':
                 attr.writable = True
 
+        child_info = self.child_info('PanelManager', '0xWindowID')
+        child_info.user_name_pattern = '0xWindowID'
+        child_info.type = ClassName('Panel')
+
     def attribute_info(self, classname: str, attr_cpp_name: str):
         """return the AttributeInformation object for
         a for a class and its attribute whose C++ variable name is
@@ -577,6 +603,8 @@ class ObjectInformation:
                             'type': member.type.to_user_type_name(),
                             'class': member.child_class,
                         }
+                        if member.user_name_pattern is not None:
+                            obj['name_pattern'] = member.user_name_pattern
                         if member.doc is not None:
                             obj['doc'] = member.doc
                         children[member.user_name] = obj
@@ -622,9 +650,14 @@ class TokTreeInfoExtrator:
             name = arg.value
         if stream.try_match('<'):
             tok = stream.pop('expected template argument')
+            if stream.try_match('*'):
+                tok += '*'
             tmpl_args.append(tok)
             while stream.try_match(','):
-                tmpl_args.append(stream.pop('expected template argument'))
+                tok = stream.pop('expected template argument')
+                if stream.try_match('*'):
+                    tok += '*'
+                tmpl_args.append(tok)
             stream.assert_match('>',
                                 msg='expecting > after last template argument')
         return ClassName(name, type_modifier=type_modifier,
@@ -806,7 +839,7 @@ def main():
         objInfo = ObjectInformation()
         for f in files():
             # print("parsing file {}".format(f), file=sys.stderr)
-            toks = [t for t in extract_file_tokens(f) if t.strip() != '']
+            toks = [t for t in strip_comment_tokens(extract_file_tokens(f)) if t.strip() != '']
             toktree = list(build_token_tree_list(TokenStream(toks)))
             extractor = TokTreeInfoExtrator(objInfo)
             extractor.main(list(toktree))
