@@ -10,8 +10,13 @@ using std::string;
 ByName::ByName(Object& parent_)
     : parent(parent_)
 {
-    parent_.addStaticChild(this, "by-name");
+    parent_.addChild(this, "by-name");
     parent_.addHook(this);
+    setDoc(
+        "This has an entry \'name\' for every object with "
+        "the given \'name\'. If an object has an empty name "
+        "then it is not listed here."
+    );
 }
 
 ByName::~ByName() {
