@@ -36,6 +36,7 @@ static Client* lastfocus = nullptr;
 Client::Client(Window window, bool visible_already, ClientManager& cm)
     : window_(window)
     , dec(make_unique<Decoration>(this, *cm.settings))
+    , float_size_(this, "floating_geometry",  {0, 0, 100, 100})
     , visible_(this, "visible", visible_already)
     , urgent_(this, "urgent", false)
     , floating_(this,  "floating", false)
@@ -56,10 +57,12 @@ Client::Client(Window window, bool visible_already, ClientManager& cm)
     , sizehints_tiling_(this, "sizehints_tiling", false)
     , window_class_(this, "class", &Client::getWindowClass)
     , window_instance_(this, "instance", &Client::getWindowInstance)
+    , content_geometry_(this, "content_geometry", {})
     , manager(cm)
     , theme(*cm.theme)
     , settings(*cm.settings)
     , ewmh(*cm.ewmh)
+    , X_(*cm.X_)
     , mostRecentThemeType(Theme::Type::Tiling)
 {
     stringstream tmp;
@@ -92,7 +95,14 @@ Client::Client(Window window, bool visible_already, ClientManager& cm)
         updateEwmhState();
         hook_emit({"fullscreen", fullscreen_() ? "on" : "off", WindowID(window_).str()});
     });
-    minimized_.changed().connect(this, &Client::updateEwmhState);
+    minimized_.changed().connect([this]() {
+        static long long minimizedTick = 0;
+        minimizedLastChange_ = minimizedTick++;
+        this->updateEwmhState();
+    });
+
+    float_size_.setWritable();
+    float_size_.changedByUser().connect(this, &Client::floatingGeometryChanged);
 
     init_from_X();
     visible_.setDoc("whether this client is rendered currently");
@@ -134,6 +144,16 @@ Client::Client(Window window, bool visible_already, ClientManager& cm)
                              "should be respected in tiling mode");
     sizehints_floating_.setDoc("if sizehints for this client should "
                                "be respected in floating mode");
+    float_size_.setDoc(
+                "the geometry of the client content if the client is in "
+                "floating mode. The position is relative to the monitor "
+                "and does not take the window decoration into account.");
+    content_geometry_.setDoc(
+                "the geometry of the application content, that is, not taking "
+                "the decoration into account. "
+                "Also, this is the last window geometry that "
+                "was reported to the client application."
+     );
 }
 
 void Client::init_from_X() {
@@ -153,11 +173,11 @@ void Client::init_from_X() {
 
 void Client::make_full_client() {
     // setup decoration
-    XSetWindowBorderWidth(g_display, window_, 0);
+    XSetWindowBorderWidth(X_.display(), window_, 0);
     // specify that the client window survives if hlwm dies, i.e. it will be
     // reparented back to root
-    XChangeSaveSet(g_display, window_, SetModeInsert);
-    XReparentWindow(g_display, window_, dec->decorationWindow(), 40, 40);
+    XChangeSaveSet(X_.display(), window_, SetModeInsert);
+    XReparentWindow(X_.display(), window_, dec->decorationWindow(), 40, 40);
     // if this client is visible, then reparenting will make it invisible
     // and will create a unmap notify event
     if (visible_()) {
@@ -165,11 +185,11 @@ void Client::make_full_client() {
         visible_ = false;
     }
     // get events from window
-    XSelectInput(g_display, dec->decorationWindow(), (EnterWindowMask | LeaveWindowMask |
+    XSelectInput(X_.display(), dec->decorationWindow(), (EnterWindowMask | LeaveWindowMask |
                             ButtonPressMask | ButtonReleaseMask |
                             ExposureMask |
                             SubstructureRedirectMask | FocusChangeMask));
-    XSelectInput(g_display, window_,
+    XSelectInput(X_.display(), window_,
                             StructureNotifyMask|FocusChangeMask
                             |EnterWindowMask|PropertyChangeMask);
     // redraw decoration on title change
@@ -177,7 +197,7 @@ void Client::make_full_client() {
 }
 
 void Client::listen_for_events() {
-    XSelectInput(g_display, window_, PropertyChangeMask);
+    XSelectInput(X_.display(), window_, PropertyChangeMask);
 }
 
 void Client::setTag(HSTag *tag) {
@@ -236,7 +256,7 @@ void Client::window_unfocus_last() {
 void Client::window_focus() {
     // set keyboard focus
     if (!this->neverfocus_) {
-        XSetInputFocus(g_display, this->window_, RevertToPointerRoot, CurrentTime);
+        XSetInputFocus(X_.display(), this->window_, RevertToPointerRoot, CurrentTime);
     } else {
         ewmh.sendEvent(window_, Ewmh::WM::TakeFocus, True);
     }
@@ -285,6 +305,11 @@ void Client::raise() {
     this->tag()->stack->raiseSlice(this->slice);
 }
 
+void Client::lower()
+{
+    this->tag()->stack->lowerSlice(this->slice);
+}
+
 /**
  * @brief Client::resize_tiling
  * @param the outer geometry of the client
@@ -298,7 +323,7 @@ void Client::resize_tiling(Rectangle rect, bool isFocused, bool minimalDecoratio
     mostRecentThemeType = themetype;
     auto& scheme = theme[themetype](isFocused, urgent_());
     if (this->pseudotile_) {
-        auto inner = this->float_size_;
+        Rectangle inner = this->float_size_;
         applysizehints(&inner.width, &inner.height);
         auto outline = scheme.inner_rect_to_outline(inner);
         rect.x += std::max(0, (rect.width - outline.width)/2);
@@ -371,7 +396,7 @@ void Client::updatesizehints() {
     long msize;
     XSizeHints size;
 
-    if (!XGetWMNormalHints(g_display, this->window_, &size, &msize)) {
+    if (!XGetWMNormalHints(X_.display(), this->window_, &size, &msize)) {
         /* size is uninitialized, ensure that size.flags aren't used */
         size.flags = PSize;
     }
@@ -420,11 +445,20 @@ void Client::updatesizehints() {
 
 
 
-void Client::send_configure() {
+void Client::send_configure(bool force) {
     auto last_inner_rect = dec->last_inner();
+    // force is just a quick fix: sometimes it is mandatory
+    // to send a configure request even if the geometry didn't change.
+    if (content_geometry_ == last_inner_rect && !force) {
+        // only send the configure notify if the window geometry really changed.
+        // otherwise, sending this might trigger an endless loop between clients
+        // and hlwm.
+        return;
+    }
+    content_geometry_ = last_inner_rect;
     XConfigureEvent ce;
     ce.type = ConfigureNotify;
-    ce.display = g_display;
+    ce.display = X_.display();
     ce.event = this->window_;
     ce.window = this->window_;
     ce.x = last_inner_rect.x;
@@ -434,14 +468,14 @@ void Client::send_configure() {
     ce.border_width = 0;
     ce.above = None;
     ce.override_redirect = False;
-    XSendEvent(g_display, this->window_, False, StructureNotifyMask, (XEvent *)&ce);
+    XSendEvent(X_.display(), this->window_, False, StructureNotifyMask, (XEvent *)&ce);
 }
 
 void Client::resize_floating(Monitor* m, bool isFocused) {
     if (!m) {
         return;
     }
-    auto rect = this->float_size_;
+    Rectangle rect = this->float_size_;
     rect.x += m->rect->x;
     rect.y += m->rect->y;
     rect.x += m->pad_left();
@@ -464,18 +498,6 @@ Rectangle Client::outer_floating_rect() {
     return dec->inner_to_outer(float_size_);
 }
 
-int close_command(Input input, Output) {
-    string winid = "";
-    input >> winid; // try to read, use "" otherwise
-    auto window = get_window(winid);
-    if (window != 0) {
-        Ewmh::get().windowClose(window);
-    } else {
-        return HERBST_INVALID_ARGUMENT;
-    }
-    return 0;
-}
-
 bool Client::is_client_floated() {
     if (floating_()) {
         return true;
@@ -500,16 +522,16 @@ void Client::set_visible(bool visible) {
         /* Grab the server to make sure that the frame window is mapped before
            the client gets its MapNotify, i.e. to make sure the client is
            _visible_ when it gets MapNotify. */
-        XGrabServer(g_display);
+        XGrabServer(X_.display());
         ewmh.windowUpdateWmState(this->window_, WmState::WSNormalState);
-        XMapWindow(g_display, this->window_);
-        XMapWindow(g_display, this->dec->decorationWindow());
-        XUngrabServer(g_display);
+        XMapWindow(X_.display(), this->window_);
+        XMapWindow(X_.display(), this->dec->decorationWindow());
+        XUngrabServer(X_.display());
     } else {
         /* we unmap the client itself so that we can get MapRequest
            events, and because the ICCCM tells us to! */
-        XUnmapWindow(g_display, this->dec->decorationWindow());
-        XUnmapWindow(g_display, this->window_);
+        XUnmapWindow(X_.display(), this->dec->decorationWindow());
+        XUnmapWindow(X_.display(), this->window_);
         ewmh.windowUpdateWmState(this->window_, WmState::WSIconicState);
         this->ignore_unmaps_++;
     }
@@ -538,7 +560,7 @@ void Client::set_urgent_force(bool state) {
     setup_border(this == manager.focus());
 
     XWMHints* wmh;
-    if (!(wmh = XGetWMHints(g_display, this->window_))) {
+    if (!(wmh = XGetWMHints(X_.display(), this->window_))) {
         // just allocate new wm hints for the case the window
         // did not have wm hints set before.
         // here, we ignore what happens on insufficient memory
@@ -550,7 +572,7 @@ void Client::set_urgent_force(bool state) {
         wmh->flags &= ~XUrgencyHint;
     }
 
-    XSetWMHints(g_display, this->window_, wmh);
+    XSetWMHints(X_.display(), this->window_, wmh);
     XFree(wmh);
     ewmh.updateWindowState(this);
     // report changes to tags
@@ -559,7 +581,7 @@ void Client::set_urgent_force(bool state) {
 
 // heavily inspired by dwm.c
 void Client::update_wm_hints() {
-    XWMHints* wmh = XGetWMHints(g_display, this->window_);
+    XWMHints* wmh = XGetWMHints(X_.display(), this->window_);
     if (!wmh) {
         return;
     }
@@ -569,7 +591,7 @@ void Client::update_wm_hints() {
         && wmh->flags & XUrgencyHint) {
         // remove urgency hint if window is focused
         wmh->flags &= ~XUrgencyHint;
-        XSetWMHints(g_display, this->window_, wmh);
+        XSetWMHints(X_.display(), this->window_, wmh);
     } else {
         bool newval = (wmh->flags & XUrgencyHint) ? true : false;
         if (newval != this->urgent_()) {
@@ -607,6 +629,18 @@ void Client::updateEwmhState() {
     ewmh.updateWindowState(this);
 }
 
+void Client::floatingGeometryChanged()
+{
+    Rectangle geom = float_size_();
+    if (sizehints_floating_()) {
+        applysizehints(&geom.width, &geom.height);
+        float_size_ = geom;
+    }
+    if (is_client_floated()) {
+        needsRelayout.emit(tag());
+    }
+}
+
 string Client::getWindowClass()
 {
     return ewmh.X().getClass(window_);
@@ -633,52 +667,11 @@ void Client::requestRedraw()
     }
 }
 
-/**
- * \brief   Resolve a window description to a client
- *
- * \param   str     Describes the window: "" means the focused one, "urgent"
- *                  resolves to a arbitrary urgent window, "0x..." just
- *                  resolves to the given window given its hexadecimal window id,
- *                  a decimal number its decimal window id.
- * \return          Pointer to the resolved client, or null, if client not found
- */
-Client* get_client(const char* str) {
-    if (!strcmp(str, "")) {
-        return get_current_client();
-    } else {
-        return Root::get()->clients()->client(str);
-    }
-}
-
-/**
- * \brief   Resolve a window description to a window
- *
- * \param   str     Describes the window: "" means the focused one, "urgent"
- *                  resolves to a arbitrary urgent window, "0x..." just
- *                  resolves to the given window given its hexadecimal window id,
- *                  a decimal number its decimal window id.
- * \return          Window id, or 0, if unconvertable
- */
-Window get_window(const string& str) {
-    // managed window?
-    auto client = get_client(str.c_str());
-    if (client) {
-        return client->window_;
-    }
-
-    // unmanaged window? try to convert from base 16 or base 10 at the same time
-    try {
-        return Converter<WindowID>::parse(str);
-    } catch (...) {
-        return 0;
-    }
-}
-
 void Client::fuzzy_fix_initial_position() {
     // find out the top-left-most position of the decoration,
     // considering the current settings of possible floating decorations
-    int extreme_x = float_size_.x;
-    int extreme_y = float_size_.y;
+    int extreme_x = float_size_->x;
+    int extreme_y = float_size_->y;
     const auto& t = theme[Theme::Type::Floating];
     mostRecentThemeType = Theme::Type::Floating;
     auto r = t.active.inner_rect_to_outline(float_size_);
@@ -691,8 +684,12 @@ void Client::fuzzy_fix_initial_position() {
     extreme_x = std::min(extreme_x, r.x);
     extreme_y = std::min(extreme_y, r.y);
     // if top left corner might be outside of the monitor, move it accordingly
-    if (extreme_x < 0) { float_size_.x += abs(extreme_x); }
-    if (extreme_y < 0) { float_size_.y += abs(extreme_y); }
+    Point2D delta = {0, 0};
+    if (extreme_x < 0) { delta.x = abs(extreme_x); }
+    if (extreme_y < 0) { delta.y = abs(extreme_y); }
+    if (delta.x || delta.y) {
+        float_size_ = float_size_->shifted(delta);
+    }
 }
 
 void Client::clear_properties() {
