@@ -7,11 +7,11 @@
 #include "attribute_.h"
 #include "child.h"
 #include "commandio.h"
+#include "converter.h"
 #include "object.h"
 #include "rectangle.h"
 #include "regexstr.h"
 #include "theme.h"
-#include "types.h"
 #include "x11-types.h"
 
 class Decoration;
@@ -23,6 +23,7 @@ class HSTag;
 class Monitor;
 class Settings;
 class ClientManager;
+class XConnection;
 
 class Client : public Object {
 public:
@@ -32,7 +33,7 @@ public:
     Window      window_;
     std::unique_ptr<Decoration> dec; // pimpl
     Rectangle   last_size_;      // last size excluding the window border
-    Rectangle   float_size_ = {0, 0, 100, 100};     // floating size without the window border
+    Attribute_<Rectangle> float_size_;     // floating size without the window border
     HSTag*      tag_ = {};
     Slice* slice = {};
     bool        ewmhfullscreen_ = false; // ewmh fullscreen state
@@ -42,6 +43,8 @@ public:
     int         ignore_unmaps_ = 0;  // Ignore one unmap for each reparenting
                                 // action, because reparenting creates an unmap
                                 // notify event
+    //! the last time when minimized_ was changed (with discrete time ticks).
+    long long int minimizedLastChange_ = 0;
     // for size hints
     float mina_ = 0;
     float maxa_ = 0;
@@ -77,6 +80,7 @@ public:
     Attribute_<bool> sizehints_tiling_;  // respect size hints regarding this client in tiling mode
     DynAttribute_<std::string> window_class_;
     DynAttribute_<std::string> window_instance_;
+    Attribute_<Rectangle> content_geometry_;
 
 public:
     void init_from_X();
@@ -111,8 +115,9 @@ public:
     void update_wm_hints();
     void update_title();
     void raise();
+    void lower();
 
-    void send_configure();
+    void send_configure(bool force);
     bool applysizehints(int *w, int *h);
     void updatesizehints();
 
@@ -126,6 +131,7 @@ public:
 
     void updateEwmhState();
 private:
+    void floatingGeometryChanged();
     std::string getWindowClass();
     std::string getWindowInstance();
     std::string triggerRelayoutMonitor();
@@ -136,6 +142,7 @@ private:
     Theme& theme;
     Settings& settings;
     Ewmh& ewmh;
+    XConnection& X_;
     std::string tagName();
     const DecTriple& getDecTriple();
     Theme::Type mostRecentThemeType;
@@ -147,10 +154,6 @@ void reset_client_colors();
 
 Client* get_client_from_window(Window window);
 Client* get_current_client();
-Client* get_client(const char* str);
-Window get_window(const std::string& str);
-
-int close_command(Input input, Output output);
 
 // sets a client property, depending on argv[0]
 int client_set_property_command(int argc, char** argv);
