@@ -3,6 +3,7 @@
 #include <X11/Xlib.h>
 #include <algorithm>
 #include <iostream>
+#include <stdexcept>
 #include <string>
 
 #include "attribute.h"
@@ -10,7 +11,6 @@
 #include "completion.h"
 #include "decoration.h"
 #include "ewmh.h"
-#include "globals.h"
 #include "ipc-protocol.h"
 #include "monitor.h"
 #include "monitormanager.h"
@@ -28,31 +28,35 @@ using std::function;
 using std::string;
 using std::vector;
 
+template<>
+RunTimeConverter<Client*>* Converter<Client*>::converter = nullptr;
+
 ClientManager::ClientManager()
     : focus(*this, "focus")
     , dragged(*this, "dragged")
     , theme(nullptr)
     , settings(nullptr)
     , ewmh(nullptr)
+    , X_(nullptr)
 {
     setDoc("The managed windows. For every (managed) window id there "
            "is an entry here.");
     focus.setDoc("the focused client (only exists if a client is focused)");
     dragged.setDoc("the object of a client which is currently dragged"
                    " by the mouse, if any. See the documentation of the"
-                   "  mousebind command for examples.");
+                   " mousebind command for examples.");
 }
 
 ClientManager::~ClientManager()
 {
     // make all clients visible at their original floating position
     for (auto c : clients_) {
-        auto r = c.second->float_size_;
+        Rectangle r = c.second->float_size_;
         auto window = c.second->x11Window();
-        XMoveResizeWindow(g_display, window, r.x, r.y, r.width, r.height);
-        XReparentWindow(g_display, window, g_root, r.x, r.y);
+        XMoveResizeWindow(X_->display(), window, r.x, r.y, r.width, r.height);
+        XReparentWindow(X_->display(), window, X_->root(), r.x, r.y);
         ewmh->updateFrameExtents(window, 0,0,0,0);
-        XMapWindow(g_display, window);
+        XMapWindow(X_->display(), window);
         delete c.second;
     }
 }
@@ -61,6 +65,12 @@ void ClientManager::injectDependencies(Settings* s, Theme* t, Ewmh* e) {
     settings = s;
     theme = t;
     ewmh = e;
+    X_ = &e->X();
+}
+
+string ClientManager::str(Client* client)
+{
+    return client->window_id_str;
 }
 
 Client* ClientManager::client(Window window)
@@ -73,18 +83,24 @@ Client* ClientManager::client(Window window)
 }
 
 /**
- * \brief   Resolve a window description to a client
+ * \brief   Resolve a window description to a client. If there is
+ *          no such client, throw an exception.
  *
  * \param   str     Describes the window: "" means the focused one, "urgent"
  *                  resolves to a arbitrary urgent window, "0x..." just
  *                  resolves to the given window given its hexadecimal window id,
  *                  a decimal number its decimal window id.
- * \return          Pointer to the resolved client, or null, if client not found
+ * \return          Pointer to the resolved client.
  */
-Client* ClientManager::client(const string &identifier)
+Client* ClientManager::parse(const string& identifier)
 {
     if (identifier.empty()) {
-        return focus();
+        Client* c = focus();
+        if (c) {
+            return c;
+        } else {
+            throw std::invalid_argument("No client is focused");
+        }
     }
     if (identifier == "urgent") {
         for (auto c : clients_) {
@@ -92,20 +108,45 @@ Client* ClientManager::client(const string &identifier)
                 return c.second;
             }
         }
-        return {}; // no urgent client found
+        throw std::invalid_argument("No client is urgent");
     }
+    if (identifier == "last-minimized" || identifier == "longest-minimized") {
+        bool oldest = identifier == "longest-minimized";
+        Client *c = Root::get()->tags()->focus_->minimizedClient(oldest);
+        if (c) {
+            return c;
+        } else {
+            throw std::invalid_argument("No client is minimized");
+        }
+    }
+    Window win = {};
     try {
-        Window win = Converter<WindowID>::parse(identifier);
-        return client(win);
-    } catch (...) {
+        win = Converter<WindowID>::parse(identifier);
+    } catch (const std::exception&) {
+        throw std::invalid_argument("Invalid format, expecting 0xWINID or a description, e.g. \'urgent\'.");
+    }
+    auto entry = clients_.find(win);
+    if (entry != clients_.end()) {
+        return entry->second;
+    } else {
+        throw std::invalid_argument("No managed client with window id " + identifier);
+    }
+}
+
+Client* ClientManager::client(const string &identifier) {
+    try {
+        return parse(identifier);
+    }  catch (...) {
         return nullptr;
     }
 }
 
 //! the completion-counterpart of ClientManager::client()
-void ClientManager::completeClients(Completion& complete)
+void ClientManager::completeEntries(Completion& complete)
 {
     complete.full("urgent");
+    complete.full("last-minimized");
+    complete.full("longest-minimized");
     for (const auto& it : clients_) {
         complete.full(Converter<WindowID>::str(it.first));
     }
@@ -142,7 +183,7 @@ void ClientManager::remove(Window window)
 
 Client* ClientManager::manage_client(Window win, bool visible_already, bool force_unmanage,
                                      function<void(ClientChanges&)> additionalRules) {
-    if (is_herbstluft_window(g_display, win)) {
+    if (is_herbstluft_window(X_->display(), win)) {
         // ignore our own window
         return nullptr;
     }
@@ -161,10 +202,11 @@ Client* ClientManager::manage_client(Window win, bool visible_already, bool forc
     if (additionalRules) {
         additionalRules(changes);
     }
-    changes = Root::get()->rules()->evaluateRules(client, std::cerr, changes);
+    auto stdio = OutputChannels::stdio();
+    changes = Root::get()->rules()->evaluateRules(client, stdio, changes);
     if (!changes.manage || force_unmanage) {
         // map it... just to be sure
-        XMapWindow(g_display, win);
+        XMapWindow(X_->display(), win);
         delete client;
         return {};
     }
@@ -242,7 +284,7 @@ Client* ClientManager::manage_client(Window win, bool visible_already, bool forc
             ewmh->windowUpdateWmState(client->window_, WmState::WSIconicState);
         }
     }
-    client->send_configure();
+    client->send_configure(true);
 
     // TODO: make this better
     Root::get()->mouse->grab_client_buttons(client, false);
@@ -282,7 +324,7 @@ ClientChanges ClientManager::applyDefaultRules(Window win)
     {
         changes.floating = True;
     }
-    if (ewmh->X().getTransientForHint(win).has_value()) {
+    if (X_->getTransientForHint(win).has_value()) {
         changes.floating = true;
     }
     return changes;
@@ -314,6 +356,15 @@ void ClientManager::setSimpleClientAttributes(Client* client, const ClientChange
     if (changes.keysInactive.has_value()) {
         client->keysInactive_ = changes.keysInactive.value();
     }
+    if (changes.floatingGeometry.has_value()) {
+        Rectangle geo = changes.floatingGeometry.value();
+        // do not simply copy the geometry to the attribute
+        // but possibly apply the size hints:
+        if (client->sizehints_floating_()) {
+            client->applysizehints(&geo.width, &geo.height);
+        }
+        client->float_size_ = geo;
+    }
 }
 
 int ClientManager::applyRulesCmd(Input input, Output output) {
@@ -333,7 +384,7 @@ int ClientManager::applyRulesCmd(Input input, Output output) {
     } else {
         Client* client = this->client(winid);
         if (!client) {
-            output << "No such (managed) client: " << winid << "\n";
+            output.perror() << "No such (managed) client: " << winid << "\n";
             return HERBST_INVALID_ARGUMENT;
         }
         return applyRules(client, output);
@@ -358,11 +409,17 @@ int ClientManager::applyChanges(Client* client, ClientChanges changes, Output ou
     if (changes.manage == false) {
         // only make unmanaging clients possible as soon as it is
         // possible to make them managed again
-        output << "Unmanaging clients not yet possible.\n";
+        output.perror() << "Unmanaging clients not yet possible.\n";
         return HERBST_INVALID_ARGUMENT;
     }
     // do the simple attributes first
     setSimpleClientAttributes(client, changes);
+    bool clientNeedsRelayout = false;
+    if (changes.floatingGeometry.has_value()) {
+        // if the floating geometry changed, make sure
+        // that the client's position is updated
+        clientNeedsRelayout = true;
+    }
     if (changes.fullscreen.has_value()) {
         client->fullscreen_ = changes.fullscreen.value();
     }
@@ -393,14 +450,20 @@ int ClientManager::applyChanges(Client* client, ClientChanges changes, Output ou
         }
         TagManager* tagman = Root::get()->tags();
         tagman->moveClient(client, tag, changes.tree_index, changes.focus);
+        clientNeedsRelayout = false; // the above relayouts the tag
     } else if (changes.focus && (client != focus())) {
         // focus the client
         client->tag()->focusClient(client);
         Root::get()->monitors->relayoutTag(client->tag());
+        clientNeedsRelayout = false; // the above relayouts the tag
     }
     if (monitor && switch_tag && tag) {
         monitor_set_tag(monitor, tag);
     }
+    if (clientNeedsRelayout) {
+        // if the client still has not been resized yet:
+        Root::get()->monitors->relayoutTag(client->tag());
+    }
     return 0;
 }
 
@@ -408,7 +471,7 @@ void ClientManager::applyRulesCompletion(Completion& complete)
 {
     if (complete == 0) {
         complete.full("--all");
-        completeClients(complete);
+        completeEntries(complete);
     } else {
         complete.none();
     }
@@ -432,7 +495,7 @@ int ClientManager::applyTmpRuleCmd(Input input, Output output)
         // to only one client
         Client* client = this->client(clientStr);
         if (!client) {
-            output << "No such (managed) client: " << clientStr << "\n";
+            output.perror() << "No such (managed) client: " << clientStr << "\n";
             return HERBST_INVALID_ARGUMENT;
         }
         (*applyTo)[client->x11Window()] = client;
@@ -463,7 +526,7 @@ void ClientManager::applyTmpRuleCompletion(Completion& complete)
 {
     if (complete == 0) {
         complete.full("--all");
-        completeClients(complete);
+        completeEntries(complete);
     } else {
         Root::get()->rules->addRuleCompletion(complete);
     }
@@ -490,11 +553,11 @@ void ClientManager::force_unmanage(Client* client) {
     // remove from tag
     client->tag()->removeClient(client);
     // ignore events from it
-    XSelectInput(g_display, client->window_, 0);
-    //XUngrabButton(g_display, AnyButton, AnyModifier, win);
+    XSelectInput(X_->display(), client->window_, 0);
+    //XUngrabButton(X_->display(), AnyButton, AnyModifier, win);
     // permanently remove it
-    XUnmapWindow(g_display, client->decorationWindow());
-    XReparentWindow(g_display, client->window_, g_root, 0, 0);
+    XUnmapWindow(X_->display(), client->decorationWindow());
+    XReparentWindow(X_->display(), client->window_, X_->root(), 0, 0);
     client->clear_properties();
     HSTag* tag = client->tag();
 
@@ -527,7 +590,7 @@ int ClientManager::clientSetAttribute(string attribute,
         }
         string error_message = a->change(value);
         if (!error_message.empty()) {
-            output << input.command() << ": illegal argument \""
+            output.perror() << "illegal argument \""
                    << value << "\": "
                    << error_message << endl;
             return HERBST_INVALID_ARGUMENT;
