@@ -7,18 +7,6 @@
 
 #include "commandio.h"
 
-// returns a command binding that internalizes object to given a command that
-// calls the member function of the given object
-#define BIND_OBJECT(OBJECT, MEMBER) \
-    (CommandBinding([OBJECT](Input in, Output out) { \
-        return (OBJECT)->MEMBER(in, out); \
-    }))
-
-#define BIND_PARAMETER(PARAM, FUNCTION) \
-    (CommandBinding([PARAM](Input in, Output out) { \
-        return FUNCTION(PARAM, in, out); \
-    }))
-
 class Completion;
 
 /** User facing commands.
@@ -91,14 +79,41 @@ public:
     {
     }
 
+    /** Binding to a combined command invokation and completion
+     *  function.
+     */
+    CommandBinding(std::function<void(CallOrComplete)> callOrCompl)
+    {
+        command = [callOrCompl](Input input, Output output) -> int {
+            std::pair<Input, Output> io = {input, output};
+            int exitCode = 0;
+            CallOrComplete invoc;
+            invoc.inputOutput_ = &io;
+            invoc.exitCode_ = &exitCode;
+            callOrCompl(invoc);
+            return exitCode;
+        };
+        completion_ = [callOrCompl](Completion& complete) {
+            CallOrComplete invoc;
+            invoc.complete_ = &complete;
+            callOrCompl(invoc);
+        };
+    }
+
+    template <typename ClassName>
+    CommandBinding(ClassName* object,
+                   void(ClassName::*member)(CallOrComplete))
+        : CommandBinding(std::bind(member, object,
+                            std::placeholders::_1))
+    {
+    }
+
     // FIXME: Remove after C++ transition
     // The following constructors are only there to ease the transition from
     // C functions to C++
     CommandBinding(int func(int argc, char** argv, Output output))
         : command(commandFromCFunc(func)) {}
-    CommandBinding(int func(int argc, const char** argv, Output output));
     CommandBinding(int func(int argc, char** argv));
-    CommandBinding(int func(int argc, const char** argv));
 
     bool hasCompletion() const { return (bool)completion_; }
     void complete(Completion& completion) const;
@@ -143,7 +158,7 @@ namespace Commands {
 
 // commands
 int list_commands(Output output);
-int complete_command(int argc, char** argv, Output output);
+int completeCommand(Input input, Output output);
 
 #endif
 
