@@ -8,6 +8,7 @@
 #include "argparse.h"
 #include "client.h"
 #include "completion.h"
+#include "finite.h"
 #include "fixprecdec.h"
 #include "framedata.h"
 #include "frameparser.h"
@@ -140,33 +141,6 @@ shared_ptr<FrameLeaf> FrameTree::focusedFrame(shared_ptr<Frame> node) {
 }
 
 
-int FrameTree::cycleSelectionCommand(Input input, Output output) {
-    int delta = 1;
-    string deltaStr;
-    if (input >> deltaStr) {
-        delta = atoi(deltaStr.c_str());
-    }
-    // find current selection
-    auto frame = focusedFrame();
-    auto count = frame->clientCount();
-    if (count != 0) {
-        frame->setSelection(MOD(frame->getSelection() + delta, count));
-    }
-    get_current_monitor()->applyLayout();
-    return 0;
-}
-
-//! focus the nth window within the focused frame
-int FrameTree::focusNthCommand(Input input, Output output) {
-    string index;
-    if (!(input >> index)) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    focusedFrame()->setSelection(atoi(index.c_str()));
-    get_current_monitor()->applyLayout();
-    return 0;
-}
-
 //! command that removes the focused frame
 int FrameTree::removeFrameCommand() {
     auto frame = focusedFrame();
@@ -243,7 +217,7 @@ int FrameTree::rotateCommand() {
 }
 
 template<>
-Finite<FrameTree::MirrorDirection>::ValueList Finite<FrameTree::MirrorDirection>::values = {
+Finite<FrameTree::MirrorDirection>::ValueList Finite<FrameTree::MirrorDirection>::values = ValueListPlain {
     { FrameTree::MirrorDirection::Horizontal, "horizontal" },
     { FrameTree::MirrorDirection::Vertical, "vertical" },
     { FrameTree::MirrorDirection::Both, "both" },
@@ -525,6 +499,36 @@ bool FrameTree::focusInDirection(Direction direction, bool externalOnly)
     return false;
 }
 
+bool FrameTree::shiftInDirection(Direction direction, bool externalOnly) {
+    shared_ptr<FrameLeaf> sourceFrame = this->focusedFrame();
+    Client* client = sourceFrame->focusedClient();
+    if (!client) {
+        return false;
+    }
+    // don't look for neighbours within the frame if 'external_only' is set
+    int indexInFrame = externalOnly ? (-1) : sourceFrame->getInnerNeighbourIndex(direction);
+    if (indexInFrame >= 0) {
+        sourceFrame->moveClient(indexInFrame);
+        return true;
+    } else {
+        shared_ptr<Frame> neighbour = sourceFrame->neighbour(direction);
+        if (neighbour) { // if neighbour was found
+            // move window to neighbour
+            sourceFrame->removeClient(client);
+            FrameTree::focusedFrame(neighbour)->insertClient(client);
+            neighbour->frameWithClient(client)->select(client);
+
+            // change selection in parent
+            shared_ptr<FrameSplit> parent = neighbour->getParent();
+            assert(parent);
+            parent->swapSelection();
+            return true;
+        } else {
+            return false;
+        }
+    }
+}
+
 //! go to the specified frame. Return true on success, return false if
 //! the end is reached (this command never wraps). Skips covered windows
 //! if skipInvisible is set.
@@ -629,10 +633,10 @@ int FrameTree::cycleFrameCommand(Input input, Output output) {
     try {
         delta = std::stoi(s);
     } catch (std::invalid_argument const& e) {
-        output << "invalid argument: " << e.what() << endl;
+        output.perror() << "invalid argument: " << e.what() << endl;
         return HERBST_INVALID_ARGUMENT;
     } catch (std::out_of_range const& e) {
-        output << "out of range: " << e.what() << endl;
+        output.perror() << "out of range: " << e.what() << endl;
         return HERBST_INVALID_ARGUMENT;
     }
     cycle_frame(delta);
@@ -640,6 +644,26 @@ int FrameTree::cycleFrameCommand(Input input, Output output) {
     return 0;
 }
 
+void FrameTree::loadCompletion(Completion& complete)
+{
+    if (complete == 0) {
+        Converter<HSTag*>::complete(complete);
+    } else if (complete == 1) {
+        try {
+            // check if the first parameter parses as a tag
+            // name.
+            Converter<HSTag*>::parse(complete[0]);
+        }  catch (...) {
+            // if not, then complete[0] was already the
+            // layout string, so there is no second argument
+            complete.none();
+        }
+    } else {
+        // there can be up to two arguments
+        complete.none();
+    }
+}
+
 int FrameTree::loadCommand(Input input, Output output) {
     // usage: load TAG LAYOUT
     HSTag* tag = nullptr;
@@ -648,7 +672,7 @@ int FrameTree::loadCommand(Input input, Output output) {
         input >> tagName >> layoutString;
         tag = find_tag(tagName.c_str());
         if (!tag) {
-            output << input.command() << ": Tag \"" << tagName << "\" not found\n";
+            output.perror() << "Tag \"" << tagName << "\" not found\n";
             return HERBST_INVALID_ARGUMENT;
         }
     } else if (input.size() == 1) {
@@ -660,29 +684,29 @@ int FrameTree::loadCommand(Input input, Output output) {
     assert(tag != nullptr);
     FrameParser parsingResult(layoutString);
     if (parsingResult.error_) {
-        output << input.command() << ": Syntax error at "
+        output.perror() << "Syntax error at "
                << parsingResult.error_->first.first << ": "
                << parsingResult.error_->second << ":"
                << endl;
         std::regex whitespace ("[ \n\t]");
         // print the layout again
-        output << "\"" << std::regex_replace(layoutString, whitespace, string(" "))
+        output.error() << "\"" << std::regex_replace(layoutString, whitespace, string(" "))
                << "\"" << endl;
         // and underline the token
         int token_len = std::max((size_t)1, parsingResult.error_->first.second.size());
-        output << " " // for the \" above
+        output.error() << " " // for the \" above
                << string(parsingResult.error_->first.first, ' ')
                << string(token_len, '~')
                << endl;
         return HERBST_INVALID_ARGUMENT;
     }
     if (!parsingResult.unknownWindowIDs_.empty()) {
-        output << "Warning: Unknown window IDs";
+        output.perror() << "Warning: Unknown window IDs";
         for (const auto& e : parsingResult.unknownWindowIDs_) {
-            output << " " << WindowID(e.second).str()
+            output.error() << " " << WindowID(e.second).str()
                    << "(\'" << e.first.second << "\')";
         }
-        output << endl;
+        output.error() << endl;
     }
     // apply the new frame tree
     tag->frame->applyFrameTree(tag->frame->root_, parsingResult.root_);
@@ -786,7 +810,7 @@ int FrameTree::cycleLayoutCommand(Input input, Output output) {
         try {
             delta = Converter<int>::parse(deltaStr);
         } catch (const std::exception& e) {
-            output << input.command() << ": " << e.what();
+            output.perror() << e.what() << endl;
             return HERBST_INVALID_ARGUMENT;
         }
     }
@@ -805,7 +829,7 @@ int FrameTree::cycleLayoutCommand(Input input, Output output) {
         try {
             layout_index = (int)Converter<LayoutAlgorithm>::parse(*(input.begin() + idx));
         } catch (const std::exception& e) {
-            output << input.command() << ": " << e.what();
+            output.perror() << e.what() << endl;
             return HERBST_INVALID_ARGUMENT;
         }
     } else {
@@ -851,17 +875,18 @@ void FrameTree::setLayoutCompletion(Completion& complete) {
     }
 }
 
+
+
 //! modes for the 'split' command
 class SplitMode {
 public:
-    SplitMode(string name_, SplitAlign align_, bool frameToFirst_, int selection_)
-        : name(name_)
-          , align(align_)
+
+    SplitMode(SplitAlign align_, bool frameToFirst_, int selection_)
+          : align(align_)
           , frameToFirst(frameToFirst_)
           , selection(selection_)
     {}
 
-    string name;
     SplitAlign align;
 
     //! If former frame moves to first child
@@ -870,51 +895,74 @@ public:
     //! Which child to select after the split
     int selection;
 
-    static vector<SplitMode> modes(SplitAlign align_explode = SplitAlign::horizontal, SplitAlign align_auto = SplitAlign::horizontal);
+    static std::map<SplitModeName,SplitMode> modes(SplitAlign align_auto);
 };
 
-vector<SplitMode> SplitMode::modes(SplitAlign align_explode, SplitAlign align_auto)
+template<>
+Finite<SplitModeName>::ValueList Finite<SplitModeName>::values = {
+    { FiniteNameFlags::AllowPrefixesAsNames },
+    {
+        { SplitModeName::Top,        "top" },
+        { SplitModeName::Bottom,     "bottom" },
+        { SplitModeName::Vertical,   "vertical" },
+        { SplitModeName::Right,      "right" },
+        { SplitModeName::Horizontal, "horizontal" },
+        { SplitModeName::Left,       "left" },
+        { SplitModeName::Explode,    "explode" },
+        { SplitModeName::Auto,       "auto" },
+}};
+
+std::map<SplitModeName, SplitMode> SplitMode::modes(SplitAlign align_auto)
 {
     return {
-        { "top",        SplitAlign::vertical,     false,  1   },
-        { "bottom",     SplitAlign::vertical,     true,   0   },
-        { "vertical",   SplitAlign::vertical,     true,   0   },
-        { "right",      SplitAlign::horizontal,   true,   0   },
-        { "horizontal", SplitAlign::horizontal,   true,   0   },
-        { "left",       SplitAlign::horizontal,   false,  1   },
-        { "explode",    align_explode,            true,   -1  },
-        { "auto",       align_auto,               true,   0   },
+        { SplitModeName::Top,        { SplitAlign::vertical,     false,  1   }},
+        { SplitModeName::Bottom,     { SplitAlign::vertical,     true,   0   }},
+        { SplitModeName::Vertical,   { SplitAlign::vertical,     true,   0   }},
+        { SplitModeName::Right,      { SplitAlign::horizontal,   true,   0   }},
+        { SplitModeName::Horizontal, { SplitAlign::horizontal,   true,   0   }},
+        { SplitModeName::Left,       { SplitAlign::horizontal,   false,  1   }},
+        { SplitModeName::Explode,    { SplitAlign::vertical,     true,   -1  }},
+        { SplitModeName::Auto,       { align_auto,               true,   0   }},
     };
 }
 
-int FrameTree::splitCommand(Input input, Output output)
+void FrameTree::splitCommand(CallOrComplete invoc)
 {
-    // usage: split t|b|l|r|h|v FRACTION [Frameindex]
-    string splitType;
+    // usage: split t|b|l|r|h|v [FRACTION] [Frameindex]
+    vector<string> fractionSuggestions = {
+        "0.1", "0.2", "0.3", "0.4", "0.5", "0.6", "0.7", "0.8", "0.9"
+    };
+    SplitModeName splitType;
     bool userDefinedFraction = false;
     FixPrecDec fraction = FixPrecDec::approxFrac(1, 2);
     string frameIndex = "@"; // split the focused frame per default
     ArgParse ap;
-    ap.mandatory(splitType).optional(fraction, &userDefinedFraction);
+    ap.mandatory(splitType).optional(fraction, fractionSuggestions, &userDefinedFraction);
     ap.optional(frameIndex);
-    if (ap.parsingFails(input, output)) {
-        return ap.exitCode();
-    }
+    ap.command(invoc, [&](Output) {
+        splitFrame(frameIndex, splitType, fraction, userDefinedFraction);
+        return 0;
+    });
+}
+
+/**
+ * @brief FrameTree::splitFrame
+ * @param The frame to split
+ * @param how to split it
+ * @param an optional split ratio
+ * @param Wether the split ratio is explicitly wanted or can be ignored in mode 'explode'
+ */
+void FrameTree::splitFrame(string frameIndex, SplitModeName mode, FixPrecDec fraction, bool userDefinedFraction) {
     fraction = FrameSplit::clampFraction(fraction);
     shared_ptr<Frame> frame = lookup(frameIndex);
     int lh = frame->lastRect().height;
     int lw = frame->lastRect().width;
     SplitAlign align_auto = (lw > lh) ? SplitAlign::horizontal : SplitAlign::vertical;
-    SplitAlign align_explode = SplitAlign::vertical;
-    auto availableModes = SplitMode::modes(align_explode, align_auto);
-    auto mode = std::find_if(
-            availableModes.begin(), availableModes.end(),
-            [=](const SplitMode &x){ return x.name[0] == splitType[0]; });
-    if (mode == availableModes.end()) {
-        output << input.command() << ": Invalid alignment \"" << splitType << "\"\n";
-        return HERBST_INVALID_ARGUMENT;
-    }
-    SplitMode m = *mode;
+    auto allSplitModes = SplitMode::modes(align_auto);
+    auto it = allSplitModes.find(mode);
+    // we know that 'mode' must be in the map allSplitModes, but let us stay safe
+    // and fall back to the first if it is not the case:
+    SplitMode m = ((it != allSplitModes.end()) ? it : allSplitModes.begin())->second;
     auto layout = frame->switchcase<LayoutAlgorithm>(&FrameLeaf::getLayout,
         [] (shared_ptr<FrameSplit> f) {
             return splitAlignToLayoutAlgorithm(f->getAlign());
@@ -924,7 +972,7 @@ int FrameTree::splitCommand(Input input, Output output)
     // that stay in the 'old frame'.
     auto windowcount = frame->switchcase<size_t>(&FrameLeaf::clientCount,
         [](shared_ptr<FrameSplit>) { return 0; });
-    bool exploding = m.name == "explode";
+    bool exploding = mode == SplitModeName::Explode;
     if (exploding) {
         if (windowcount <= 1) {
             m.align = align_auto;
@@ -955,13 +1003,13 @@ int FrameTree::splitCommand(Input input, Output output)
     shared_ptr<FrameSplit> frameParent = {}; // the new parent
     if (frameIsLeaf) {
         if (!frameIsLeaf->split(m.align, fraction, childrenLeaving)) {
-            return 0;
+            return;
         }
         frameParent = frameIsLeaf->getParent();
     }
     if (frameIsSplit) {
         if (!frameIsSplit->split(m.align, fraction)) {
-            return 0;
+            return;
         }
         frameParent = frameIsSplit->getParent();
     }
@@ -974,8 +1022,8 @@ int FrameTree::splitCommand(Input input, Output output)
     }
 
     // redraw monitor
-    get_current_monitor()->applyLayout();
-    return 0;
+    tag_->needsRelayout_.emit();
+    return;
 }
 
 
@@ -991,7 +1039,7 @@ int FrameTree::dumpLayoutCommand(Input input, Output output) {
         if (!tagName.empty()) {
             HSTag* tag = find_tag(tagName.c_str());
             if (!tag) {
-                output << input.command() << ": Tag \"" << tagName << "\" not found\n";
+                output.perror() << "Tag \"" << tagName << "\" not found\n";
                 return HERBST_INVALID_ARGUMENT;
             }
             tree = tag->frame();
@@ -1013,7 +1061,7 @@ int FrameTree::dumpLayoutCommand(Input input, Output output) {
 
 void FrameTree::dumpLayoutCompletion(Completion& complete) {
     if (complete == 0) {
-        global_tags->completeTag(complete);
+        global_tags->completeEntries(complete);
     } else if (complete == 1) {
         // no completion for frame index
     } else {
