@@ -42,14 +42,10 @@ Frame::~Frame() = default;
 FrameLeaf::FrameLeaf(HSTag* tag, Settings* settings, weak_ptr<FrameSplit> parent)
     : Frame(tag, settings, parent)
     , client_count_(this, "client_count", [this]() {return clientCount(); })
-    , selectionAttr_(this, "selection", &FrameLeaf::getSelection)
-    , algorithmAttr_(this, "algorithm", &FrameLeaf::getLayout)
+    , selectionAttr_(this, "selection", &FrameLeaf::getSelection, &FrameLeaf::userSetsSelection)
+    , algorithmAttr_(this, "algorithm", &FrameLeaf::getLayout, &FrameLeaf::userSetsLayout)
 {
-    auto l = settings->default_frame_layout();
-    if (l >= layoutAlgorithmCount()) {
-        l = 0;
-    }
-    layout = (LayoutAlgorithm) l;
+    layout = settings->default_frame_layout();
 
     decoration = new FrameDecoration(*this, tag, settings);
 }
@@ -58,9 +54,9 @@ FrameSplit::FrameSplit(HSTag* tag, Settings* settings, weak_ptr<FrameSplit> pare
                        FixPrecDec fraction, SplitAlign align, shared_ptr<Frame> a,
                        shared_ptr<Frame> b)
              : Frame(tag, settings, parent)
-             , splitTypeAttr_(this, "split_type", &FrameSplit::getAlign)
-             , fractionAttr_(this, "fraction", &FrameSplit::getFraction)
-             , selectionAttr_(this, "selection", [this]() { return selection_;})
+             , splitTypeAttr_(this, "split_type", &FrameSplit::getAlign, &FrameSplit::userSetsSplitType)
+             , fractionAttr_(this, "fraction", &FrameSplit::getFraction, &FrameSplit::userSetsFraction)
+             , selectionAttr_(this, "selection", &FrameSplit::getSelection, &FrameSplit::userSetsSelection)
              , aLink_(*this, "0")
              , bLink_(*this, "1")
 {
@@ -158,8 +154,28 @@ shared_ptr<FrameSplit> FrameSplit::thisSplit() {
     return dynamic_pointer_cast<FrameSplit>(shared_from_this());
 }
 
-shared_ptr<FrameLeaf> Frame::getGloballyFocusedFrame() {
-    return get_current_monitor()->tag->frame->focusedFrame();
+string FrameSplit::userSetsSplitType(SplitAlign align)
+{
+    align_ = align;
+    relayout();
+    return {};
+}
+
+string FrameSplit::userSetsSelection(int idx)
+{
+    if (idx < 0 || idx > 1) {
+        return "index out of range";
+    }
+    selection_ = idx;
+    relayout();
+    return {};
+}
+
+string FrameSplit::userSetsFraction(FixPrecDec fraction)
+{
+    setFraction(fraction);
+    relayout();
+    return {};
 }
 
 string Frame::frameIndex() const
@@ -175,6 +191,16 @@ string Frame::frameIndex() const
     }
 }
 
+/**
+ * @brief trigger relayouting of the frame tree.
+ * This should only be called if the user changes something
+ * via the attribute system.
+ */
+void Frame::relayout()
+{
+    tag_->needsRelayout_.emit();
+}
+
 TilingResult FrameLeaf::layoutLinear(Rectangle rect, bool vertical) {
     TilingResult res;
     auto cur = rect;
@@ -429,37 +455,6 @@ void Frame::foreachClient(ClientAction action) {
          0);
 }
 
-int frame_current_bring(int argc, char** argv, Output output) {
-    if (argc < 2) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    auto client = get_client(argv[1]);
-    if (!client) {
-        output << argv[0] << ": Could not find client";
-        if (argc > 1) {
-            output << " \"" << argv[1] << "\".\n";
-        } else {
-            output << ".\n";
-        }
-        return HERBST_INVALID_ARGUMENT;
-    }
-    HSTag* tag = get_current_monitor()->tag;
-    global_tags->moveClient(client, tag, {}, true);
-    // mark as un-minimized first, such that a minimized tiling client
-    // is added to the frame tree now.
-    client->minimized_ = false;
-    auto frame = tag->frame->root_->frameWithClient(client);
-    if (frame && !frame->isFocused()) {
-        // regardless of the client's floating or minimization state:
-        // if the client was in the frame tree, it is moved
-        // to the focused frame
-        frame->removeClient(client);
-        tag->frame->focusedFrame()->insertClient(client, true);
-    }
-    focus_client(client, false, false, true);
-    return 0;
-}
-
 void FrameLeaf::setSelection(int index) {
     if (clients.empty()) {
         return;
@@ -702,6 +697,29 @@ int FrameLeaf::getInnerNeighbourIndex(Direction direction) {
     return index;
 }
 
+string FrameLeaf::userSetsLayout(LayoutAlgorithm algo)
+{
+    setLayout(algo);
+    relayout();
+    return {};
+}
+
+string FrameLeaf::userSetsSelection(int index)
+{
+    if (clients.empty() && index == 0) {
+        // if there is no client in this frame
+        // then index 0 is the fallback value and
+        // there is nothing to be done here
+        return {};
+    }
+    if (index < 0 || static_cast<size_t>(index) >= clients.size()) {
+        return "index out of range";
+    }
+    setSelection(index);
+    relayout();
+    return {};
+}
+
 void FrameLeaf::moveClient(int new_index) {
     swap(clients[new_index], clients[selection]);
     selection = new_index;
@@ -799,32 +817,3 @@ vector<Client*> FrameLeaf::removeAllClients() {
     selection = 0;
     return result;
 }
-
-int frame_focus_edge(Input input, Output output) {
-    // Puts the focus to the edge in the specified direction
-    g_monitors->lock();
-    int oldval = g_settings->focus_crosses_monitor_boundaries();
-    g_settings->focus_crosses_monitor_boundaries = false;
-    Input inp = {"focus", input.toVector()};
-    while (0 == Commands::call(inp, output)) {
-        ;
-    }
-    g_settings->focus_crosses_monitor_boundaries = oldval;
-    g_monitors->unlock();
-    return 0;
-}
-
-int frame_move_window_edge(Input input, Output output) {
-    // Moves a window to the edge in the specified direction
-    g_monitors->lock();
-    int oldval = g_settings->focus_crosses_monitor_boundaries();
-    g_settings->focus_crosses_monitor_boundaries = false;
-    Input inp = {"shift", input.toVector()};
-    while (0 == Commands::call(inp, output)) {
-        ;
-    }
-    g_settings->focus_crosses_monitor_boundaries = oldval;
-    g_monitors->unlock();
-    return 0;
-}
-
