@@ -7,11 +7,11 @@
 #include <memory>
 
 #include "attribute_.h"
+#include "converter.h"
 #include "framedata.h"
 #include "link.h"
 #include "object.h"
 #include "tilingresult.h"
-#include "types.h"
 #include "x11-types.h"
 
 // execute an action on an client
@@ -20,6 +20,7 @@ class Client;
 typedef std::function<void(Client*)> ClientAction;
 
 class HSTag;
+class FrameDecoration;
 class FrameLeaf;
 class FrameSplit;
 class Settings;
@@ -52,12 +53,8 @@ public:
     // count the number of splits to the root with alignment "align"
     virtual int splitsToRoot(SplitAlign align);
 
-    HSTag* getTag() { return tag_; };
-
     void setVisibleRecursive(bool visible);
 
-    static std::shared_ptr<FrameLeaf> getGloballyFocusedFrame();
-
     /*! a case distinction on the type of tree node. If `this` is a
      * FrameSplit, then onSplit is called, and otherwise onLeaf is called.
      * The return value is passed through.
@@ -82,11 +79,12 @@ public:
     friend class HSTag; // for HSTag::foreachClient()
     DynAttribute_<std::string> frameIndexAttr_;
     std::string frameIndex() const;
+    void foreachClient(ClientAction action);
 public: // soon will be protected:
     virtual std::shared_ptr<FrameSplit> isSplit() { return std::shared_ptr<FrameSplit>(); };
     virtual std::shared_ptr<FrameLeaf> isLeaf() { return std::shared_ptr<FrameLeaf>(); };
 protected:
-    void foreachClient(ClientAction action);
+    void relayout();
     HSTag* tag_;
     Settings* settings_;
     std::weak_ptr<FrameSplit> parent_;
@@ -137,6 +135,9 @@ public:
     DynAttribute_<int> selectionAttr_;
     DynAttribute_<LayoutAlgorithm> algorithmAttr_;
 private:
+    std::string userSetsLayout(LayoutAlgorithm algo);
+    std::string userSetsSelection(int index);
+    friend class FrameDecoration;
     friend class FrameTree;
     // layout algorithms
     TilingResult layoutLinear(Rectangle rect, bool vertical);
@@ -175,29 +176,27 @@ public:
     void swapChildren();
     void adjustFraction(FixPrecDec delta);
     void setFraction(FixPrecDec fraction);
-    FixPrecDec getFraction() const { return fraction_; }
+    FixPrecDec getFraction() { return fraction_; }
     static FixPrecDec clampFraction(FixPrecDec fraction);
     std::shared_ptr<FrameSplit> thisSplit();
     std::shared_ptr<FrameSplit> isSplit() override { return thisSplit(); }
     SplitAlign getAlign() { return align_; }
     void swapSelection() { selection_ = selection_ == 0 ? 1 : 0; }
     void setSelection(int s) { selection_ = s; }
+    int getSelection() { return selection_; }
     DynAttribute_<SplitAlign> splitTypeAttr_;
     DynAttribute_<FixPrecDec> fractionAttr_;
     DynAttribute_<int> selectionAttr_;
     Link_<Frame> aLink_;
     Link_<Frame> bLink_;
 private:
+    std::string userSetsSplitType(SplitAlign align);
+    std::string userSetsFraction(FixPrecDec fraction);
+    std::string userSetsSelection(int idx);
     friend class FrameTree;
 };
 
-// functions
-int frame_current_bring(int argc, char** argv, Output output);
-
 bool focus_client(Client* client, bool switch_tag, bool switch_monitor, bool raise);
 
-int frame_focus_edge(Input input, Output output);
-int frame_move_window_edge(Input input, Output output);
-
 #endif
 
