@@ -17,6 +17,7 @@
 #include "ewmh.h"
 #include "fontdata.h"
 #include "frametree.h"
+#include "globalcommands.h"
 #include "globals.h"
 #include "hook.h"
 #include "ipc-protocol.h"
@@ -48,7 +49,6 @@ using std::unique_ptr;
 // globals:
 int g_verbose = 0;
 Display*    g_display;
-int         g_screen;
 Window      g_root;
 
 // module internals:
@@ -59,17 +59,15 @@ static XMainLoop* g_main_loop = nullptr;
 
 int quit();
 int version(Output output);
-int print_tag_status_command(int argc, char** argv, Output output);
 void execute_autostart_file();
-int raise_command(int argc, char** argv, Output output);
 int spawn(int argc, char** argv);
 int wmexec(int argc, char** argv);
 static void remove_zombies(int signal);
 int custom_hook_emit(Input input);
-int jumpto_command(int argc, char** argv, Output output);
 
 unique_ptr<CommandTable> commands(shared_ptr<Root> root) {
     MetaCommands* meta_commands = root->meta_commands.get();
+    GlobalCommands* global_cmds = root->global_commands.get();
 
     ClientManager* clients = root->clients();
     KeyManager *keys = root->keys();
@@ -113,26 +111,25 @@ unique_ptr<CommandTable> commands(shared_ptr<Root> root) {
         {"spawn",          spawn},
         {"wmexec",         wmexec},
         {"emit_hook",      { custom_hook_emit }},
-        {"bring",          frame_current_bring},
-        {"focus_nth",      { tags->frameCommand(&FrameTree::focusNthCommand) }},
-        {"cycle",          { tags->frameCommand(&FrameTree::cycleSelectionCommand) }},
-        {"cycle_all",      monitors->tagCommand(&HSTag::cycleAllCommand,
-                                                &HSTag::cycleAllCompletion)},
+        {"bring",          {global_cmds, &GlobalCommands::bringCommand }},
+        {"focus_nth",      {global_cmds, &GlobalCommands::focusNthCommand }},
+        {"cycle",          { monitors->tagCommand(&HSTag::cycleCommand) }},
+        {"cycle_all",      monitors->tagCommand(&HSTag::cycleAllCommand)},
         {"cycle_layout",   tags->frameCommand(&FrameTree::cycleLayoutCommand, &FrameTree::cycleLayoutCompletion) },
         {"cycle_frame",    { tags->frameCommand(&FrameTree::cycleFrameCommand) }},
-        {"close",          { close_command }},
+        {"close",          { global_cmds, &GlobalCommands::closeCommand }},
         {"close_or_remove",{ monitors->tagCommand(&HSTag::closeOrRemoveCommand) }},
         {"close_and_remove",{ monitors->tagCommand(&HSTag::closeAndRemoveCommand) }},
         {"split",          { tags->frameCommand(&FrameTree::splitCommand) }},
         {"resize",         monitors->tagCommand(&HSTag::resizeCommand,
                                                 &HSTag::resizeCompletion)},
-        {"focus_edge",     frame_focus_edge},
-        {"focus",          monitors->tagCommand(&HSTag::focusInDirCommand,
-                                                &HSTag::focusInDirCompletion)},
-        {"shift_edge",     frame_move_window_edge},
-        {"shift",          monitors->tagCommand(&HSTag::shiftInDirCommand,
-                                                &HSTag::shiftInDirCompletion)},
-        {"shift_to_monitor",shift_to_monitor},
+        {"focus_edge",     {global_cmds, &GlobalCommands::focusEdgeCommand,
+                                         &GlobalCommands::focusEdgeCompletion}},
+        {"focus",          monitors->tagCommand(&HSTag::focusInDirCommand)},
+        {"shift_edge",     {global_cmds, &GlobalCommands::shiftEdgeCommand,
+                                         &GlobalCommands::shiftEdgeCompletion}},
+        {"shift",          monitors->tagCommand(&HSTag::shiftInDirCommand)},
+        {"shift_to_monitor",{ monitors, &MonitorManager::shiftToMonitorCommand }},
         {"remove",         { tags->frameCommand(&FrameTree::removeFrameCommand) }},
         {"set",            { settings, &Settings::set_cmd,
                                        &Settings::set_complete }},
@@ -140,37 +137,39 @@ unique_ptr<CommandTable> commands(shared_ptr<Root> root) {
                                        &Settings::get_complete }},
         {"toggle",         { settings, &Settings::toggle_cmd,
                                        &Settings::toggle_complete}},
-        {"cycle_value",    { settings, &Settings::cycle_value_cmd,
-                                       &Settings::cycle_value_complete}},
-        {"cycle_monitor",  monitor_cycle_command},
-        {"focus_monitor",  monitor_focus_command},
-        {"add",            BIND_OBJECT(tags, tag_add_command) },
-        {"use",            monitor_set_tag_command},
-        {"use_index",      monitor_set_tag_by_index_command},
-        {"use_previous",   { monitor_set_previous_tag_command }},
-        {"jumpto",         jumpto_command},
+        {"cycle_value",    { global_cmds, &GlobalCommands::cycleValueCommand,
+                                          &GlobalCommands::cycleValueCompletion}},
+        {"cycle_monitor",  { monitors, &MonitorManager::cycleCommand }},
+        {"focus_monitor",  { monitors, &MonitorManager::focusCommand }},
+        {"add",            { tags, &TagManager::addCommand }},
+        {"use",            { global_cmds, &GlobalCommands::useTagCommand }},
+        {"use_index",      { global_cmds, &GlobalCommands::useTagByIndexCommand }},
+        {"use_previous",   { global_cmds, &GlobalCommands::usePreviousCommand }},
+        {"jumpto",         { global_cmds, &GlobalCommands::jumptoCommand }},
         {"floating",       { tags, &TagManager::floatingCmd,
                                    &TagManager::floatingComplete }},
         {"fullscreen",     {clients, &ClientManager::fullscreen_cmd,
                                      &ClientManager::fullscreen_complete}},
         {"pseudotile",     {clients, &ClientManager::pseudotile_cmd,
                                      &ClientManager::pseudotile_complete}},
-        {"tag_status",     print_tag_status_command},
-        {"merge_tag",      BIND_OBJECT(tags, removeTag)},
-        {"rename",         BIND_OBJECT(tags, tag_rename_command) },
-        {"move",           BIND_OBJECT(tags, tag_move_window_command) },
+        {"tag_status",     {global_cmds, &GlobalCommands::tagStatusCommand}},
+        {"merge_tag",      { tags, &TagManager::mergeTagCommand }},
+        {"rename",         { tags, &TagManager::tag_rename_command }},
+        {"move",           { tags, &TagManager::tag_move_window_command }},
         {"rotate",         { tags->frameCommand(&FrameTree::rotateCommand) }},
         {"mirror",         { tags->frameCommand(&FrameTree::mirrorCommand, &FrameTree::mirrorCompletion) }},
-        {"move_index",     BIND_OBJECT(tags, tag_move_window_by_index_command) },
-        {"add_monitor",    BIND_OBJECT(monitors, addMonitor)},
+        {"move_index",     { tags, &TagManager::tag_move_window_by_index_command }},
+        {"add_monitor",    { monitors, &MonitorManager::addMonitorCommand }},
         {"raise_monitor",  { monitors, &MonitorManager::raiseMonitorCommand,
                                        &MonitorManager::raiseMonitorCompletion }},
-        {"remove_monitor", BIND_OBJECT(monitors, removeMonitor)},
+        {"remove_monitor", { monitors, &MonitorManager::removeMonitorCommand }},
         {"move_monitor",   monitors->byFirstArg(&Monitor::move_cmd, &Monitor::move_complete) } ,
         {"rename_monitor", monitors->byFirstArg(&Monitor::renameCommand, &Monitor::renameComplete) },
-        {"monitor_rect",   monitor_rect_command},
-        {"pad",            monitor_set_pad_command},
-        {"raise",          raise_command},
+        {"monitor_rect",   { monitors, &MonitorManager::rectCommand }},
+        {"pad",            { monitors, &MonitorManager::padCommand }},
+        {"raise",          { global_cmds, &GlobalCommands::raiseCommand }},
+        {"lower",          { global_cmds, &GlobalCommands::lowerCommand }},
+        {"list_clients",   { global_cmds, &GlobalCommands::listClientsCommand }},
         {"rule",           {rules, &RuleManager::addRuleCommand,
                                    &RuleManager::addRuleCompletion}},
         {"unrule",         {rules, &RuleManager::unruleCommand,
@@ -183,9 +182,9 @@ unique_ptr<CommandTable> commands(shared_ptr<Root> root) {
         {"layout",         tags->frameCommand(&FrameTree::dumpLayoutCommand, &FrameTree::dumpLayoutCompletion)},
         {"stack",          { monitors, &MonitorManager::stackCommand }},
         {"dump",           tags->frameCommand(&FrameTree::dumpLayoutCommand, &FrameTree::dumpLayoutCompletion)},
-        {"load",           { tags->frameCommand(&FrameTree::loadCommand) }},
-        {"complete",       complete_command},
-        {"complete_shell", complete_command},
+        {"load",           { tags->frameCommand(&FrameTree::loadCommand, &FrameTree::loadCompletion ) }},
+        {"complete",       completeCommand},
+        {"complete_shell", completeCommand},
         {"lock",           { [monitors] { monitors->lock(); return 0; } }},
         {"unlock",         { [monitors] { monitors->unlock(); return 0; } }},
         {"lock_tag",       monitors->byFirstArg(&Monitor::lock_tag_cmd, &Monitor::noComplete) },
@@ -227,6 +226,8 @@ unique_ptr<CommandTable> commands(shared_ptr<Root> root) {
                                             &MetaCommands::get_attr_complete }},
         {"set_attr",       { meta_commands, &MetaCommands::set_attr_cmd,
                                             &MetaCommands::set_attr_complete }},
+        {"attr_type",      { meta_commands, &MetaCommands::attrTypeCommand,
+                                            &MetaCommands::attrTypeCompletion }},
         {"help",           { meta_commands, &MetaCommands::helpCommand,
                                             &MetaCommands::helpCompletion }},
         {"attr",           { meta_commands, &MetaCommands::attr_cmd,
@@ -257,48 +258,6 @@ int version(Output output) {
     return 0;
 }
 
-int print_tag_status_command(int argc, char** argv, Output output) {
-    Monitor* monitor;
-    if (argc >= 2) {
-        monitor = string_to_monitor(argv[1]);
-    } else {
-        monitor = get_current_monitor();
-    }
-    if (!monitor) {
-        output << argv[0] << ": Monitor \"" << argv[1] << "\" not found!\n";
-        return HERBST_INVALID_ARGUMENT;
-    }
-    tag_update_flags();
-    output << '\t';
-    for (int i = 0; i < tag_get_count(); i++) {
-        HSTag* tag = get_tag_by_index(i);
-        // print flags
-        char c = '.';
-        if (tag->flags & TAG_FLAG_USED) {
-            c = ':';
-        }
-        Monitor *tag_monitor = find_monitor_with_tag(tag);
-        if (tag_monitor == monitor) {
-            c = '+';
-            if (monitor == get_current_monitor()) {
-                c = '#';
-            }
-        } else if (tag_monitor) {
-            c = '-';
-            if (get_current_monitor() == tag_monitor) {
-                c = '%';
-            }
-        }
-        if (tag->flags & TAG_FLAG_URGENT) {
-            c = '!';
-        }
-        output << c;
-        output << *tag->name;
-        output << '\t';
-    }
-    return 0;
-}
-
 int custom_hook_emit(Input input) {
     hook_emit(input.toVector());
     return 0;
@@ -306,7 +265,7 @@ int custom_hook_emit(Input input) {
 
 static void execvp_helper(char *const command[]) {
     execvp(command[0], command);
-    std::cerr << "herbstluftwm: execvp \"" << command << "\"";
+    std::cerr << "herbstluftwm: execvp \"" << command[0] << "\"";
     perror(" failed");
 }
 
@@ -359,40 +318,6 @@ int wmexec(int argc, char** argv) {
     return EXIT_SUCCESS;
 }
 
-int raise_command(int argc, char** argv, Output output) {
-    if (argc < 2) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    auto client = get_client(argv[1]);
-    if (client) {
-        client->raise();
-        client->needsRelayout.emit(client->tag());
-    } else {
-        auto window = get_window(argv[1]);
-        if (window) {
-            XRaiseWindow(g_display, window);
-        } else {
-            output << argv[0] << ": Could not find client \"" << argv[1] << "\".\n";
-            return HERBST_INVALID_ARGUMENT;
-        }
-    }
-    return 0;
-}
-
-int jumpto_command(int argc, char** argv, Output output) {
-    if (argc < 2) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    auto client = get_client(argv[1]);
-    if (client) {
-        focus_client(client, true, true, true);
-        return 0;
-    } else {
-        output << argv[0] << ": Could not find client \"" << argv[1] << "\".\n";
-        return HERBST_INVALID_ARGUMENT;
-    }
-}
-
 void execute_autostart_file() {
     string path;
     if (g_autostart_path) {
@@ -432,12 +357,16 @@ void execute_autostart_file() {
 
 static void parse_arguments(int argc, char** argv, Globals& g) {
     int exit_on_xerror = g.exitOnXlibError;
+    int noTransparency = !g.trueTransparency;
     int no_tag_import = 0;
+    int replace = 0;
     struct option long_options[] = {
         {"version",         0, nullptr, 'v'},
         {"help",            0, nullptr, 'h'},
         {"autostart",       1, nullptr, 'c'},
         {"locked",          0, nullptr, 'l'},
+        {"replace",         0, &replace, 1},
+        {"no-transparency", 0, &noTransparency, 1},
         {"exit-on-xerror",  0, &exit_on_xerror, 1},
         {"no-tag-import",   0, &no_tag_import, 1},
         {"verbose",         0, &g_verbose, 1},
@@ -455,8 +384,11 @@ static void parse_arguments(int argc, char** argv, Globals& g) {
                 /* ignore recognized long option */
                 break;
             case 'v':
-                version(std::cout);
-                exit(0);
+                {
+                    auto stdio = OutputChannels::stdio();
+                    version(stdio);
+                    exit(0);
+                }
             case 'c':
                 g_autostart_path = optarg;
                 break;
@@ -492,8 +424,10 @@ static void parse_arguments(int argc, char** argv, Globals& g) {
                 exit(EXIT_FAILURE);
         }
     }
+    g.replaceExistingWm = replace != 0;
     g.exitOnXlibError = exit_on_xerror != 0;
     g.importTagsFromEwmh = (no_tag_import == 0);
+    g.trueTransparency = !noTransparency;
 }
 
 static void remove_zombies(int) {
@@ -537,25 +471,29 @@ int main(int argc, char* argv[]) {
         delete X;
         exit(EXIT_FAILURE);
     }
-    if (X->checkotherwm()) {
-        std::cerr << "herbstluftwm: another window manager is already running" << endl;
+    Ewmh* ewmh = new Ewmh(*X);
+    if (!ewmh->acquireScreenSelection(g.replaceExistingWm) || X->otherWmListensRoot()) {
+        std::cerr << "herbstluftwm: another window manager is already running (try --replace)" << endl;
         delete X;
         exit(EXIT_FAILURE);
     }
+    if (g.trueTransparency) {
+        X->tryInitTransparency();
+    }
     // remove zombies on SIGCHLD
     sigaction_signal(SIGCHLD, remove_zombies);
     sigaction_signal(SIGINT,  handle_signal);
     sigaction_signal(SIGQUIT, handle_signal);
     sigaction_signal(SIGTERM, handle_signal);
     // set some globals
-    g_screen = X->screen();
     g_root = X->root();
     XSelectInput(X->display(), X->root(), SubstructureRedirectMask|SubstructureNotifyMask|ButtonPressMask|EnterWindowMask|LeaveWindowMask|StructureNotifyMask);
+    ewmh->installWmWindow();
 
     // setup ipc server
     IpcServer* ipcServer = new IpcServer(*X);
     FontData::s_xconnection = X;
-    auto root = make_shared<Root>(g, *X, *ipcServer);
+    auto root = make_shared<Root>(g, *X, *ewmh, *ipcServer);
     Root::setRoot(root);
     //test_object_system();
 
@@ -566,7 +504,7 @@ int main(int argc, char* argv[]) {
 
     // setup
     if (g.importTagsFromEwmh) {
-        const auto& initialState = root->ewmh->initialState();
+        const auto& initialState = ewmh->initialState();
         for (auto n : initialState.desktopNames) {
             root->tags->add_tag(n.c_str());
         }
@@ -575,18 +513,22 @@ int main(int argc, char* argv[]) {
     mainloop.scanExistingClients();
     tag_force_update_flags();
     all_monitors_apply_layout();
-    root->ewmh->updateAll();
+    ewmh->updateAll();
     execute_autostart_file();
 
     // main loop
     mainloop.run();
 
-    // enforce to clear the root
+    // Shut everything down. Root::get() still works.
+    root->shutdown();
+    // clear the root to destroy the object.
+    // Now Root::get() does not work anymore.
     root.reset();
     Root::setRoot(root);
     // and then close the x connection
     FontData::s_xconnection = nullptr;
     delete ipcServer;
+    delete ewmh;
     delete X;
     // check if we shall restart an other window manager
     if (g_exec_before_quit) {
