@@ -57,7 +57,7 @@ int MetaCommands::set_attr_cmd(Input in, Output output) {
     if (error_message.empty()) {
         return 0;
     } else {
-        output << in.command() << ": \""
+        output.perror() << "\""
             << new_value << "\" is not a valid value for "
             << a->name() << ": "
             << error_message << endl;
@@ -96,7 +96,7 @@ int MetaCommands::attr_cmd(Input in, Output output) {
         if (error_message.empty()) {
             return 0;
         } else {
-            output << in.command() << ": \""
+            output.perror() << "\""
                 << new_value << "\" is not a valid value for "
                 << a->name() << ": "
                 << error_message << endl;
@@ -109,7 +109,7 @@ Attribute* MetaCommands::getAttribute(string path, Output output) {
     auto attr_path = Object::splitPath(path);
     auto child = root.child(attr_path.first);
     if (!child) {
-        output << "No such object " << attr_path.first.join('.') << endl;
+        output.perror() << "No such object " << attr_path.first.join('.') << endl;
         return nullptr;
     }
     Attribute* a = child->attribute(attr_path.second);
@@ -121,7 +121,7 @@ Attribute* MetaCommands::getAttribute(string path, Output output) {
             // equip object_path with quotes
             object_path = "Object \"" + object_path + "\"";
         }
-        output << object_path
+        output.perror() << object_path
                << " has no attribute \"" << attr_path.second << "\""
                << endl;
         return nullptr;
@@ -136,7 +136,7 @@ int MetaCommands::print_object_tree_command(Input in, Output output) {
     }
     auto child = root.child(path);
     if (!child) {
-        output << "No such object " << Path(path).join('.') << endl;
+        output.perror() << "No such object " << Path(path).join('.') << endl;
         return HERBST_INVALID_ARGUMENT;
     }
     child->printTree(output, Path(path).join('.'));
@@ -151,6 +151,37 @@ void MetaCommands::print_object_tree_complete(Completion& complete) {
     }
 }
 
+int MetaCommands::attrTypeCommand(Input input, Output output)
+{
+    string attrName;
+    ArgParse ap;
+    ap.mandatory(attrName);
+    if (ap.parsingAllFails(input, output)) {
+        return ap.exitCode();
+    }
+
+    Attribute* a = getAttribute(attrName, output);
+    if (!a) {
+        return HERBST_INVALID_ARGUMENT;
+    }
+    auto it = type_strings.find(a->type());
+    string typeName = "unknown"; // this should not happen anyway
+    if (it != type_strings.end()) {
+        typeName = it->second.first;
+    }
+    output << typeName << endl;
+    return 0;
+}
+
+void MetaCommands::attrTypeCompletion(Completion& complete)
+{
+    if (complete == 0) {
+        completeAttributePath(complete);
+    } else {
+        complete.none();
+    }
+}
+
 
 int MetaCommands::substitute_cmd(Input input, Output output)
 {
@@ -175,9 +206,12 @@ void MetaCommands::substitute_complete(Completion& complete)
     } else if (complete == 1) {
         completeAttributePath(complete);
     } else {
-        // later, complete the identifier
-        complete.full(complete[0]);
         complete.completeCommands(2);
+        if (!complete.noParameterExpected()) {
+            // later, complete the identifier if the nested command
+            // still expects parameters
+            complete.full(complete[0]);
+        }
     }
 }
 
@@ -282,7 +316,7 @@ int MetaCommands::sprintf_cmd(Input input, Output output)
     try {
         format = parseFormatString(formatStringSrc);
     }  catch (const std::invalid_argument& e) {
-        output << input.command() << ": " << e.what() << endl;
+        output.perror() << e.what() << endl;
         return HERBST_INVALID_ARGUMENT;
     }
     // evaluate placeholders in the format string
@@ -362,7 +396,7 @@ static std::map<string, function<Attribute*(string)>> name2constructor {
 Attribute* MetaCommands::newAttributeWithType(string typestr, string attr_name, Output output) {
     auto it = name2constructor.find(typestr);
     if (it == name2constructor.end()) {
-        output << "error: unknown type \"" << typestr << "\"";
+        output.perror() << "unknown type \"" << typestr << "\"";
         return nullptr;
     }
     auto attr = it->second(attr_name);
@@ -396,15 +430,15 @@ int MetaCommands::new_attr_cmd(Input input, Output output)
         return HERBST_INVALID_ARGUMENT;
     }
     if (attr_name.substr(0,strlen(USER_ATTRIBUTE_PREFIX)) != USER_ATTRIBUTE_PREFIX) {
-        output
-            << input.command() << ": attribute name must start with \""
+        output.perror()
+            << "attribute name must start with \""
             << USER_ATTRIBUTE_PREFIX << "\""
             << " but is actually \"" << attr_name << "\"" << endl;
         return HERBST_INVALID_ARGUMENT;
     }
     if (obj->attribute(attr_name)) {
-        output
-            << input.command() << ": object \"" << obj_path_and_attr.first.join()
+        output.perror()
+            << "object \"" << obj_path_and_attr.first.join()
             << "\" already has an attribute named \"" << attr_name
             <<  "\"" << endl;
         return HERBST_INVALID_ARGUMENT;
@@ -420,7 +454,7 @@ int MetaCommands::new_attr_cmd(Input input, Output output)
     if (initialValueSupplied) {
         string msg = a->change(initialValue);
         if (!msg.empty()) {
-            output << input.command() << ": \""
+            output.perror() << "\""
                    << initialValue << "\" is an invalid "
                    << "value for " << path << ": " << msg << endl;
             return HERBST_INVALID_ARGUMENT;
@@ -460,7 +494,7 @@ int MetaCommands::remove_attr_cmd(Input input, Output output)
         return HERBST_INVALID_ARGUMENT;
     }
     if (a->name().substr(0,strlen(USER_ATTRIBUTE_PREFIX)) != USER_ATTRIBUTE_PREFIX) {
-        output << input.command() << ": Cannot remove built-in attribute \"" << path << "\"" << endl;
+        output.perror() << "Cannot remove built-in attribute \"" << path << "\"" << endl;
         return HERBST_INVALID_ARGUMENT;
     }
     a->detachFromOwner();
@@ -513,7 +547,7 @@ template <typename T> int parse_and_compare(string a, string b, Output o) {
         try {
             vals.push_back(Converter<T>::parse(x));
         } catch(std::exception& e) {
-            o << "cannot parse \"" << x << "\" to "
+            o.perror() << "cannot parse \"" << x << "\" to "
               << typeid(T).name() << ": " << e.what() << endl;
             return (int) HERBST_INVALID_ARGUMENT;
         }
@@ -526,7 +560,7 @@ using CompareOperator = pair<bool, vector<int> >;
 
 template <>
 struct is_finite<CompareOperator> : std::true_type {};
-template<> Finite<CompareOperator>::ValueList Finite<CompareOperator>::values = {
+template<> Finite<CompareOperator>::ValueList Finite<CompareOperator>::values = ValueListPlain {
     // map operator names to "for numeric types only" and possible return codes
     { { false, { 0 }, }, "=", },
     { { false, { -1, 1 } }, "!=", },
@@ -556,7 +590,7 @@ int MetaCommands::compare_cmd(Input input, Output output)
     std::map<Type, pair<bool, function<int(string,string,Output)>>> type2compare {
         // map a type name to "is it numeric" and a comperator function
         { Type::INT,      { true,  parse_and_compare<int> }, },
-        { Type::ULONG,    { true,  parse_and_compare<int> }, },
+        { Type::ULONG,    { true,  parse_and_compare<unsigned long> }, },
         { Type::BOOL,     { false, parse_and_compare<bool> }, },
         { Type::COLOR,    { false, parse_and_compare<Color> }, },
     };
@@ -568,7 +602,8 @@ int MetaCommands::compare_cmd(Input input, Output output)
         comparator = it->second;
     }
     if (oper.first && !comparator.first) {
-        output << "operator \"" << Converter<CompareOperator>::str(oper) << "\" "
+        output.perror()
+            << "operator \"" << Converter<CompareOperator>::str(oper) << "\" "
             << "only allowed for numeric types, but the attribute "
             << path << " is of non-numeric type "
             << Entity::typestr(a->type()) << endl;
@@ -707,7 +742,7 @@ int MetaCommands::helpCommand(Input input, Output output)
         object->ls(output);
     }
     if (!helpFound) {
-        output << "No help found for \'" << needle << "\'" << endl;
+        output.perror() << "No help found for \'" << needle << "\'" << endl;
         return HERBST_INVALID_ARGUMENT;
     }
     return 0;
@@ -770,8 +805,12 @@ int MetaCommands::tryCommand(Input input, Output output) {
 
 int MetaCommands::silentCommand(Input input, Output output) {
     stringstream dummyOutput;
+    // discard output and error channel
+    // TODO: pass through the error channel as soon as
+    // the ipc-protocol supports it.
+    OutputChannels discardOutputChannels(output.command(), dummyOutput, dummyOutput);
     // drop output but pass exit code
-    return Commands::call(input.fromHere(), dummyOutput);
+    return Commands::call(input.fromHere(), discardOutputChannels);
 }
 
 int MetaCommands::negateCommand(Input input, Output output)
@@ -802,8 +841,8 @@ int MetaCommands::setenvCommand(Input input, Output output) {
         return HERBST_NEED_MORE_ARGS;
     }
     if (setenv(name.c_str(), value.c_str(), 1) != 0) {
-        output << input.command()
-               << ": Could not set environment variable: "
+        output.perror()
+               << "Could not set environment variable: "
                << strerror(errno) << endl;
         return HERBST_UNKNOWN_ERROR;
     }
@@ -874,8 +913,8 @@ int MetaCommands::unsetenvCommand(Input input, Output output) {
         return HERBST_NEED_MORE_ARGS;
     }
     if (unsetenv(name.c_str()) != 0) {
-        output << input.command()
-               << ": Could not unset environment variable: "
+        output.perror()
+               << "Could not unset environment variable: "
                << strerror(errno) << endl;
         return HERBST_UNKNOWN_ERROR;
     }
@@ -936,6 +975,10 @@ void MetaCommands::chainCompletion(Completion& complete)
             complete.full(complete[0]);
         }
         complete.completeCommands(lastsep + 1);
+        // even if the completion of the parameter
+        // claimed that no parameters are expected, there still
+        // could be another separator and another command
+        complete.parametersStillExpected();
     }
 }
 
