@@ -7,7 +7,7 @@
 
 #include "attribute.h"
 #include "commandio.h"
-#include "types.h"
+#include "converter.h"
 
 class Object;
 class Completion;
@@ -42,6 +42,8 @@ public:
     void attr_complete(Completion& complete);
     int print_object_tree_command(Input in, Output output);
     void print_object_tree_complete(Completion& complete);
+    int attrTypeCommand(Input input, Output output);
+    void attrTypeCompletion(Completion& complete);
 
     int substitute_cmd(Input input, Output output);
     void substitute_complete(Completion& complete);
