@@ -300,7 +300,7 @@ int Monitor::move_cmd(Input input, Output output) {
     }
     auto new_rect = Rectangle::fromStr(input.front());
     if (new_rect.width < WINDOW_MIN_WIDTH || new_rect.height < WINDOW_MIN_HEIGHT) {
-        output << input.command() << ": Rectangle is too small\n";
+        output.perror() << "Rectangle is too small\n";
         return HERBST_INVALID_ARGUMENT;
     }
     // else: just move it:
@@ -347,7 +347,7 @@ int Monitor::renameCommand(Input input, Output output) {
     }
     string error = name.change(new_name);
     if (!error.empty()) {
-        output << input.command() << ": " << error << "\n";
+        output.perror() << error << endl;
         return HERBST_INVALID_ARGUMENT;
     } else {
         return 0;
@@ -364,75 +364,6 @@ void Monitor::renameComplete(Completion& complete)
    }
 }
 
-int monitor_rect_command(int argc, char** argv, Output output) {
-    // usage: monitor_rect [[-p] INDEX]
-    char* monitor_str = nullptr;
-    Monitor* m = nullptr;
-    bool with_pad = false;
-
-    // if monitor is supplied
-    if (argc > 1) {
-        monitor_str = argv[1];
-    }
-    // if -p is supplied
-    if (argc > 2) {
-        monitor_str = argv[2];
-        if (!strcmp("-p", argv[1])) {
-            with_pad = true;
-        } else {
-            output << argv[0] <<
-                ": Invalid argument \"" << argv[1] << "\"\n";
-            return HERBST_INVALID_ARGUMENT;
-        }
-    }
-    // if an index is set
-    if (monitor_str) {
-        m = string_to_monitor(monitor_str);
-        if (!m) {
-            output << argv[0] <<
-                ": Monitor \"" << monitor_str << "\" not found!\n";
-            return HERBST_INVALID_ARGUMENT;
-        }
-    } else {
-        m = get_current_monitor();
-    }
-    auto rect = m->rect();
-    if (with_pad) {
-        rect.x += m->pad_left;
-        rect.width -= m->pad_left + m->pad_right;
-        rect.y += m->pad_up;
-        rect.height -= m->pad_up + m->pad_down;
-    }
-    output << rect.x << " " << rect.y << " " << rect.width << " " << rect.height;
-    return 0;
-}
-
-int monitor_set_pad_command(int argc, char** argv, Output output) {
-    if (argc < 2) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    Monitor* monitor = string_to_monitor(argv[1]);
-    if (!monitor) {
-        output << argv[0] <<
-            ": Monitor \"" << argv[1] << "\" not found!\n";
-        return HERBST_INVALID_ARGUMENT;
-    }
-    if (argc > 2 && argv[2][0] != '\0') {
-        monitor->pad_up = atoi(argv[2]);
-    }
-    if (argc > 3 && argv[3][0] != '\0') {
-        monitor->pad_right = atoi(argv[3]);
-    }
-    if (argc > 4 && argv[4][0] != '\0') {
-        monitor->pad_down = atoi(argv[4]);
-    }
-    if (argc > 5 && argv[5][0] != '\0') {
-        monitor->pad_left = atoi(argv[5]);
-    }
-    monitor->applyLayout();
-    return 0;
-}
-
 Monitor* find_monitor_with_tag(HSTag* tag) {
     for (auto m : *g_monitors) {
         if (m->tag == tag) {
@@ -523,93 +454,6 @@ int monitor_set_tag(Monitor* monitor, HSTag* tag) {
     return 0;
 }
 
-int monitor_set_tag_command(int argc, char** argv, Output output) {
-    if (argc < 2) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    Monitor* monitor = get_current_monitor();
-    HSTag*  tag = find_tag(argv[1]);
-    if (monitor && tag) {
-        int ret = monitor_set_tag(monitor, tag);
-        if (ret != 0) {
-            output << argv[0] << ": Could not change tag";
-            if (monitor->lock_tag) {
-                output << " (monitor " << monitor->index() << " is locked)";
-            }
-            output << "\n";
-        }
-        return ret;
-    } else {
-        output << argv[0] <<
-            ": Invalid tag \"" << argv[1] << "\"\n";
-        return HERBST_INVALID_ARGUMENT;
-    }
-}
-
-int monitor_set_tag_by_index_command(int argc, char** argv, Output output) {
-    if (argc < 2) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    bool skip_visible = false;
-    if (argc >= 3 && !strcmp(argv[2], "--skip-visible")) {
-        skip_visible = true;
-    }
-    HSTag* tag = global_tags->byIndexStr(argv[1], skip_visible);
-    if (!tag) {
-        output << argv[0] <<
-            ": Invalid index \"" << argv[1] << "\"\n";
-        return HERBST_INVALID_ARGUMENT;
-    }
-    int ret = monitor_set_tag(get_current_monitor(), &* tag);
-    if (ret != 0) {
-        output << argv[0] <<
-            ": Could not change tag (maybe monitor is locked?)\n";
-    }
-    return ret;
-}
-
-int monitor_set_previous_tag_command(Output output) {
-    Monitor* monitor = get_current_monitor();
-    HSTag*  tag = monitor->tag_previous;
-    if (monitor && tag) {
-        int ret = monitor_set_tag(monitor, tag);
-        if (ret != 0) {
-            output << "use_previous: Could not change tag (maybe monitor is locked?)\n";
-        }
-        return ret;
-    } else {
-        output << "use_previous: Invalid monitor or tag\n";
-        return HERBST_INVALID_ARGUMENT;
-    }
-}
-
-int monitor_focus_command(int argc, char** argv, Output output) {
-    if (argc < 2) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    int new_selection = g_monitors->string_to_monitor_index(argv[1]);
-    if (new_selection < 0) {
-        output << argv[0] <<
-            ": Monitor \"" << argv[1] << "\" not found!\n";
-        return HERBST_INVALID_ARGUMENT;
-    }
-    // really change selection
-    monitor_focus_by_index((unsigned)new_selection);
-    return 0;
-}
-
-int monitor_cycle_command(int argc, char** argv) {
-    int delta = 1;
-    auto count = g_monitors->size();
-    if (argc >= 2) {
-        delta = atoi(argv[1]);
-    }
-    int new_selection = g_monitors->cur_monitor + delta; // signed for delta calculations
-    // really change selection
-    monitor_focus_by_index((unsigned)MOD(new_selection, count));
-    return 0;
-}
-
 void monitor_focus_by_index(unsigned new_selection) {
     // clamp to last
     new_selection = std::min(g_monitors->size() - 1, (size_t)new_selection);
@@ -709,28 +553,6 @@ void Monitor::restack() {
     XRestackWindows(g_display, buf.data(), buf.size());
 }
 
-int shift_to_monitor(int argc, char** argv, Output output) {
-    if (argc <= 1) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    char* monitor_str = argv[1];
-    Monitor* monitor = string_to_monitor(monitor_str);
-    if (!monitor) {
-        output << monitor_str << ": Invalid monitor\n";
-        return HERBST_INVALID_ARGUMENT;
-    }
-    global_tags->moveFocusedClient(monitor->tag);
-    return 0;
-}
-
-void all_monitors_replace_previous_tag(HSTag *old, HSTag *newmon) {
-    for (auto m : *g_monitors) {
-        if (m->tag_previous == old) {
-            m->tag_previous = newmon;
-        }
-    }
-}
-
 Rectangle Monitor::getFloatingArea() const {
     auto m = this;
     auto r = m->rect();
@@ -761,9 +583,10 @@ void Monitor::evaluateClientPlacement(Client* client, ClientPlacement placement)
                     // the center of the monitor
                     getFloatingArea().dimensions() / 2
                     // minus half the dimensions of the client
-                    - client->float_size_.dimensions() / 2;
-                client->float_size_.x = new_tl.x;
-                client->float_size_.y = new_tl.y;
+                    - client->float_size_->dimensions() / 2;
+                client->float_size_ = Rectangle(new_tl.x, new_tl.y,
+                                                client->float_size_->width,
+                                                client->float_size_->height);
             }
             break;
 
@@ -772,8 +595,9 @@ void Monitor::evaluateClientPlacement(Client* client, ClientPlacement placement)
                 Point2D area = getFloatingArea().dimensions();
                 Point2D new_tl = Floating::smartPlacement(tag, client,
                                              area, settings->snap_gap);
-                client->float_size_.x = new_tl.x;
-                client->float_size_.y = new_tl.y;
+                client->float_size_ = Rectangle(new_tl.x, new_tl.y,
+                                                client->float_size_->width,
+                                                client->float_size_->height);
             }
             break;
 
