@@ -20,8 +20,8 @@ public:
     int relativeX(int x_root);
     int relativeY(int y_root);
 
-    HSTag*      tag;    // currently viewed tag
-    HSTag*      tag_previous;    // previously viewed tag
+    HSTag*      tag;    // currently viewed tag, this is always non-null
+    HSTag*      tag_previous;    // previously viewed tag, this is always non-null
     Attribute_<std::string>   name;
     Attribute_<unsigned long> index;
     DynAttribute_<std::string>   tag_string;
@@ -57,8 +57,8 @@ public:
     void restack();
     std::string getDescription();
     void evaluateClientPlacement(Client* client, ClientPlacement placement) const;
+    static std::string atLeastMinWindowSize(Rectangle geom);
 private:
-    std::string atLeastMinWindowSize(Rectangle geom);
     std::string getTagString();
     std::string setTagString(std::string new_tag);
     Settings* settings;
@@ -68,26 +68,15 @@ private:
 // adds a new monitor to the monitors list and returns a pointer to it
 Monitor* find_monitor_with_tag(HSTag* tag);
 void monitor_focus_by_index(unsigned new_selection);
-int monitor_cycle_command(int argc, char** argv);
-int monitor_focus_command(int argc, char** argv, Output output);
 Monitor* find_monitor_by_name(const char* name);
 Monitor* string_to_monitor(const char* string);
-int remove_monitor_command(int argc, char** argv, Output output);
-int remove_monitor(int index);
-int monitor_rect_command(int argc, char** argv, Output output);
 Monitor* get_current_monitor();
 int monitor_set_tag(Monitor* monitor, HSTag* tag);
-int monitor_set_pad_command(int argc, char** argv, Output output);
-int monitor_set_tag_command(int argc, char** argv, Output output);
-int monitor_set_tag_by_index_command(int argc, char** argv, Output output);
-int monitor_set_previous_tag_command(Output output);
 void all_monitors_apply_layout();
 void ensure_monitors_are_available();
-void all_monitors_replace_previous_tag(HSTag* old, HSTag* newmon);
 
 void monitor_update_focus_objects();
 
-int shift_to_monitor(int argc, char** argv, Output output);
 
 #endif
 
