@@ -4,8 +4,10 @@
 #include <cassert>
 #include <memory>
 
+#include "argparse.h"
 #include "command.h"
 #include "completion.h"
+#include "desktopwindow.h"
 #include "ewmh.h"
 #include "floating.h"
 #include "frametree.h"
@@ -33,6 +35,9 @@ using std::string;
 using std::to_string;
 using std::vector;
 
+template<>
+RunTimeConverter<Monitor*>* Converter<Monitor*>::converter = nullptr;
+
 MonitorManager* g_monitors;
 
 MonitorManager::MonitorManager()
@@ -45,9 +50,9 @@ MonitorManager::MonitorManager()
 {
     cur_monitor = 0;
     setDoc("Every monitor is a rectangular part of the screen "
-           "on which a tag is shown. These monitors may or may"
+           "on which a tag is shown. These monitors may or may "
            "not match the actual outputs.\n"
-           "This has an entry \'INDEX\' for each monitor with"
+           "This has an entry \'INDEX\' for each monitor with "
            "index \'INDEX\'.");
     focus.setDoc("the focused monitor.");
     // TODO: add this as soon as by_name_ is of type Child_<ByName>
@@ -182,6 +187,30 @@ Monitor* MonitorManager::byString(string str) {
     return ((idx >= 0) && idx < static_cast<int>(size())) ? byIdx(idx) : nullptr;
 }
 
+Monitor* MonitorManager::parse(const string& str)
+{
+    int idx = string_to_monitor_index(str);
+    if ((idx >= 0) && idx < static_cast<int>(size())) {
+        return byIdx(static_cast<size_t>(idx));
+    } else {
+        throw std::invalid_argument(string("No such monitor: ") + str);
+    }
+}
+
+string MonitorManager::str(Monitor* monitor)
+{
+    if (monitor) {
+        return monitor->index.str();
+    } else {
+        return {};
+    }
+}
+
+void MonitorManager::completeEntries(Completion& completion)
+{
+    completeMonitorName(completion);
+}
+
 function<int(Input, Output)> MonitorManager::byFirstArg(MonitorCommand cmd)
 {
     return [this,cmd](Input input, Output output) -> int {
@@ -192,8 +221,8 @@ function<int(Input, Output)> MonitorManager::byFirstArg(MonitorCommand cmd)
         } else {
             monitor = byString(monitor_name);
             if (!monitor) {
-                output << input.command() <<
-                    ": Monitor \"" << monitor_name << "\" not found!\n";
+                output.perror() <<
+                    "Monitor \"" << monitor_name << "\" not found!\n";
                 return HERBST_INVALID_ARGUMENT;
             }
         }
@@ -236,6 +265,13 @@ CommandBinding MonitorManager::tagCommand(function<int (HSTag&)> cmd)
     });
 }
 
+CommandBinding MonitorManager::tagCommand(TagCallOrComplete cmd)
+{
+    return CommandBinding([this,cmd](CallOrComplete invoc) {
+        cmd(*(this->focus()->tag), invoc);
+    });
+}
+
 
 Monitor* MonitorManager::byTag(HSTag* tag) {
     for (Monitor* m : *this) {
@@ -289,27 +325,19 @@ void MonitorManager::relayoutAll()
     }
 }
 
-int MonitorManager::removeMonitor(Input input, Output output)
+void MonitorManager::removeMonitorCommand(CallOrComplete invoc)
 {
-    string monitorIdxString;
-    if (!(input >> monitorIdxString)) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    auto monitor = byString(monitorIdxString);
-
-    if (monitor == nullptr) {
-        output << input.command() << ": Monitor \"" << monitorIdxString << "\" not found!\n";
-        return HERBST_INVALID_ARGUMENT;
-    }
-
-    if (size() <= 1) {
-        output << input.command() << ": Can't remove the last monitor\n";
-        return HERBST_FORBIDDEN;
-    }
-
-    removeMonitor(monitor);
-
-    return HERBST_EXIT_SUCCESS;
+    Monitor* monitor = nullptr;
+    ArgParse().mandatory(monitor).command(invoc,
+        [&] (Output output) {
+            if (size() <= 1) {
+                output.perror() << "Can't remove the last monitor\n";
+                return HERBST_FORBIDDEN;
+            }
+            this->removeMonitor(monitor);
+            return HERBST_EXIT_SUCCESS;
+        }
+    );
 }
 
 void MonitorManager::removeMonitor(Monitor* monitor)
@@ -342,59 +370,52 @@ void MonitorManager::removeMonitor(Monitor* monitor)
     monitor_update_focus_objects();
 }
 
-int MonitorManager::addMonitor(Input input, Output output)
+void MonitorManager::addMonitorCommand(CallOrComplete invoc)
 {
-    // usage: add_monitor RECTANGLE [TAG [NAME]]
-    string rectString, tagName, monitorName;
-    input >> rectString;
-    if (!input) {
-        return HERBST_NEED_MORE_ARGS;
-    }
+    Rectangle geo;
     HSTag* tag = nullptr;
-    if (input >> tagName) {
-        tag = find_tag(tagName.c_str());
+    bool nameSupplied = false;
+    string monitorName;
+    ArgParse().mandatory(geo)
+            .optional(tag)
+            .optional(monitorName, &nameSupplied)
+            .command(invoc, [&] (Output output)
+    {
         if (!tag) {
-            output << input.command() << ": Tag \"" << tagName << "\" does not exist\n";
-            return HERBST_INVALID_ARGUMENT;
+            tag = tags_->unusedTag();
+        }
+        if (!tag) {
+            output.perror() << "There are not enough free tags\n";
+            return HERBST_TAG_IN_USE;
         }
         if (find_monitor_with_tag(tag)) {
-            output << input.command() <<
-                ": Tag \"" << tagName << "\" is already being viewed on a monitor\n";
+            output.perror() <<
+                "Tag \"" << tag->name() << "\" is already being viewed on a monitor\n";
             return HERBST_TAG_IN_USE;
         }
-    } else { // if no tag is supplied
-        tag = tags_->unusedTag();
-        if (!tag) {
-            output << input.command() << ": There are not enough free tags\n";
-            return HERBST_TAG_IN_USE;
-        }
-    }
-    // TODO: error message on invalid rectString
-    auto rect = Rectangle::fromStr(rectString);
-    if (input >> monitorName) {
-        string error;
-        if (monitorName.empty()) {
-            error = "An empty monitor name is not permitted";
-        } else {
-            error = isValidMonitorName(monitorName);
+        string error = Monitor::atLeastMinWindowSize(geo);
+        if (error.empty() && nameSupplied) {
+            if (monitorName.empty()) {
+                error = "An empty monitor name is not permitted";
+            } else {
+                error = isValidMonitorName(monitorName);
+            }
         }
         if (!error.empty()) {
-            output << input.command() << ": " << error << "\n";
+            output.perror() << error << "\n";
             return HERBST_INVALID_ARGUMENT;
         }
-    }
-    auto monitor = addMonitor(rect, tag);
-    if (!monitorName.empty()) {
-        monitor->name = monitorName;
-    }
-
-    autoUpdatePads();
-    monitor->applyLayout();
-    tag->setVisible(true);
-    emit_tag_changed(tag, g_monitors->size() - 1);
-    dropEnterNotifyEvents.emit();
-
-    return HERBST_EXIT_SUCCESS;
+        auto monitor = addMonitor(geo, tag);
+        if (!monitorName.empty()) {
+            monitor->name = monitorName;
+        }
+        autoUpdatePads();
+        monitor->applyLayout();
+        tag->setVisible(true);
+        emit_tag_changed(tag, static_cast<int>(monitor->index()));
+        dropEnterNotifyEvents.emit();
+        return HERBST_EXIT_SUCCESS;
+    });
 }
 
 string MonitorManager::isValidMonitorName(string name) {
@@ -457,18 +478,15 @@ Monitor* MonitorManager::addMonitor(Rectangle rect, HSTag* tag) {
 
 void MonitorManager::lock() {
     settings_->monitors_locked = settings_->monitors_locked() + 1;
-    lock_number_changed();
 }
 
 void MonitorManager::unlock() {
-    settings_->monitors_locked = std::max(0, settings_->monitors_locked() - 1);
-    lock_number_changed();
+    if (settings_->monitors_locked > 0) {
+        settings_->monitors_locked = settings_->monitors_locked() - 1;
+    }
 }
 
-string MonitorManager::lock_number_changed() {
-    if (settings_->monitors_locked() < 0) {
-        return "must be non-negative";
-    }
+void MonitorManager::lock_number_changed() {
     if (!settings_->monitors_locked()) {
         // if not locked anymore, then repaint all the dirty monitors
         for (auto m : *this) {
@@ -477,7 +495,6 @@ string MonitorManager::lock_number_changed() {
             }
         }
     }
-    return {};
 }
 
 //! return the stack of windows by successive calls to the given yield
@@ -497,6 +514,9 @@ void MonitorManager::extractWindowStack(bool real_clients, function<void(Window)
 void MonitorManager::restack() {
     vector<Window> buf;
     extractWindowStack(false, [&buf](Window w) { buf.push_back(w); });
+    DesktopWindow::foreachDesktopWindow([&buf](DesktopWindow& dw) {
+        buf.push_back(dw.window());
+    });
     XRestackWindows(g_display, buf.data(), buf.size());
     Ewmh::get().updateClientListStacking();
 }
@@ -545,6 +565,15 @@ int MonitorManager::stackCommand(Output output) {
 
         monitors.push_back(make_shared<StringTree>(monitor->getDescription(), layers));
     }
+    vector<shared_ptr<StringTree>> desktopWins;
+    XConnection& X_ = XConnection::get();
+    DesktopWindow::foreachDesktopWindow([&desktopWins,&X_](DesktopWindow& dw) {
+        string txt = Converter<WindowID>::str(dw.window());
+        auto info = X_.getClassHint(dw.window());
+        txt += " " + info.first + " " + info.second;
+        desktopWins.push_back(make_shared<StringTree>(txt));
+    });
+    monitors.push_back(make_shared<StringTree>("Desktop Windows", desktopWins));
 
     auto stackRoot = make_shared<StringTree>("", monitors);
     tree_print_to(stackRoot, output);
@@ -593,8 +622,8 @@ int MonitorManager::setMonitorsCommand(Input input, Output output) {
         Rectangle rect = Rectangle::fromStr(rectangleString);
         if (rect.width == 0 || rect.height == 0)
         {
-            output << input.command()
-                   << ": Rectangle invalid or too small: "
+            output.perror()
+                   << "Rectangle invalid or too small: "
                    << rectangleString << endl;
             return HERBST_INVALID_ARGUMENT;
         }
@@ -606,7 +635,7 @@ int MonitorManager::setMonitorsCommand(Input input, Output output) {
     int status = setMonitors(templates);
 
     if (status == HERBST_TAG_IN_USE) {
-        output << input.command() << ": There are not enough free tags\n";
+        output.perror() << "There are not enough free tags\n";
     } else if (status == HERBST_INVALID_ARGUMENT) {
         return HERBST_NEED_MORE_ARGS;
     }
@@ -623,6 +652,60 @@ void MonitorManager::detectMonitorsCompletion(Completion& complete)
     complete.full({"-l", "--list", "--list-all", "--no-disjoin"});
 }
 
+void MonitorManager::focusCommand(CallOrComplete invoc)
+{
+    Monitor* monitor = nullptr;
+    ArgParse().mandatory(monitor)
+              .command(invoc, [&](Output)
+    {
+        monitor_focus_by_index(monitor->index);
+        return 0;
+    });
+}
+
+void MonitorManager::cycleCommand(CallOrComplete invoc)
+{
+    int delta = 1;
+    ArgParse().optional(delta, {"-1", "+1"})
+              .command(invoc, [&](Output)
+    {
+        int new_selection = cur_monitor + delta; // signed for delta calculations
+        monitor_focus_by_index((unsigned)MOD(new_selection, size()));
+        return 0;
+    });
+}
+
+void MonitorManager::rectCommand(CallOrComplete invoc)
+{
+    Monitor* m = focus();
+    bool subtractPad = false;
+    ArgParse().optional(m)
+              .flags({{"-p", &subtractPad}})
+              .command(invoc, [&](Output output)
+    {
+        auto rect = m->rect();
+        if (subtractPad) {
+            rect.x += m->pad_left;
+            rect.width -= m->pad_left + m->pad_right;
+            rect.y += m->pad_up;
+            rect.height -= m->pad_up + m->pad_down;
+        }
+        output << rect.x << " " << rect.y << " " << rect.width << " " << rect.height;
+        return 0;
+    });
+}
+
+void MonitorManager::shiftToMonitorCommand(CallOrComplete invoc)
+{
+    Monitor* monitor = nullptr;
+    ArgParse().mandatory(monitor)
+              .command(invoc, [&](Output)
+    {
+        tags_->moveFocusedClient(monitor->tag);
+        return 0;
+    });
+}
+
 int MonitorManager::detectMonitorsCommand(Input input, Output output)
 {
     bool list_all = false;
@@ -637,7 +720,7 @@ int MonitorManager::detectMonitorsCommand(Input input, Output output)
         } else if (arg == "--no-disjoin") {
             disjoin = false;
         } else {
-            output << input.command() << ": unknown flag \"" << arg << "\"\n";
+            output.perror() << "unknown flag \"" << arg << "\"\n";
             return HERBST_INVALID_ARGUMENT;
         }
     }
@@ -686,13 +769,77 @@ int MonitorManager::detectMonitorsCommand(Input input, Output output)
         // apply it
         int ret = g_monitors->setMonitors(monitor_rects);
         if (ret == HERBST_TAG_IN_USE) {
-            output << input.command() << ": There are not enough free tags\n";
+            output.perror() << "There are not enough free tags" << endl;
         }
         return ret;
     }
     return 0;
 }
 
+
+/**
+ * @brief The type 'int' extended by an 'undefined' value represented by
+ * the empty string.
+ */
+class EmptyOrInt {
+public:
+    int value_;
+    bool defined_;
+    static EmptyOrInt undefined() {
+        return { 0, false };
+    }
+};
+
+template<>
+EmptyOrInt Converter<EmptyOrInt>::parse(const string& source) {
+    if (source.empty()) {
+        return EmptyOrInt::undefined();
+    } else {
+        return EmptyOrInt { Converter<int>::parse(source), true };
+    }
+}
+
+template<>
+string Converter<EmptyOrInt>::str(EmptyOrInt payload) {
+    return payload.defined_ ? Converter<int>::str(payload.value_) : "";
+}
+
+template<>
+void Converter<EmptyOrInt>::complete(Completion& complete, EmptyOrInt const* relTo) {
+    complete.full("");
+    Converter<int>::complete(complete, (relTo && relTo->defined_) ? &relTo->value_ : nullptr);
+}
+
+void MonitorManager::padCommand(CallOrComplete invoc)
+{
+    Monitor* monitor = focus();
+    vector<pair<Attribute_<int> Monitor::*, EmptyOrInt>> padAttributes = {
+        // the pad attributes in the parameter order
+        // of the 'pad' command:
+        { &Monitor::pad_up,     EmptyOrInt::undefined() },
+        { &Monitor::pad_right,  EmptyOrInt::undefined() },
+        { &Monitor::pad_down,   EmptyOrInt::undefined() },
+        { &Monitor::pad_left,   EmptyOrInt::undefined() },
+    };
+    ArgParse().mandatory(monitor)
+              .optional(padAttributes[0].second, {"0"})
+              .optional(padAttributes[1].second, {"0"})
+              .optional(padAttributes[2].second, {"0"})
+              .optional(padAttributes[3].second, {"0"})
+              .command(invoc, [&](Output) {
+        for (const auto& pad : padAttributes) {
+            // update those pad attributes that were given
+            // in the command line arguments
+            if (pad.second.defined_) {
+                Attribute_<int> Monitor::* pad_attribute = pad.first;
+                monitor->*pad_attribute = pad.second.value_;
+            }
+            monitor->applyLayout();
+        }
+        return 0;
+    });
+}
+
 /**
  * @brief Transform a rectangle on the screen into a rectangle relative to one of the monitor.
  * Currently, we pick the monitor that has the biggest intersection with the given rectangle.
@@ -722,12 +869,21 @@ Rectangle MonitorManager::interpretGlobalGeometry(Rectangle globalGeometry)
     return globalGeometry;
 }
 
+void MonitorManager::replacePreviousTag(HSTag* tagToRemove, HSTag* targetTag)
+{
+    for (Monitor* m : *this) {
+        if (m->tag_previous == tagToRemove) {
+            m->tag_previous = targetTag;
+        }
+    }
+}
+
 int MonitorManager::raiseMonitorCommand(Input input, Output output) {
     string monitorName = "";
     input >> monitorName;
     Monitor* monitor = string_to_monitor(monitorName.c_str());
     if (!monitor) {
-        output << input.command() << ": Monitor \"" << monitorName << "\" not found!\n";
+        output.perror() << "Monitor \"" << monitorName << "\" not found!\n";
         return HERBST_INVALID_ARGUMENT;
     }
     monitorStack_.raise(monitor);
