@@ -2,6 +2,7 @@
 
 #include <X11/Xlib.h>
 #include <X11/cursorfont.h>
+#include <iostream>
 #include <ostream>
 #include <string>
 #include <vector>
@@ -62,7 +63,7 @@ int MouseManager::addMouseBindCommand(Input input, Output output) {
     }
     auto action = string2mousefunction(mouseFunctionName);
     if (!action) {
-        output << input.command() << ": Unknown mouse action \"" << mouseFunctionName << "\"" << endl;
+        output.perror() << "Unknown mouse action \"" << mouseFunctionName << "\"" << endl;
         return HERBST_INVALID_ARGUMENT;
     }
 
@@ -129,21 +130,21 @@ int MouseManager::dragCommand(Input input, Output output)
     }
     Client* client = Root::get()->clients->client(winid);
     if (!client) {
-        output << "Could not find client \"" << winid << "\"\n";
+        output.perror() << "Could not find client \"" << winid << "\"\n";
         return HERBST_INVALID_ARGUMENT;
     }
     MouseFunction action = string2mousefunction(mouseAction.c_str());
     if (!action) {
-        output << input.command() << ": Unknown mouse action \"" << mouseAction << "\"" << endl;
+        output.perror() << "Unknown mouse action \"" << mouseAction << "\"" << endl;
         return HERBST_INVALID_ARGUMENT;
     }
     if (monitors_->byTag(client->tag()) == nullptr || client->minimized_()) {
-        output << input.command() << ": cannot drag invisible client \"" << winid << "\"" << endl;
+        output.perror() << "cannot drag invisible client \"" << winid << "\"" << endl;
         return HERBST_INVALID_ARGUMENT;
     }
     string errorMsg = (this ->* action)(client, input.toVector());
     if (!errorMsg.empty()) {
-        output << input.command() << ": cannot drag: " << errorMsg << "\n";
+        output.perror() << "cannot drag: " << errorMsg << "\n";
         return HERBST_UNKNOWN_ERROR;
     }
     return 0;
@@ -152,7 +153,7 @@ int MouseManager::dragCommand(Input input, Output output)
 void MouseManager::dragCompletion(Completion& complete)
 {
     if (complete == 0) {
-        Root::get()->clients->completeClients(complete);
+        Root::get()->clients->completeEntries(complete);
     } else if (complete == 1) {
         for (auto& it : mouseFunctions_) {
             complete.full(it.first);
@@ -205,7 +206,9 @@ string MouseManager::mouse_call_command(Client* client, const vector<string> &cm
     // Execute the bound command
     std::ostringstream discardedOutput;
     Input input(cmd.front(), {cmd.begin() + 1, cmd.end()});
-    Commands::call(input, discardedOutput);
+    // discard output, but forward errors to std::cerr
+    OutputChannels channels(cmd.front(), discardedOutput, std::cerr);
+    Commands::call(input, channels);
 
     clients_->setDragged(nullptr);
     return {};
@@ -217,9 +220,6 @@ string MouseManager::mouse_initiate_drag(Client *client, const MouseDragHandler:
         dragHandler_ = createHandler(monitors_, client);
         // only grab pointer if dragHandler_ could be started
         clients_->setDragged(client);
-        XGrabPointer(g_display, client->x11Window(), True,
-            PointerMotionMask|ButtonReleaseMask, GrabModeAsync,
-                GrabModeAsync, None, None, CurrentTime);
     }  catch (const MouseDragHandler::DragNotPossible& e) {
         // clear all fields, just to be sure
         dragHandler_ = {};
@@ -241,14 +241,6 @@ void MouseManager::mouse_stop_drag() {
         }
         dragHandler_ = {};
     }
-    XUngrabPointer(g_display, CurrentTime);
-    // remove all enternotify-events from the event queue that were
-    // generated by the XUngrabPointer
-    XEvent ev;
-    XSync(g_display, False);
-    while (XCheckMaskEvent(g_display, EnterWindowMask, &ev)) {
-        ;
-    }
 }
 
 void MouseManager::handle_motion_event(Point2D newCursorPos) {
