@@ -126,7 +126,8 @@ Object* Object::child(const string &name) {
 
 Object* Object::child(Path path) {
     std::ostringstream out;
-    return child(path, out);
+    OutputChannels channels("", out, out);
+    return child(path, channels);
 }
 
 Object* Object::child(Path path, Output output) {
@@ -135,7 +136,7 @@ Object* Object::child(Path path, Output output) {
     while (!path.empty()) {
         cur = cur->child(path.front());
         if (!cur) {
-            output << "Object \"" << cur_path << "\""
+            output.perror() << "Object \"" << cur_path << "\""
                 << " has no child named \"" << path.front()
                 << "\"" << endl;
             return nullptr;
@@ -262,7 +263,8 @@ void Object::printTree(Output output, string rootLabel) {
 
 Attribute* Object::deepAttribute(const string &path) {
     std::ostringstream output;
-    return deepAttribute(path, output);
+    OutputChannels channels("", output, output);
+    return deepAttribute(path, channels);
 }
 
 Attribute* Object::deepAttribute(const string &path, Output output) {
@@ -273,7 +275,8 @@ Attribute* Object::deepAttribute(const string &path, Output output) {
     }
     Attribute* a = attribute_owner->attribute(attr_path.second);
     if (!a) {
-        output << "Object \"" << attr_path.first.join()
+        output.perror()
+            << "Object \"" << attr_path.first.join()
             << "\" has no attribute \"" << attr_path.second
             << "\"."
             << endl;
