@@ -5,6 +5,7 @@
 #include "client.h"
 #include "clientmanager.h"
 #include "ewmh.h"
+#include "globalcommands.h"
 #include "hlwmcommon.h"
 #include "keymanager.h"
 #include "layout.h"
@@ -18,6 +19,7 @@
 #include "tagmanager.h"
 #include "theme.h"
 #include "tmp.h"
+#include "typesdoc.h"
 #include "utils.h"
 #include "watchers.h"
 
@@ -25,44 +27,48 @@ using std::shared_ptr;
 
 shared_ptr<Root> Root::root_;
 
-Root::Root(Globals g, XConnection& xconnection, IpcServer& ipcServer)
+Root::Root(Globals g, XConnection& xconnection, Ewmh& ewmh, IpcServer& ipcServer)
     : clients(*this, "clients")
     , keys(*this, "keys")
     , monitors(*this, "monitors")
     , mouse(*this, "mouse")
+    , panels(*this, "panels")
     , rules(*this, "rules")
     , settings(*this, "settings")
     , tags(*this, "tags")
     , theme(*this, "theme")
     , tmp(*this, TMP_OBJECT_PATH)
+    , types(*this, "types")
     , watchers(*this, "watchers")
     , globals(g)
     , meta_commands(make_unique<MetaCommands>(*this))
+    , global_commands(make_unique<GlobalCommands>(*this))
     , X(xconnection)
     , ipcServer_(ipcServer)
-    , panels(make_unique<PanelManager>(xconnection))
-    , ewmh(make_unique<Ewmh>(xconnection))
+    , ewmh_(ewmh)
 {
     // initialize root children (alphabetically)
     clients.init();
     keys.init();
     monitors.init();
     mouse.init();
+    panels.init(xconnection);
     rules.init();
     settings.init();
     tags.init();
     theme.init();
     tmp.init();
+    types.init();
     watchers.init();
 
     // inject dependencies where needed
-    ewmh->injectDependencies(this);
+    ewmh_.injectDependencies(this);
     settings->injectDependencies(this);
     tags->injectDependencies(monitors(), settings());
-    clients->injectDependencies(settings(), theme(), ewmh.get());
-    monitors->injectDependencies(settings(), tags(), panels.get());
-    mouse->injectDependencies(clients(), monitors());
+    clients->injectDependencies(settings(), theme(), &ewmh_);
     panels->injectDependencies(settings());
+    monitors->injectDependencies(settings(), tags(), panels());
+    mouse->injectDependencies(clients(), monitors());
     watchers->injectDependencies(this);
 
     // set temporary globals
@@ -79,7 +85,10 @@ Root::Root(Globals g, XConnection& xconnection, IpcServer& ipcServer)
     panels->panels_changed_.connect(monitors(), &MonitorManager::autoUpdatePads);
 }
 
-Root::~Root()
+Root::~Root() {
+}
+
+void Root::shutdown()
 {
     // Note: delete in reverse order of initialization!
     mouse.reset();
@@ -87,6 +96,8 @@ Root::~Root()
     // MonitorManager needs the other for shutting down, so we do that first:
     monitors.reset();
 
+    panels.reset(); // needs to be reset after monitors
+
     // Shut down children with dependencies first:
     clients.reset();
     tags.reset();
