@@ -118,13 +118,6 @@ Settings::Settings()
     }
     wmname.changed().connect([]() { Ewmh::get().updateWmName(); });
 
-    default_frame_layout.setValidator([] (size_t layout) {
-        if (layout >= layoutAlgorithmCount()) {
-            return "layout number must be at most "
-                + to_string(layoutAlgorithmCount() - 1);
-        }
-        return string();
-    });
     tree_style.setValidator([] (string new_value) {
         if (utf8_string_length(new_value) < 8) {
             return string("tree_style needs 8 characters");
@@ -213,14 +206,13 @@ int Settings::set_cmd(Input input, Output output) {
 
     auto attr = attribute(set_name);
     if (!attr) {
-        output << input.command() <<
-            ": Setting \"" << set_name << "\" not found\n";
+        output.perror() << "Setting \"" << set_name << "\" not found\n";
         return HERBST_SETTING_NOT_FOUND;
     }
     auto msg = attr->change(value);
     if (!msg.empty()) {
-        output << input.command()
-               << ": Invalid value \"" << value
+        output.perror()
+               << "Invalid value \"" << value
                << "\" for setting \"" << set_name << "\": "
                << msg << endl;
         return HERBST_INVALID_ARGUMENT;
@@ -250,15 +242,14 @@ int Settings::toggle_cmd(Input argv, Output output) {
     auto set_name = argv.front();
     auto attr = attribute(set_name);
     if (!attr) {
-        output << argv.command() <<
-            ": Setting \"" << set_name << "\" not found\n";
+        output.perror() << "Setting \"" << set_name << "\" not found\n";
         return HERBST_SETTING_NOT_FOUND;
     }
     if (attr->type() == Type::BOOL) {
         attr->change("toggle");
     } else {
-        output << argv.command()
-            << ": Setting \"" << set_name
+        output.perror()
+            << "Setting \"" << set_name
             << "\" is not of type bool\n";
         return HERBST_INVALID_ARGUMENT;
     }
@@ -277,53 +268,13 @@ void Settings::toggle_complete(Completion& complete) {
     }
 }
 
-int Settings::cycle_value_cmd(Input argv, Output output) {
-    if (argv.empty()) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    auto set_name = argv.front();
-    argv.shift();
-    if (argv.empty()) {
-        return HERBST_NEED_MORE_ARGS;
-    }
-    auto attr = attribute(set_name);
-    if (!attr) {
-        output << argv.command() <<
-            ": Setting \"" << set_name << "\" not found\n";
-        return HERBST_SETTING_NOT_FOUND;
-    }
-    auto msg = attr->cycleValue(argv.begin(), argv.end());
-    if (!msg.empty()) {
-        output << argv.command()
-               << ": Invalid value for setting \""
-               << set_name << "\": "
-               << msg << endl;
-        return HERBST_INVALID_ARGUMENT;
-    }
-    return 0;
-}
-
-void Settings::cycle_value_complete(Completion& complete) {
-    if (complete == 0) {
-        for (auto a : attributes()) {
-            complete.full(a.first);
-        }
-    } else {
-        Attribute* a = attribute(complete[0]);
-        if (a) {
-            a->complete(complete);
-        }
-    }
-}
-
 int Settings::get_cmd(Input argv, Output output) {
     if (argv.empty()) {
         return HERBST_NEED_MORE_ARGS;
     }
     auto attr = attribute(argv.front());
     if (!attr) {
-        output << argv.command() <<
-            ": Setting \"" << argv.front() << "\" not found\n";
+        output.perror() << "Setting \"" << argv.front() << "\" not found\n";
         return HERBST_SETTING_NOT_FOUND;
     }
     output << attr->str();
