@@ -5,6 +5,7 @@
 
 #include "argparse.h"
 #include "client.h"
+#include "clientmanager.h"
 #include "completion.h"
 #include "ewmh.h"
 #include "floating.h"
@@ -18,6 +19,7 @@
 #include "settings.h"
 #include "stack.h"
 #include "tagmanager.h"
+#include "utils.h"
 
 using std::endl;
 using std::function;
@@ -225,6 +227,38 @@ void HSTag::focusFrame(shared_ptr<FrameLeaf> frameToFocus)
     needsRelayout_.emit();
 }
 
+/**
+ * @brief Find a minimized client on this tag
+ * @param if oldest is true, return the client that's minimized
+ * for the longest time, otherwise, return the most recently minimized client
+ * @return A pointer to the minimized client and nullptr
+ * if there is no minimized client on this tag.
+ */
+Client* HSTag::minimizedClient(bool oldest)
+{
+    Client* chosen = nullptr;
+    for (Client* c : floating_clients_) {
+        if (!c->minimized_()) {
+            continue;
+        }
+        // always pick c if chosen is still null
+        bool cIsBetter = chosen == nullptr;
+        if (oldest) {
+            // looking for the longest minimized client
+            // by the lazy `||` we are safe if chosen == nullptr
+            cIsBetter = cIsBetter || c->minimizedLastChange_ < chosen->minimizedLastChange_;
+        } else {
+            // looking for the most recently minimized client
+            // by the lazy `||` we are safe if chosen == nullptr
+            cIsBetter = cIsBetter || c->minimizedLastChange_ > chosen->minimizedLastChange_;
+        }
+        if (cIsBetter) {
+            chosen = c;
+        }
+    }
+    return chosen;
+}
+
 Client *HSTag::focusedClient()
 {
     if (floating_focused()) {
@@ -277,7 +311,7 @@ void HSTag::removeClientSlice(Client* client)
 }
 
 //! directional focus command
-int HSTag::focusInDirCommand(Input input, Output output)
+void HSTag::focusInDirCommand(CallOrComplete invoc)
 {
     bool external_only = settings_->default_direction_external_only();
     Direction direction = Direction::Left; // some default to satisfy the linter
@@ -287,10 +321,14 @@ int HSTag::focusInDirCommand(Input input, Output output)
         {"-e", [&external_only] () { external_only = true; }},
     });
     ap.mandatory(direction);
-    if (ap.parsingAllFails(input, output)) {
-        return ap.exitCode();
-    }
+    ap.command(invoc,
+               [&] (Output output) {
+                    return focusInDir(direction, external_only, output);
+               });
+}
 
+int HSTag::focusInDir(Direction direction, bool external_only, Output output)
+{
     auto focusedFrame = frame->focusedFrame();
     bool neighbour_found = true;
     if (floating || floating_focused) {
@@ -310,27 +348,13 @@ int HSTag::focusInDirCommand(Input input, Output output)
         }
     }
     if (!neighbour_found) {
-        output << input.command() << ": No neighbour found\n";
+        output.perror() << "No neighbour found\n";
         return HERBST_FORBIDDEN;
     }
     return 0;
 }
 
-void HSTag::focusInDirCompletion(Completion &complete)
-{
-    if (complete == 0) {
-        complete.full({"-i", "-e"});
-        Converter<Direction>::complete(complete, nullptr);
-    } else if (complete == 1
-               && (complete[0] == "-i" || complete[0] == "-e"))
-    {
-        Converter<Direction>::complete(complete, nullptr);
-    } else {
-        complete.none();
-    }
-}
-
-int HSTag::shiftInDirCommand(Input input, Output output)
+void HSTag::shiftInDirCommand(CallOrComplete invoc)
 {
     bool external_only = settings_->default_direction_external_only();
     Direction direction = Direction::Left; // some default to satisfy the linter
@@ -340,69 +364,71 @@ int HSTag::shiftInDirCommand(Input input, Output output)
         {"-e", [&external_only] () { external_only = true; }},
     });
     ap.mandatory(direction);
-    if (ap.parsingAllFails(input, output)) {
-        return ap.exitCode();
-    }
-    shared_ptr<FrameLeaf> sourceFrame = this->frame->focusedFrame();
+    ap.command(invoc,
+               [&] (Output output) {
+                    return shiftInDir(direction, external_only, output);
+               });
+}
+
+int HSTag::shiftInDir(Direction direction, bool external_only, Output output)
+{
     Client* currentClient = focusedClient();
-    if (currentClient && currentClient->is_client_floated()) {
-        // try to move the floating window
-        bool success = Floating::shiftDirection(direction);
-        return success ? 0 : HERBST_FORBIDDEN;
+    if (!currentClient) {
+        output.perror() << "No client focused\n";
+        return HERBST_FORBIDDEN;
     }
-    // don't look for neighbours within the frame if 'external_only' is set
-    int indexInFrame = external_only ? (-1) : sourceFrame->getInnerNeighbourIndex(direction);
-    if (indexInFrame >= 0) {
-        sourceFrame->moveClient(indexInFrame);
-        needsRelayout_.emit();
+    bool success = true;
+    if (currentClient->is_client_floated()) {
+        // try to move the floating window
+        success = Floating::shiftDirection(direction);
     } else {
-        shared_ptr<Frame> neighbour = sourceFrame->neighbour(direction);
-        Client* client = sourceFrame->focusedClient();
-        if (client && neighbour) { // if neighbour was found
-            // move window to neighbour
-            sourceFrame->removeClient(client);
-            FrameTree::focusedFrame(neighbour)->insertClient(client);
-            neighbour->frameWithClient(client)->select(client);
-
-            // change selection in parent
-            shared_ptr<FrameSplit> parent = neighbour->getParent();
-            assert(parent);
-            parent->swapSelection();
-
-            // layout was changed, so update it
-            get_current_monitor()->applyLayout();
-        } else if (!client) {
-            output << input.command() << ": No client focused\n";
-            return HERBST_FORBIDDEN;
-        } else {
-            output << input.command() << ": No neighbour found\n";
-            return HERBST_FORBIDDEN;
+        success = frame->shiftInDirection(direction, external_only);
+        if (success) {
+            needsRelayout_.emit();
         }
     }
-    return 0;
+    if (!success && settings_->focus_crosses_monitor_boundaries()) {
+        // find monitor in the specified direction
+        MonitorManager* monitors = g_monitors;
+        int idx = monitors->indexInDirection(monitors->focus(), direction);
+        if (idx >= 0) {
+            Monitor* targetMonitor = monitors->byIdx(idx);
+            tags_->moveClient(currentClient, targetMonitor->tag);
+            monitor_focus_by_index(idx);
+            success = true;
+        }
+    }
+    if (success) {
+        return 0;
+    } else {
+        output.perror() << "No neighbour found\n";
+        return HERBST_FORBIDDEN;
+    }
 }
 
-void HSTag::shiftInDirCompletion(Completion& complete)
-{
-    focusInDirCompletion(complete);
-}
-
-int HSTag::cycleAllCommand(Input input, Output output)
+void HSTag::cycleAllCommand(CallOrComplete invoc)
 {
     bool skip_invisible = false;
     int delta = 1;
-    ArgParse ap;
-    ap.flags({{"--skip-invisible", &skip_invisible}}).optional(delta);
-    if (ap.parsingAllFails(input, output)) {
-        return HERBST_INVALID_ARGUMENT;
-    }
-    if (delta < -1 || delta > 1) {
-        output << "argument out of range." << endl;
-        return HERBST_INVALID_ARGUMENT;
-    }
-    if (delta == 0) {
-        return 0; // nothing to do
-    }
+    ArgParse().flags({{"--skip-invisible", &skip_invisible}})
+              .optional(delta, {"+1", "-1"})
+              .command(invoc,
+                       [&] (Output output) {
+        if (delta < -1 || delta > 1) {
+            output.perror() << "argument out of range." << endl;
+            return HERBST_INVALID_ARGUMENT;
+        }
+        if (delta == 0) {
+            return HERBST_EXIT_SUCCESS; // nothing to do
+        }
+        cycleAll(delta == 1, skip_invisible);
+        return HERBST_EXIT_SUCCESS;
+    });
+}
+
+void HSTag::cycleAll(bool forward, bool skip_invisible)
+{
+    int delta = forward ? 1 : -1;
     if (floating_focused()) {
         int newIndex = static_cast<int>(floating_clients_focus_) + delta;
         // skip minimized clients
@@ -456,18 +482,33 @@ int HSTag::cycleAllCommand(Input input, Output output)
     }
     // finally, redraw the layout
     needsRelayout_.emit();
-    return 0;
 }
 
-void HSTag::cycleAllCompletion(Completion& complete)
+void HSTag::cycleCommand(CallOrComplete invoc)
 {
-    if (complete == 0) {
-        complete.full({"+1", "-1", "--skip-invisible" });
-    } else if (complete == 1 && complete[0] == "--skip-invisible") {
-        complete.full({"+1", "-1"});
-    } else {
-        complete.none();
-    }
+    int delta = 1;
+    ArgParse().optional(delta, {"+1", "-1"})
+              .command(invoc, [&] (Output) {
+        if (floating_focused()) {
+            if (floating_clients_.empty()) {
+                return 0;
+            }
+            int idx = static_cast<int>(floating_clients_focus_);
+            int count = static_cast<int>(floating_clients_.size());
+            idx = MOD(idx + delta, count);
+            floating_clients_focus_ = static_cast<size_t>(idx);
+            needsRelayout_.emit();
+        } else {
+            auto curFrame = frame->focusedFrame();
+            int idx = curFrame->getSelection();
+            auto count = static_cast<int>(curFrame->clientCount());
+            if (count != 0) {
+                curFrame->setSelection(MOD(idx + delta, count));
+            }
+            needsRelayout_.emit();
+        }
+        return 0;
+    });
 }
 
 int HSTag::resizeCommand(Input input, Output output)
@@ -481,12 +522,13 @@ int HSTag::resizeCommand(Input input, Output output)
     Client* client = focusedClient();
     if (client && client->is_client_floated()) {
         if (!Floating::resizeDirection(this, client, direction)) {
-            // no error message because this shouldn't happen anyway
+            output.perror() << "window " << Converter<Client*>::str(client)
+                            << " is too small to be shrunk" << endl;
             return HERBST_FORBIDDEN;
         }
     } else {
         if (!frame->resizeFrame(delta, direction)) {
-            output << input.command() << ": No neighbour found\n";
+            output.perror() << "No neighbour found\n";
             return HERBST_FORBIDDEN;
         }
     }
