@@ -241,81 +241,20 @@ void tree_print_to(shared_ptr<TreeInterface> intface, Output output) {
     subtree_print_to(intface, " ", rootIndicator, output);
 }
 
-char* posix_sh_escape(const char* source) {
-    size_t count = 0;
-    int i;
-    for (i = 0; source[i]; i++) {
-        int j = LENGTH(ESCAPE_CHARACTERS) - 1; // = strlen(ESCAPE_CHARACTERS)
-        slow_assert(j == strlen(ESCAPE_CHARACTERS));
-        while (j--) {
-            slow_assert(0 <= j && j < strlen(ESCAPE_CHARACTERS));
-            if (source[i] == ESCAPE_CHARACTERS[j]) {
-                count++;
-                break;
-            }
+string posix_sh_escape(const string& source) {
+    if (source.empty()) {
+        return "\'\'";
+    }
+    string target = "";
+    for (char ch : source) {
+        // escape everything in ESCAPE_CHARACTERS
+        bool needsEscape = strchr(ESCAPE_CHARACTERS, ch);
+        // and escape a tilde at the beginning of a string
+        needsEscape = needsEscape || (target.empty() && ch == '~');
+        if (needsEscape) {
+            target += '\\';
         }
+        target += ch;
     }
-    auto source_len = (size_t)i;
-    // special chars:
-    if (source[0] == '~') {
-        count++;
-    }
-    // if there is nothing to escape
-    if (count == 0) {
-        return nullptr;
-    }
-    // TODO migrate to new
-    char* target = (char*)malloc(sizeof(char) * (count + source_len + 1));
-
-    // do the actual escaping
-    // special chars:
-    int s = 0; // position in the source
-    int t = 0; // position in the target
-    slow_assert(s < strlen(source));
-    slow_assert(t < (count + source_len));
-    if (source[0] == '~') {
-        target[t++] = '\\';
-        target[t++] = source[s++];
-    }
-    slow_assert(s < strlen(source));
-    slow_assert(t < (count + source_len));
-    while (source[s]) {
-        // check if we need to escape the next char
-        int j = LENGTH(ESCAPE_CHARACTERS) - 1; // = strlen(ESCAPE_CHARACTERS)
-        slow_assert(s < strlen(source));
-        slow_assert(t < (count + source_len));
-        while (j--) {
-            if (source[s] == ESCAPE_CHARACTERS[j]) {
-                // if source[s] needs to be escape, then put an backslash first
-                target[t++] = '\\';
-                break;
-            }
-        }
-        slow_assert(s < strlen(source));
-        slow_assert(t < (count + source_len));
-        // put the actual character
-        target[t++] = source[s++];
-    }
-    slow_assert(s == strlen(source));
-    slow_assert(t == (count + source_len));
-    // terminate the string
-    target[t] = '\0';
     return target;
 }
-
-void posix_sh_compress_inplace(char* str) {
-    int offset = 0;
-    for (int i = 0; true ; i++) {
-        if (str[i] == '\\' && str[i + 1] ) {
-            str[i + offset] = str[i + 1];
-            i++;
-            offset --;
-        } else {
-            str[i + offset] = str[i];
-        }
-        if (!str[i]) {
-            break;
-        }
-    }
-}
-
