@@ -6,12 +6,14 @@
 #include <string>
 #include <vector>
 
-#include "types.h"
+#include "converter.h"
+
+class XConnection;
 
 class Color {
 public:
     Color();
-    Color(XColor xcol);
+    Color(XColor xcol, unsigned short alpha = 0xff);
     Color(std::string name);
 
     static Color black();
@@ -24,10 +26,15 @@ public:
     XColor toXColor() const;
     unsigned long toX11Pixel() const { return x11pixelValue_; }
 
+    static unsigned long x11PixelPlusAlpha(unsigned long x11pixel, unsigned short alpha) {
+        return (x11pixel & 0xffffffu) | (alpha << 24);
+    }
+
     bool operator==(const Color& other) const {
         return red_ == other.red_
             && green_ == other.green_
-            && blue_ == other.blue_;
+            && blue_ == other.blue_
+            && alpha_ == other.alpha_;
     };
     bool operator!=(const Color& other) const {
         return !operator==(other);
@@ -40,10 +47,10 @@ public:
     unsigned short red_ = 0;
     unsigned short green_ = 0;
     unsigned short blue_ = 0;
+    unsigned short alpha_ = 0xff; // 0 is fully transparent, 0xff is fully opaque
 
 private:
-
-    // the x11 internal pixel value.
+    // the x11 internal pixel value
     unsigned long x11pixelValue_ = 0;
 };
 
@@ -102,7 +109,12 @@ inline std::string Converter<WindowID>::str(WindowID payload) {
 template<>
 inline WindowID Converter<WindowID>::parse(const std::string &payload) {
     size_t bytes_read = 0;
-    unsigned long winid = std::stoul(payload, &bytes_read, 0);
+    unsigned long winid;
+    try {
+        winid = std::stoul(payload, &bytes_read, 0);
+    }  catch (...) {
+        throw std::invalid_argument("Window id is not numeric (decimal or 0xHEX)");
+    }
     if (bytes_read != payload.size()) {
         std::stringstream message;
         message << "invalid characters at position " << bytes_read
