@@ -4,8 +4,10 @@
 #include <X11/Xlib.h>
 #include <X11/Xproto.h>
 #include <X11/Xutil.h>
+#include <X11/extensions/Xrender.h>
 #include <unistd.h>
 #include <climits>
+#include <cstring>
 #include <iostream>
 
 #include "globals.h"
@@ -17,6 +19,7 @@ using std::string;
 using std::vector;
 
 bool XConnection::exitOnError_ = false;
+XConnection* XConnection::s_connection = nullptr;
 
 void XConnection::setExitOnError(bool exitOnError)
 {
@@ -30,6 +33,9 @@ XConnection::XConnection(Display* disp)
     m_screen_height = DisplayHeight(m_display, m_screen);
     m_root = RootWindow(m_display, m_screen);
     utf8StringAtom_ = XInternAtom(m_display, "UTF8_STRING", False);
+    visual_ = DefaultVisual(m_display, m_screen);
+    depth_ = DefaultDepth(m_display, m_screen);
+    colormap_ = DefaultColormap(m_display, m_screen);
 }
 
 XConnection::~XConnection() {
@@ -44,7 +50,8 @@ XConnection* XConnection::connect(string display_name) {
         std::cerr << "herbstluftwm: XOpenDisplay() failed" << endl;
         exit(EXIT_FAILURE);
     }
-    return new XConnection(d);
+    s_connection = new XConnection(d);
+    return s_connection;
 }
 
 static bool g_other_wm_running = false;
@@ -107,8 +114,12 @@ int XConnection::xerror(Display *dpy, XErrorEvent *ee) {
 }
 
 
-// from dwm.c
-bool XConnection::checkotherwm() {
+/**
+ * @brief Check whether another window manager listens
+ * for events on the root window.
+ * @return
+ */
+bool XConnection::otherWmListensRoot() {
     g_other_wm_running = False;
     g_xerrorxlib = XSetErrorHandler(xerrorstart);
     /* this causes an error if some other window manager is running */
@@ -123,10 +134,43 @@ bool XConnection::checkotherwm() {
     }
 }
 
+void XConnection::tryInitTransparency()
+{
+    XVisualInfo* infos;
+    XRenderPictFormat* fmt;
+    int nitems;
+    int i;
+
+    XVisualInfo tpl = {
+        nullptr,
+        0,
+        m_screen, // screen
+        32, // depth
+        TrueColor, // class
+        // remaining unused members:
+        0, 0, 0, 0, 0,
+    };
+    long masks = VisualScreenMask | VisualDepthMask | VisualClassMask;
+
+    infos = XGetVisualInfo(m_display, masks, &tpl, &nitems);
+    for(i = 0; i < nitems; i ++) {
+        fmt = XRenderFindVisualFormat(m_display, infos[i].visual);
+        if (fmt->type == PictTypeDirect && fmt->direct.alphaMask) {
+            visual_ = infos[i].visual;
+            depth_ = infos[i].depth;
+            colormap_ = XCreateColormap(m_display, m_root, visual_, AllocNone);
+            usesTransparency_ = true;
+            break;
+        }
+    }
+
+    XFree(infos);
+}
+
 Rectangle XConnection::windowSize(Window window) {
-    unsigned int border, depth;
-    int x, y;
-    unsigned int w, h;
+    unsigned int border = 0, depth = 0;
+    int x = 0, y = 0;
+    unsigned int w = 0, h = 0;
     XGetGeometry(m_display, window, &m_root, &x, &y, &w, &h, &border, &depth);
     // treat wanted coordinates as floating coords
     return { x, y, (int)w, (int)h };
@@ -200,30 +244,15 @@ static string iso_8859_1_to_utf8(const char* source) {
 }
 
 std::experimental::optional<string> XConnection::getWindowProperty(Window window, Atom atom) {
-    string result;
-    char** list = nullptr;
-    int n = 0;
-    XTextProperty prop;
-
-    if (0 == XGetTextProperty(m_display, window, &prop, atom)) {
-        return std::experimental::optional<string>();
+    std::experimental::optional<vector<string>> elements
+            = getWindowPropertyTextList(window, atom);
+    if (!elements.has_value()) {
+        return {};
     }
-    // convert text property to a gstring
-    if (prop.encoding == XA_STRING) {
-        // a XA_STRING is always encoded in ISO 8859-1
-        result = iso_8859_1_to_utf8(reinterpret_cast<char *>(prop.value));
-    } else if (prop.encoding == utf8StringAtom_) {
-        result = reinterpret_cast<char *>(prop.value);
-    } else {
-        if (XmbTextPropertyToTextList(m_display, &prop, &list, &n) >= Success
-            && n > 0 && *list)
-        {
-            result = *list;
-            XFreeStringList(list);
-        }
+    if (elements.value().empty()) {
+        return {};
     }
-    XFree(prop.value);
-    return result;
+    return elements.value()[0];
 }
 
 //! implement XChangeProperty for type=ATOM('UTF8_STRING')
@@ -244,9 +273,19 @@ void XConnection::setPropertyString(Window w, Atom property, const vector<string
         value_c_str.push_back(s.c_str());
     }
     XTextProperty text_prop;
-    Xutf8TextListToTextProperty(
-        m_display, (char**) value_c_str.data(), value_c_str.size(),
-        XUTF8StringStyle, &text_prop);
+    int status = Xutf8TextListToTextProperty(
+                    m_display, (char**) value_c_str.data(), value_c_str.size(),
+                    XUTF8StringStyle, &text_prop);
+    if (status != Success) {
+        // maybe no locale support.
+        status = XmbTextListToTextProperty(
+                    m_display, (char**) value_c_str.data(), value_c_str.size(),
+                    XStdICCTextStyle, &text_prop);
+    }
+    if (status != Success) {
+        HSDebug("Can not create text list\n");
+        return;
+    }
     XSetTextProperty(m_display, w, &text_prop, property);
     XFree(text_prop.value);
 }
@@ -346,22 +385,70 @@ std::experimental::optional<vector<Window>>
 std::experimental::optional<vector<string>>
     XConnection::getWindowPropertyTextList(Window window, Atom property)
 {
-    XTextProperty text_prop;
-    if (!XGetTextProperty(m_display, window, &text_prop, property)) {
+    Atom prop_type;
+    int format;
+    unsigned long bytes_left;
+    unsigned char* items_return;
+    unsigned long count;
+    int status = XGetWindowProperty(m_display, window,
+            property, 0, ULONG_MAX, False, AnyPropertyType,
+            &prop_type, &format, &count, &bytes_left,
+            &items_return);
+    if (Success != status || prop_type == None || format == 0) {
         return {};
     }
-    char** list_return;
-    int count;
-    if (Success != Xutf8TextPropertyToTextList(m_display, &text_prop, &list_return, &count)) {
-        XFree(text_prop.value);
+    if (format != 8) {
+        fprintf(stderr, "herbstluftwm: error: can not parse the"
+                        " atom \'%s\' of window 0x%lx: expected format=8 but got"
+                        " format=%d\n",
+                        atomName(property).c_str(), window, format);
+        XFree(items_return);
         return {};
     }
+    unsigned long offset = 0;
     vector<string> arguments;
-    for (int i = 0; i < count; i++) {
-        arguments.push_back(list_return[i]);
+    // the trailing 0 at items_return[count] might be crucial:
+    // if the string list ends with the empty string, then we
+    // need to access items_return[count].
+    while (offset <= count) {
+        unsigned char* textChunk = items_return + offset;
+        // let us hope that items_return is properly null-byte terminated.
+        unsigned long textChunkLen = strlen(reinterpret_cast<char*>(textChunk));
+        // copy into a new string object and convert to utf8 if necessary:
+        if (prop_type == XA_STRING) {
+            // a XA_STRING is always encoded in ISO 8859-1
+            arguments.push_back(iso_8859_1_to_utf8(reinterpret_cast<char*>(textChunk)));
+        } else if (prop_type == utf8StringAtom_) {
+            arguments.push_back(reinterpret_cast<char*>(textChunk));
+        } else {
+            // try to convert via XmbTextPropertyToTextList, just like
+            // xprop does:
+            XTextProperty textprop;
+            textprop.encoding = prop_type;
+            textprop.format = 8;
+            textprop.nitems = textChunkLen + 1;
+            textprop.value = textChunk;
+            int n = 0;
+            char** list = nullptr;
+            if (XmbTextPropertyToTextList(m_display, &textprop, &list, &n) == Success
+                && n > 0 && *list)
+            {
+                arguments.push_back(*list);
+                XFreeStringList(list);
+            } else {
+                fprintf(stderr, "herbstluftwm: error: can not parse the"
+                                " atom \'%s\' of window 0x%lx: unknown text format \'%s\'\n",
+                                atomName(property).c_str(),
+                                window,
+                                atomName(prop_type).c_str());
+                XFree(items_return);
+                return {};
+            }
+        }
+        // skip the string, and skip the null-byte
+        offset += textChunkLen + 1;
     }
-    XFreeStringList(list_return);
-    XFree(text_prop.value);
+    XFree(items_return);
     return { arguments };
 }
 
