@@ -8,6 +8,7 @@
 
 #include "client.h"
 #include "clientmanager.h"
+#include "command.h"
 #include "decoration.h"
 #include "desktopwindow.h"
 #include "ewmh.h"
@@ -32,6 +33,8 @@
 
 using std::function;
 using std::shared_ptr;
+using std::string;
+using std::vector;
 
 /** A custom event handler casting function.
  *
@@ -69,9 +72,14 @@ XMainLoop::XMainLoop(XConnection& X, Root* root)
     handlerTable_[ MotionNotify      ] = EH(&XMainLoop::motionnotify);
     handlerTable_[ PropertyNotify    ] = EH(&XMainLoop::propertynotify);
     handlerTable_[ UnmapNotify       ] = EH(&XMainLoop::unmapnotify);
+    handlerTable_[ SelectionClear    ] = EH(&XMainLoop::selectionclear);
 
+    // get events from hlwm:
     root_->monitors->dropEnterNotifyEvents
             .connect(this, &XMainLoop::dropEnterNotifyEvents);
+
+    root_->clients->dragged.changed()
+            .connect(this, &XMainLoop::draggedClientChanges);
 }
 
 //! scan for windows and add them to the list of managed clients
@@ -79,7 +87,7 @@ XMainLoop::XMainLoop(XConnection& X, Root* root)
 void XMainLoop::scanExistingClients() {
     XWindowAttributes wa;
     auto clientmanager = root_->clients();
-    auto& initialEwmhState = root_->ewmh->initialState();
+    auto& initialEwmhState = root_->ewmh_.initialState();
     auto& originalClients = initialEwmhState.original_client_list_;
     auto isInOriginalClients = [&originalClients] (Window win) {
         return originalClients.end()
@@ -91,7 +99,7 @@ void XMainLoop::scanExistingClients() {
                 return [] (ClientChanges&) {};
             }
             return [this,win] (ClientChanges& changes) {
-                long idx = this->root_->ewmh->windowGetInitialDesktop(win);
+                long idx = this->root_->ewmh_.windowGetInitialDesktop(win);
                 if (idx < 0) {
                     return;
                 }
@@ -111,16 +119,16 @@ void XMainLoop::scanExistingClients() {
         // but manage it if it was in the ewmh property _NET_CLIENT_LIST by
         // the previous window manager
         // TODO: what would dwm do?
-        if (root_->ewmh->isOwnWindow(win)) {
+        if (root_->ewmh_.isOwnWindow(win)) {
             continue;
         }
-        if (root_->ewmh->getWindowType(win) == NetWmWindowTypeDesktop)
+        if (root_->ewmh_.getWindowType(win) == NetWmWindowTypeDesktop)
         {
             DesktopWindow::registerDesktop(win);
-            DesktopWindow::lowerDesktopWindows();
+            root_->monitors->restack();
             XMapWindow(X_.display(), win);
         }
-        else if (root_->ewmh->getWindowType(win) == NetWmWindowTypeDock)
+        else if (root_->ewmh_.getWindowType(win) == NetWmWindowTypeDock)
         {
             root_->panels->registerPanel(win);
             XSelectInput(X_.display(), win, PropertyChangeMask);
@@ -147,6 +155,7 @@ void XMainLoop::scanExistingClients() {
         XReparentWindow(X_.display(), win, X_.root(), 0,0);
         clientmanager->manage_client(win, true, false, findTagForWindow(win));
     }
+    root_->monitors->restack();
 }
 
 
@@ -240,7 +249,7 @@ void XMainLoop::createnotify(XCreateWindowEvent* event) {
     if (root_->ipcServer_.isConnectable(event->window)) {
         root_->ipcServer_.addConnection(event->window);
         root_->ipcServer_.handleConnection(event->window,
-                                           HlwmCommon::callCommand);
+                                           XMainLoop::callCommand);
     }
 }
 
@@ -249,7 +258,7 @@ void XMainLoop::configurerequest(XConfigureRequestEvent* cre) {
     Client* client = root_->clients->client(cre->window);
     if (client) {
         bool changes = false;
-        auto newRect = client->float_size_;
+        Rectangle newRect = client->float_size_;
         if (client->sizehints_floating_ &&
             (client->is_client_floated() || client->pseudotile_))
         {
@@ -311,7 +320,7 @@ void XMainLoop::configurerequest(XConfigureRequestEvent* cre) {
             }
         } else {
         // FIXME: why send event and not XConfigureWindow or XMoveResizeWindow??
-            client->send_configure();
+            client->send_configure(true);
         }
     } else {
         // if client not known.. then allow configure.
@@ -329,16 +338,18 @@ void XMainLoop::configurerequest(XConfigureRequestEvent* cre) {
 }
 
 void XMainLoop::clientmessage(XClientMessageEvent* event) {
-    root_->ewmh->handleClientMessage(event);
+    root_->ewmh_.handleClientMessage(event);
 }
 
 void XMainLoop::configurenotify(XConfigureEvent* event) {
-    if (event->window == g_root) {
+    if (event->window == X_.root()) {
         root_->panels->rootWindowChanged(event->width, event->height);
         if (root_->settings->auto_detect_monitors()) {
             Input input = Input("detect_monitors");
             std::ostringstream void_output;
-            root_->monitors->detectMonitorsCommand(input, void_output);
+            // discard output, but forward errors to std:cerr
+            OutputChannels channels("", void_output, std::cerr);
+            root_->monitors->detectMonitorsCommand(input, channels);
         }
     } else {
         Rectangle geometry = { event->x, event->y, event->width, event->height };
@@ -447,7 +458,7 @@ void XMainLoop::mapnotify(XMapEvent* event) {
         }
         // also update the window title - just to be sure
         c->update_title();
-    } else if (!root_->ewmh->isOwnWindow(event->window)
+    } else if (!root_->ewmh_.isOwnWindow(event->window)
                && !is_herbstluft_window(X_.display(), event->window)) {
         // the window is not managed.
         HSDebug("MapNotify: briefly managing 0x%lx to apply rules\n", event->window);
@@ -459,7 +470,7 @@ void XMainLoop::maprequest(XMapRequestEvent* mapreq) {
     HSDebug("name is: MapRequest for 0x%lx\n", mapreq->window);
     Window window = mapreq->window;
     Client* c = root_->clients()->client(window);
-    if (root_->ewmh->isOwnWindow(window)
+    if (root_->ewmh_.isOwnWindow(window)
         || is_herbstluft_window(X_.display(), window))
     {
         // just map the window if it wants that
@@ -476,13 +487,13 @@ void XMainLoop::maprequest(XMapRequestEvent* mapreq) {
         c->minimized_ = false;
     } else {
         // c = nullptr, so the window is not yet managed.
-        if (root_->ewmh->getWindowType(window) == NetWmWindowTypeDesktop)
+        if (root_->ewmh_.getWindowType(window) == NetWmWindowTypeDesktop)
         {
             DesktopWindow::registerDesktop(window);
-            DesktopWindow::lowerDesktopWindows();
+            root_->monitors->restack();
             XMapWindow(X_.display(), window);
         }
-        else if (root_->ewmh->getWindowType(window) == NetWmWindowTypeDock)
+        else if (root_->ewmh_.getWindowType(window) == NetWmWindowTypeDock)
         {
             root_->panels->registerPanel(window);
             XSelectInput(X_.display(), window, PropertyChangeMask);
@@ -499,13 +510,23 @@ void XMainLoop::maprequest(XMapRequestEvent* mapreq) {
     }
 }
 
+void XMainLoop::selectionclear(XSelectionClearEvent* event)
+{
+    if (event->selection == root_->ewmh_.windowManagerSelection()
+        && event->window == root_->ewmh_.windowManagerWindow())
+    {
+        HSDebug("Getting replaced by another window manager. exiting.");
+        quit();
+    }
+}
+
 void XMainLoop::propertynotify(XPropertyEvent* ev) {
     // printf("name is: PropertyNotify\n");
     Client* client = root_->clients->client(ev->window);
     if (ev->state == PropertyNewValue) {
         if (root_->ipcServer_.isConnectable(ev->window)) {
             root_->ipcServer_.handleConnection(ev->window,
-                                               HlwmCommon::callCommand);
+                                               XMainLoop::callCommand);
         } else if (client != nullptr) {
             //char* atomname = XGetAtomName(X_.display(), ev->atom);
             //HSDebug("Property notify for client %s: atom %d \"%s\"\n",
@@ -521,7 +542,7 @@ void XMainLoop::propertynotify(XPropertyEvent* ev) {
                     m->applyLayout();
                 }
             } else if (ev->atom == XA_WM_NAME ||
-                       ev->atom == root_->ewmh->netatom(NetWmName)) {
+                       ev->atom == root_->ewmh_.netatom(NetWmName)) {
                 client->update_title();
             } else if (ev->atom == XA_WM_CLASS && client) {
                 // according to the ICCCM specification, the WM_CLASS property may only
@@ -529,7 +550,8 @@ void XMainLoop::propertynotify(XPropertyEvent* ev) {
                 // https://www.x.org/releases/X11R7.6/doc/xorg-docs/specs/ICCCM/icccm.html#wm_class_property
                 // If a client violates this, then the window rules like class=... etc are not applied.
                 // As a workaround, we do it now:
-                root_->clients()->applyRules(client, std::cerr);
+                auto stdio = OutputChannels::stdio();
+                root_->clients()->applyRules(client, stdio);
             }
         } else {
             root_->panels->propertyChanged(ev->window, ev->atom);
@@ -560,3 +582,39 @@ void XMainLoop::unmapnotify(XUnmapEvent* event) {
     }
 }
 
+IpcServer::CallResult XMainLoop::callCommand(const vector<string>& call)
+{
+    // the call consists of the command and its arguments
+    std::ostringstream output;
+    std::ostringstream error;
+    string commandName = (call.empty()) ? "" : call[0];
+    auto input =
+        (call.empty())
+        ? Input("", call)
+        : Input(call[0], vector<string>(call.begin() + 1, call.end()));
+    IpcServer::CallResult result;
+    OutputChannels channels(commandName, output, error);
+    result.exitCode = Commands::call(input, channels);
+    result.output = output.str();
+    result.error = error.str();
+    return result;
+}
+
+void XMainLoop::draggedClientChanges(Client* draggedClient)
+{
+    if (draggedClient) {
+        // listen for mouse motion events:
+        XGrabPointer(X_.display(), draggedClient->x11Window(), True,
+            PointerMotionMask|ButtonReleaseMask, GrabModeAsync,
+                GrabModeAsync, None, None, CurrentTime);
+    } else { // no client is dragged -> ungrab and clear the queue
+        XUngrabPointer(X_.display(), CurrentTime);
+        // remove all enternotify-events from the event queue that were
+        // generated by the XUngrabPointer
+        XEvent ev;
+        XSync(X_.display(), False);
+        while (XCheckMaskEvent(X_.display(), EnterWindowMask, &ev)) {
+        }
+    }
+}
+
