@@ -3,8 +3,10 @@
 #include <X11/X.h>
 #include <X11/Xlib.h>
 
+#include "ipc-server.h"
 #include "x11-types.h"
 
+class Client;
 class Root;
 class XConnection;
 
@@ -40,8 +42,15 @@ private:
     void motionnotify(XMotionEvent* event);
     void mapnotify(XMapEvent* event);
     void maprequest(XMapRequestEvent* mapreq);
+    void selectionclear(XSelectionClearEvent* event);
     void propertynotify(XPropertyEvent* event);
     void unmapnotify(XUnmapEvent* event);
 
     bool duringEnterNotify_ = false; //! whether we are in enternotify()
+
+    static IpcServer::CallResult callCommand(const std::vector<std::string>& call);
+
+
+    // handlers of events from hlwm
+    void draggedClientChanges(Client* draggedClient);
 };
