@@ -25,6 +25,9 @@ BINDIR = os.path.join(os.path.abspath(os.environ['PWD']))
 # * LSAN_OPTIONS: needed to suppress warnings about known memory leaks
 COPY_ENV_WHITELIST = ['LSAN_OPTIONS']
 
+# time in seconds to wait for a process to shut down
+PROCESS_SHUTDOWN_TIME = 10
+
 
 def extend_env_with_whitelist(environment):
     """Copy some whitelisted environment variables (if set) into an existing environment"""
@@ -96,8 +99,13 @@ class HlwmBridge(herbstluftwm.Herbstluftwm):
         assert proc.stdout == ""
         assert proc.stderr != ""
 
-        def f(self2, reg):
-            assert re.search(reg, self2.stderr)
+        def f(self2, needle, regex=True):
+            if regex:
+                assert re.search(needle, self2.stderr)
+            else:
+                assert self2.stderr.find(needle)
+            # allow to list multiple 'expect_stderr()' statements:
+            return self2
 
         proc.expect_stderr = types.MethodType(f, proc)
         return proc
@@ -186,6 +194,16 @@ class HlwmBridge(herbstluftwm.Herbstluftwm):
                 items.append(unescaped[0] if len(unescaped) else '')
         return sorted(items)
 
+    def command_has_all_args(self, cmd):
+        """
+        ask the completion and assert that the given command does not allow
+        any further arguments
+        """
+        res = self.unchecked_call(['complete_shell', len(cmd)] + cmd)
+        assert res.returncode == 7
+        assert res.stdout == ''
+        assert res.stderr == ''
+
     def list_children_via_attr(self, object_path):
         """
         List the names of children of the
@@ -239,12 +257,16 @@ class HlwmBridge(herbstluftwm.Herbstluftwm):
         return line[-1]
 
     def shutdown(self):
+        # first send SIGTERM to all processes, so they
+        # can shut down in parallel
         for client_proc in self.client_procs:
             client_proc.terminate()
-            client_proc.wait(2)
-
         self.hc_idle.terminate()
-        self.hc_idle.wait(2)
+
+        # and then wait for each of them to finish:
+        for client_proc in self.client_procs:
+            client_proc.wait(PROCESS_SHUTDOWN_TIME)
+        self.hc_idle.wait(PROCESS_SHUTDOWN_TIME)
 
     def bool(self, python_bool_var):
         """convert a boolean variable into hlwm's string representation"""
@@ -301,10 +323,16 @@ class HlwmProcess:
         stdout_bytes = bytes()
 
         def match_found():
-            if until_stdout and (until_stdout in stdout):
-                return True
-            if until_stderr and (until_stderr in stderr):
-                return True
+            if until_stdout:
+                if isinstance(until_stdout, str) and until_stdout in stdout:
+                    return True
+                if hasattr(until_stdout, 'search') and until_stdout.search(stdout):
+                    return True
+            if until_stderr:
+                if isinstance(until_stderr, str) and until_stderr in stderr:
+                    return True
+                if hasattr(until_stderr, 'search') and until_stderr.search(stderr):
+                    return True
             return False
 
         started = datetime.now()
@@ -425,18 +453,28 @@ class HlwmProcess:
             # only wait the process if it hasn't been cleaned up
             # this also avoids the second exception if hlwm crashed
             try:
-                assert self.proc.wait(2) == 0
+                assert self.proc.wait(PROCESS_SHUTDOWN_TIME) == 0
             except subprocess.TimeoutExpired:
                 self.proc.kill()
-                self.proc.wait(2)
+                self.proc.wait(PROCESS_SHUTDOWN_TIME)
                 raise Exception("herbstluftwm did not quit on sigterm"
                                 + " and had to be killed") from None
 
 
 class HcIdle:
-    def __init__(self, hlwm):
+    def __init__(self, hlwm, zero_separated=False):
+        """
+        hlwm is the HlwmBridge
+        zero_separated denotes whether the hooks are zero byte separated
+        (otherwise, they are newline-separated)
+        """
         self.hlwm = hlwm
-        self.proc = subprocess.Popen([hlwm.HC_PATH, '--idle'],
+        self.separator = b'\n'
+        command = [hlwm.HC_PATH, '--idle']
+        if zero_separated:
+            command = [hlwm.HC_PATH, '-0', '--idle']
+            self.separator = b'\x00'
+        self.proc = subprocess.Popen(command,
                                      stdout=subprocess.PIPE,
                                      bufsize=0)
         # we don't know how fast self.proc connects to hlwm.
@@ -451,17 +489,28 @@ class HcIdle:
         assert number_sent > 0
         number_received = 0
         while number_received < number_sent:
-            line = self.proc.stdout.readline().decode().rstrip('\n').split('\t')
+            line = self.read_hook()
             assert line[0] == 'hc_idle_bootup'
             number_received = int(line[1])
 
+    def read_hook(self):
+        """read exactly one hook. This blocks if there is on herbstclient's stdout none."""
+        line_bytes = b''
+        while True:
+            b = self.proc.stdout.read(1)
+            if b == self.separator:
+                break
+            else:
+                line_bytes += b
+        return line_bytes.decode().split('\t')
+
     def hooks(self):
         """return all hooks since the last call of this function"""
         # collect all hooks so far, so collect all up to the following hook:
         self.hlwm.call(['emit_hook', 'hc_idle_logging_marker'])
         hooks = []
         while True:
-            line = self.proc.stdout.readline().decode().rstrip('\n').split('\t')
+            line = self.read_hook()
             if line[0] == 'hc_idle_logging_marker':
                 break
             else:
@@ -471,10 +520,10 @@ class HcIdle:
     def shutdown(self):
         self.proc.terminate()
         try:
-            self.proc.wait(2)
+            self.proc.wait(PROCESS_SHUTDOWN_TIME)
         except subprocess.TimeoutExpired:
             self.proc.kill()
-            self.proc.wait(2)
+            self.proc.wait(PROCESS_SHUTDOWN_TIME)
 
 
 @pytest.fixture()
@@ -517,23 +566,42 @@ def xvfb(request):
     # also we add '-noreset' such that the server is not reset
     # when the last client connection is closed.
     #
-    # the optional parameter is the resolution. If you want another resolution,
-    # then annotate the test case with:
+    # the optional parameter is a dict for different settings. If you want
+    # another resolution, then annotate the test case with:
     #
-    #    @pytest.mark.parametrize("xvfb", [(1280, 1024)], indirect=True)
+    #    @pytest.mark.parametrize("xvfb", [{'resolution': (1280, 1024)}], indirect=True)
     #
-    screens = [(800, 600)]
+    parameters = {
+        'screens': [(800, 600)],
+        'xrender': True,
+    }
     if hasattr(request, 'param'):
-        screens = [request.param]
-    with MultiscreenDisplay(server='Xvfb', screens=screens, extra_args=['-noreset']) as xserver:
+        parameters.update(request.param)
+    if 'resolution' in parameters:
+        parameters['screens'] = [parameters['resolution']]
+    args = ['-noreset']
+    if parameters['xrender']:
+        args += ['+extension', 'RENDER']
+    with MultiscreenDisplay(server='Xvfb', screens=parameters['screens'], extra_args=args) as xserver:
         os.environ['DISPLAY'] = xserver.display
         yield xserver
 
 
 @pytest.fixture()
-def hlwm_process(hlwm_spawner, xvfb):
+def hlwm_process(hlwm_spawner, request, xvfb):
+    parameters = {
+        'transparency': True,
+    }
+    if hasattr(request, 'param'):
+        # the param must contain a dictionary possibly
+        # updating the default parameters
+        parameters.update(request.param)
+
     """Set up hlwm and also check that it shuts down gently afterwards"""
-    hlwm_proc = hlwm_spawner(['--no-tag-import'], display=xvfb.display)
+    additional_commandline = ['--no-tag-import']
+    if not parameters['transparency']:
+        additional_commandline += ['--no-transparency']
+    hlwm_proc = hlwm_spawner(additional_commandline, display=xvfb.display)
 
     yield hlwm_proc
 
@@ -552,11 +620,21 @@ def running_clients(hlwm, running_clients_num):
 @pytest.fixture()
 def x11_connection(xvfb):
     """Connect to the given xvfb display and return a Xlib.display handle"""
+    display = xlib_connect_to_display(xvfb.display)
+    yield display
+    display.close()
+
+
+def xlib_connect_to_display(display):
+    """try to connect to a display using Xlib.display.Display
+    and work around a bug in python-xlib.
+    The resuling display object still needs to be closed on shut down.
+    """
     display = None
     attempts_left = 10
     while display is None and attempts_left > 0:
         try:
-            display = Xlib.display.Display(xvfb.display)
+            display = Xlib.display.Display(display)
             # the above call may result in an exception:
             # ConnectionResetError: [Errno 104] Connection reset by peer
             # However, the handling of this error in the above function results in a
@@ -574,8 +652,7 @@ def x11_connection(xvfb):
             # wait for a moment, and then try again..
             time.sleep(2)
             attempts_left -= 1
-    yield display
-    display.close()
+    return display
 
 
 class RawImage:
@@ -605,138 +682,137 @@ class RawImage:
         return '#%02x%02x%02x' % rgb_triple
 
 
-@pytest.fixture()
-def x11(x11_connection):
-    """ Short-lived fixture for interacting with the X11 display and creating
-    clients that are automatically destroyed at the end of each test. """
-    class X11:
-        def __init__(self, x11_connection):
-            self.display = x11_connection
-            self.windows = set()
-            self.screen = self.display.screen()
-            self.root = self.screen.root
-            self.ewmh = ewmh.EWMH(self.display, self.root)
-            self.hlwm = hlwm
+class X11:
+    def __init__(self, x11_connection):
+        self.display = x11_connection
+        self.windows = set()
+        self.screen = self.display.screen()
+        self.root = self.screen.root
+        self.ewmh = ewmh.EWMH(self.display, self.root)
+        self.hlwm = hlwm
 
-        def window(self, winid_string):
-            """return python-xlib window wrapper for a string window id"""
-            winid_int = int(winid_string, 0)
-            return self.display.create_resource_object('window', winid_int)
+    def window(self, winid_string):
+        """return python-xlib window wrapper for a string window id"""
+        winid_int = int(winid_string, 0)
+        return self.display.create_resource_object('window', winid_int)
 
-        def winid_str(self, window_handle):
-            return hex(window_handle.id)
+    def winid_str(self, window_handle):
+        return hex(window_handle.id)
 
-        def make_window_urgent(self, window):
-            """make window urgent"""
-            window.set_wm_hints(flags=Xutil.UrgencyHint)
-            self.display.sync()
+    def make_window_urgent(self, window):
+        """make window urgent"""
+        window.set_wm_hints(flags=Xutil.UrgencyHint)
+        self.display.sync()
 
-        def is_window_urgent(self, window):
-            """check urgency of a given window handle"""
-            hints = window.get_wm_hints()
-            if hints is None:
-                return False
-            return bool(hints.flags & Xutil.UrgencyHint)
+    def is_window_urgent(self, window):
+        """check urgency of a given window handle"""
+        hints = window.get_wm_hints()
+        if hints is None:
+            return False
+        return bool(hints.flags & Xutil.UrgencyHint)
 
-        def set_property_textlist(self, property_name, value, utf8=True, window=None):
-            """set a ascii textlist property by its string name on the root window, or any other window"""
-            if window is None:
-                window = self.root
-            prop = self.display.intern_atom(property_name)
-            bvalue = bytearray()
-            isfirst = True
-            for entry in value:
-                if isfirst:
-                    isfirst = False
-                else:
-                    bvalue.append(0)
-                bvalue += entry.encode()
-            proptype = Xatom.STRING
+    def set_property_textlist(self, property_name, value, utf8=True, property_type=None, window=None):
+        """set a ascii textlist property by its string name on the root window, or any other window"""
+        if window is None:
+            window = self.root
+        prop = self.display.intern_atom(property_name)
+        bvalue = bytearray()
+        isfirst = True
+        for entry in value:
+            if isfirst:
+                isfirst = False
+            else:
+                bvalue.append(0)
+            bvalue += entry.encode()
+        if property_type is None:
+            property_type = Xatom.STRING
             if utf8:
-                proptype = self.display.get_atom('UTF8_STRING')
-            window.change_property(prop, proptype, 8, bytes(bvalue))
+                property_type = self.display.get_atom('UTF8_STRING')
+        window.change_property(prop, property_type, 8, bytes(bvalue))
 
-        def set_property_cardinal(self, property_name, value, window=None):
-            if window is None:
-                window = self.root
-            prop = self.display.intern_atom(property_name)
-            window.change_property(prop, Xatom.CARDINAL, 32, value)
+    def set_property_cardinal(self, property_name, value, window=None):
+        if window is None:
+            window = self.root
+        prop = self.display.intern_atom(property_name)
+        window.change_property(prop, Xatom.CARDINAL, 32, value)
 
-        def get_property(self, property_name, window=None):
-            """get a property by its string name from the root window, or any other window"""
-            if window is None:
-                window = self.root
-            prop = self.display.intern_atom(property_name)
-            resp = window.get_full_property(prop, X.AnyPropertyType)
-            return resp.value if resp is not None else None
+    def get_property(self, property_name, window=None):
+        """get a property by its string name from the root window, or any other window"""
+        if window is None:
+            window = self.root
+        prop = self.display.intern_atom(property_name)
+        resp = window.get_full_property(prop, X.AnyPropertyType)
+        return resp.value if resp is not None else None
 
-        def create_client(self, urgent=False, pid=None,
-                          geometry=(50, 50, 300, 200),  # x, y, width, height
-                          force_unmanage=False,
-                          sync_hlwm=True,
-                          wm_class=None,
-                          window_type=None,
-                          transient_for=None,
-                          pre_map=lambda wh: None,  # called before the window is mapped
-                          ):
-            w = self.root.create_window(
-                geometry[0],
-                geometry[1],
-                geometry[2],
-                geometry[3],
-                2,
-                self.screen.root_depth,
-                X.InputOutput,
-                X.CopyFromParent,
-                background_pixel=self.screen.white_pixel,
-                override_redirect=force_unmanage,
-            )
-            if wm_class is not None:
-                w.set_wm_class(wm_class[0], wm_class[1])
+    def create_client(self, urgent=False, pid=None,
+                      geometry=(50, 50, 300, 200),  # x, y, width, height
+                      force_unmanage=False,
+                      sync_hlwm=True,
+                      wm_class=None,
+                      window_type=None,
+                      transient_for=None,
+                      pre_map=lambda wh: None,  # called before the window is mapped
+                      ):
+        w = self.root.create_window(
+            geometry[0],
+            geometry[1],
+            geometry[2],
+            geometry[3],
+            2,
+            self.screen.root_depth,
+            X.InputOutput,
+            X.CopyFromParent,
+            background_pixel=self.screen.white_pixel,
+            override_redirect=force_unmanage,
+        )
+        if wm_class is not None:
+            w.set_wm_class(wm_class[0], wm_class[1])
 
-            if transient_for is not None:
-                w.set_wm_transient_for(transient_for)
+        if transient_for is not None:
+            w.set_wm_transient_for(transient_for)
 
-            # Keep track of window for later removal:
-            self.windows.add(w)
+        # Keep track of window for later removal:
+        self.windows.add(w)
 
-            w.set_wm_name('Some Window')
-            if urgent:
-                w.set_wm_hints(flags=Xutil.UrgencyHint)
+        w.set_wm_name('Some Window')
+        if urgent:
+            w.set_wm_hints(flags=Xutil.UrgencyHint)
 
-            if window_type is not None:
-                w.change_property(self.display.intern_atom('_NET_WM_WINDOW_TYPE'),
-                                  Xatom.ATOM,
-                                  32,
-                                  [self.display.intern_atom(window_type)])
+        if window_type is not None:
+            w.change_property(self.display.intern_atom('_NET_WM_WINDOW_TYPE'),
+                              Xatom.ATOM,
+                              32,
+                              [self.display.intern_atom(window_type)])
 
-            if pid is not None:
-                w.change_property(self.display.intern_atom('_NET_WM_PID'),
-                                  Xatom.CARDINAL,
-                                  32,
-                                  [pid])
+        if pid is not None:
+            w.change_property(self.display.intern_atom('_NET_WM_PID'),
+                              Xatom.CARDINAL,
+                              32,
+                              [pid])
 
-            pre_map(w)
-            w.map()
-            self.display.sync()
-            if sync_hlwm:
-                # wait for hlwm to fully recognize it as a client
-                self.sync_with_hlwm()
-            return w, self.winid_str(w)
+        pre_map(w)
+        w.map()
+        self.display.sync()
+        if sync_hlwm:
+            # wait for hlwm to fully recognize it as a client
+            self.sync_with_hlwm()
+        return w, self.winid_str(w)
 
-        def screenshot(self, win_handle):
-            geom = win_handle.get_geometry()
-            attr = win_handle.get_attributes()
-            # Xlib defines AllPlanes as: ((unsigned long)~0L)
-            all_planes = 0xffffffff
-            # Maybe, the following get_image() works differently
-            # than XGetImage(), because we still need to interpret
-            # the pixel values using the colormap, whereas the man
-            # page for XGetImage() does not mention 'colormap' at all
-            raw = win_handle.get_image(0, 0, geom.width, geom.height,
-                                       X.ZPixmap, all_planes)
-            # raw.data is a array of pixel-values, which need to
-            # be interpreted using the colormap:
+    def screenshot(self, win_handle) -> RawImage:
+        """screenshot of a windows content, not including its border"""
+        geom = win_handle.get_geometry()
+        attr = win_handle.get_attributes()
+        # Xlib defines AllPlanes as: ((unsigned long)~0L)
+        all_planes = 0xffffffff
+        # Maybe, the following get_image() works differently
+        # than XGetImage(), because we still need to interpret
+        # the pixel values using the colormap, whereas the man
+        # page for XGetImage() does not mention 'colormap' at all
+        raw = win_handle.get_image(0, 0, geom.width, geom.height,
+                                   X.ZPixmap, all_planes)
+        # raw.data is a array of pixel-values, which need to
+        # be interpreted using the colormap:
+        if raw.depth == 8:
             colorDict = {}
             for pixel in raw.data:
                 colorDict[pixel] = None
@@ -753,76 +829,94 @@ def x11(x11_connection):
             while width_padded % 4 != 0:
                 width_padded += 1
             rgbvals = [colorDict[p] for idx, p in enumerate(raw.data) if idx % width_padded < geom.width]
-            return RawImage(rgbvals, geom.width, geom.height)
+        else:
+            assert raw.depth in [32, 24]
+            # both for depth 32 and 24, the order is BGRA
+            pixelsize = 4
+            (blue, green, red) = (0, 1, 2)
+            size = geom.width * geom.height
+            assert len(raw.data) == pixelsize * size
+            rgbvals = [(raw.data[pixelsize * (y * geom.width + x) + red],
+                        raw.data[pixelsize * (y * geom.width + x) + green],
+                        raw.data[pixelsize * (y * geom.width + x) + blue])
+                       for y in range(0, geom.height)
+                       for x in range(0, geom.width)]
+        return RawImage(rgbvals, geom.width, geom.height)
 
-        def decoration_screenshot(self, win_handle):
-            decoration = self.get_decoration_window(win_handle)
-            return self.screenshot(decoration)
+    def decoration_screenshot(self, win_handle):
+        decoration = self.get_decoration_window(win_handle)
+        return self.screenshot(decoration)
 
-        def sync_with_hlwm(self):
-            self.display.sync()
-            # wait for hlwm to flush all events:
-            hlwm_bridge = HlwmBridge.INSTANCE
-            assert hlwm_bridge is not None, "hlwm must be running"
-            hlwm_bridge.call('true')
+    def sync_with_hlwm(self):
+        self.display.sync()
+        # wait for hlwm to flush all events:
+        hlwm_bridge = HlwmBridge.INSTANCE
+        assert hlwm_bridge is not None, "hlwm must be running"
+        hlwm_bridge.call('true')
 
-        def get_decoration_window(self, window):
+    def get_decoration_window(self, window):
+        tree = window.query_tree()
+        if tree.root == tree.parent:
+            return None
+        else:
+            return tree.parent
+
+    def get_absolute_top_left(self, window):
+        """return the absolute (x,y) coordinate of the given window,
+        i.e. relative to the root window"""
+        x = 0
+        y = 0
+        while True:
+            # the following coordinates are only relative
+            # to the parent of window
+            geom = window.get_geometry()
+            print('Relative geometry of {} is: x={} y={} w={} h={}'.format(
+                  self.winid_str(window), geom.x, geom.y, geom.width, geom.height))
+            x += geom.x
+            y += geom.y
+            # check if the window's parent is already the root window
             tree = window.query_tree()
             if tree.root == tree.parent:
-                return None
-            else:
-                return tree.parent
+                break
+            # if it's not, continue at its parent
+            window = tree.parent
+        return (x, y)
 
-        def get_absolute_top_left(self, window):
-            """return the absolute (x,y) coordinate of the given window,
-            i.e. relative to the root window"""
-            x = 0
-            y = 0
-            while True:
-                # the following coordinates are only relative
-                # to the parent of window
-                geom = window.get_geometry()
-                print('Relative geometry of {} is: x={} y={} w={} h={}'.format(
-                      self.winid_str(window), geom.x, geom.y, geom.width, geom.height))
-                x += geom.x
-                y += geom.y
-                # check if the window's parent is already the root window
-                tree = window.query_tree()
-                if tree.root == tree.parent:
-                    break
-                # if it's not, continue at its parent
-                window = tree.parent
-            return (x, y)
+    def get_absolute_geometry(self, window):
+        """return the geometry of the window, where the top left
+        coordinate comes from get_absolute_top_left()
+        """
+        x, y = self.get_absolute_top_left(window)
+        geom = window.get_geometry()
+        geom.x = x
+        geom.y = y
+        return geom
 
-        def get_absolute_geometry(self, window):
-            """return the geometry of the window, where the top left
-            coordinate comes from get_absolute_top_left()
-            """
-            x, y = self.get_absolute_top_left(window)
-            geom = window.get_geometry()
-            geom.x = x
-            geom.y = y
-            return geom
+    def get_hlwm_frames(self):
+        """get list of window handles of herbstluftwm
+        frame decoration windows"""
+        cmd = ['xdotool', 'search', '--class', '_HERBST_FRAME']
+        frame_wins = subprocess.run(cmd,
+                                    stdout=subprocess.PIPE,
+                                    universal_newlines=True,
+                                    check=True)
+        res = []
+        for winid_decimal_str in frame_wins.stdout.splitlines():
+            res.append(self.window(winid_decimal_str))
+        return res
 
-        def get_hlwm_frames(self):
-            """get list of window handles of herbstluftwm
-            frame decoration windows"""
-            cmd = ['xdotool', 'search', '--class', '_HERBST_FRAME']
-            frame_wins = subprocess.run(cmd,
-                                        stdout=subprocess.PIPE,
-                                        universal_newlines=True,
-                                        check=True)
-            res = []
-            for winid_decimal_str in frame_wins.stdout.splitlines():
-                res.append(self.window(winid_decimal_str))
-            return res
+    def shutdown(self):
+        # Destroy all created windows:
+        for window in self.windows:
+            window.unmap()
+            window.destroy()
+        self.display.sync()
 
-        def shutdown(self):
-            # Destroy all created windows:
-            for window in self.windows:
-                window.unmap()
-                window.destroy()
-            self.display.sync()
+
+@pytest.fixture()
+def x11(x11_connection):
+    """ Short-lived fixture for interacting with the X11 display and creating
+    clients that are automatically destroyed at the end of each test. """
 
     x11_ = X11(x11_connection)
     yield x11_
@@ -848,9 +942,10 @@ class MultiscreenDisplay:
         # ---------------------------
         self.screen_rects = []
         current_x_offset = 0
+        depth = 24
         for idx, (width, height) in enumerate(screens):
             self.screen_rects.append([current_x_offset, 0, width, height])
-            geo = '{}x{}x8'.format(width, height)
+            geo = '{}x{}x{}'.format(width, height, depth)
             if server == 'Xvfb':
                 self.full_cmd += ['-screen', str(idx), geo]
             else:
@@ -892,7 +987,7 @@ class MultiscreenDisplay:
 
     def __exit__(self, type_param, value, traceback):
         self.proc.terminate()
-        self.proc.wait(5)
+        self.proc.wait(PROCESS_SHUTDOWN_TIME)
 
 
 @pytest.fixture()
