@@ -319,7 +319,7 @@ def test_dump_layout_other_tag(hlwm, command):
 @pytest.mark.parametrize("running_clients_num", [0, 5])
 @pytest.mark.parametrize("num_splits", [0, 2])
 @pytest.mark.parametrize("cycle_delta", [-2, -1, 0, 1, 2])
-def test_cycle(hlwm, running_clients, running_clients_num, num_splits, cycle_delta):
+def test_cycle_tiling(hlwm, running_clients, running_clients_num, num_splits, cycle_delta):
     for i in range(0, num_splits):
         hlwm.call('split explode')
     windex = int(hlwm.get_attr('tags.0.curframe_windex'))
@@ -335,22 +335,62 @@ def test_cycle(hlwm, running_clients, running_clients_num, num_splits, cycle_del
         assert winids[new_windex] == hlwm.get_attr('clients.focus.winid')
 
 
-@pytest.mark.parametrize("running_clients_num", [0, 1, 5])
-@pytest.mark.parametrize("index", [0, 1, 3, 5])
-def test_focus_nth(hlwm, running_clients, running_clients_num, index):
-    # bring the clients in the right order
-    layout = '(clients vertical:0 '
-    layout += ' '.join(running_clients)
-    layout += ')'
-    hlwm.call(['load', layout])
+def test_cycle_floating_layer(hlwm):
+    hlwm.call('rule floating=off')
+    clients = hlwm.create_clients(8)
 
-    # focus the n_th
-    hlwm.call('focus_nth {}'.format(index))
+    # move some clients to floating layer
+    # but do it manually to get the order right
+    count = 5
+    floating = []
+    for w in clients[0:count]:
+        hlwm.attr.clients[w].floating = 'on'
+        floating.append(w)
 
-    windex = int(hlwm.get_attr('tags.0.curframe_windex'))
-    assert windex == max(0, min(index, running_clients_num - 1))
-    if running_clients_num > 0:
-        assert hlwm.get_attr('clients.focus.winid') == running_clients[windex]
+    for idx in [0, 1, 4]:
+        for delta in [-2, -1, 0, 1, 2, 8]:
+            hlwm.call(['jumpto', floating[idx]])
+            hlwm.call(['cycle', delta])
+            # adding 'count' 3 times will make the value in
+            # the parentheses positive:
+            expected = (idx + delta + 3 * count) % count
+            assert hlwm.attr.clients.focus.winid() == floating[expected]
+
+
+def test_cycle_stays_in_floating_layer(hlwm):
+    # some tiling clients
+    hlwm.create_clients(2)
+
+    # some floating clients
+    hlwm.call('rule floating=on')
+    floating = hlwm.create_clients(3)
+    hlwm.call(['jumpto', floating[0]])
+
+    for winid in floating:
+        assert hlwm.attr.clients.focus.winid() == winid
+        # the floating layer always stays focused
+        assert hlwm.attr.tags.focus.floating_focused() is True
+        hlwm.call(['cycle', '+1'])
+
+    # we're back a the first floating client:
+    assert hlwm.attr.clients.focus.winid() == floating[0]
+    assert hlwm.attr.tags.focus.floating_focused() is True
+
+
+def test_cycle_empty_scope(hlwm):
+    for floating in [True, False]:
+        hlwm.attr.tags.focus.floating = floating
+
+        hlwm.call('cycle -1')
+        hlwm.call('cycle +1')
+        hlwm.call('cycle 0')
+
+        assert hlwm.attr.tags.focus.tiling.focused_frame.selection() == 0
+
+
+def test_cycle_completion(hlwm):
+    assert '-1' in hlwm.complete(['cycle'])
+    hlwm.command_has_all_args(['cycle', '+1'])
 
 
 @pytest.mark.parametrize("running_clients_num", [5])
@@ -454,6 +494,49 @@ def test_cycle_all_errors(hlwm):
         .expect_stderr('Cannot.*"-s"')
     hlwm.call_xfail('cycle_all --skip-invisible -s 1') \
         .expect_stderr('Cannot.*"-s"')
+    hlwm.call_xfail('cycle_all --skip-invisible -s 1') \
+        .expect_stderr('Cannot.*"-s"')
+    hlwm.call_xfail('cycle_all 1 2 3') \
+        .expect_stderr('Unknown argument.*2')
+    hlwm.call_xfail('cycle_all 1 2 --skip-invisible') \
+        .expect_stderr('Unknown argument.*2')
+    hlwm.call_xfail('cycle_all 1 --skip-invisible 3') \
+        .expect_stderr('Unknown argument.*3')
+
+
+def test_cycle_all_optionality(hlwm):
+    # on the other hand, the error handling should not be triggered
+    # by the following:
+    hlwm.call('split explode')
+    hlwm.call('split explode')
+
+    def layout(focus1, focus2):
+        return normalize_layout_string(f"""
+            (split horizontal:0.5:{focus1}
+                (clients max:0)
+                (split vertical:0.5:{focus2}
+                    (clients max:0)
+                    (clients max:0)))
+        """)
+
+    # on the above layout, the following args must have
+    # identical results
+    for args in [[], ['--skip-invisible', '+1'], ['+1', '--skip-invisible'], ['+1']]:
+        hlwm.call(['load', layout(0, 1)])
+        hlwm.call(['cycle_all'] + args)
+        assert hlwm.call('dump').stdout == layout(1, 0)
+
+
+def test_cycle_all_completion(hlwm):
+    assert hlwm.complete(['cycle_all']) == ['+1', '--skip-invisible', '-1']
+    assert hlwm.complete(['cycle_all', '--skip-invisible']) == ['+1', '-1']
+    assert hlwm.complete(['cycle_all', '-1']) == ['--skip-invisible']
+    hlwm.command_has_all_args(['cycle_all', '-1', '--skip-invisible'])
+    hlwm.command_has_all_args(['cycle_all', '--skip-invisible', '+1'])
+    # passing too many arguments still results in no completions:
+    hlwm.command_has_all_args(['cycle_all', '1', '2', '3'])
+    hlwm.command_has_all_args(['cycle_all', '1', '2', '3', '4'])
+    hlwm.command_has_all_args(['cycle_all', '1', '2', '3', '4', '5'])
 
 
 @pytest.mark.parametrize("running_clients_num", [4])
@@ -608,7 +691,70 @@ def test_split_simple(hlwm, running_clients, align, fraction):
 
 def test_split_invalid_alignment(hlwm):
     hlwm.call_xfail('split foo') \
-        .expect_stderr('split: Invalid alignment "foo"')
+        .expect_stderr('Cannot parse argument "foo": Expecting.*top')
+
+
+def test_split_completion(hlwm):
+    modes = hlwm.complete(['split'])
+    assert 'auto' in modes
+    assert 'bottom' in modes
+    assert 'top' in modes
+    assert hlwm.complete(['split', 'auto']) == [f'0.{i}' for i in range(1, 10)]
+
+
+def test_split_invalid_arg(hlwm):
+    hlwm.call_xfail(['split', 'uto']) \
+        .expect_stderr('.*"uto": Expecting.*top')
+
+
+def test_split_incomple_mode_name(hlwm):
+    assert hlwm.attr.tags.focus.frame_count() == 1
+    hlwm.call(['split', 'e'])  # the same as 'explode'
+    assert hlwm.attr.tags.focus.frame_count() == 2
+
+
+def test_split_equivalent_modes(hlwm):
+    # all names in the same group should behave identical
+    mode_groups = [
+        ['t', 'to', 'top'],
+        ['b', 'bot', 'bottom', 'vertical'],
+        ['a', 'auto'],
+        ['l', 'left'],
+        ['r', 'right', 'horizontal', 'h'],
+        ['e', 'explode'],
+    ]
+    for g in mode_groups:
+        hlwm.call(['load', '(clients vertical:0)'])  # reset
+        hlwm.call(['split', g[0]])
+        expected_layout = hlwm.call('dump').stdout
+
+        for mode in g:
+            hlwm.call(['load', '(clients vertical:0)'])  # reset
+            hlwm.call(['split', mode])
+            assert hlwm.call('dump').stdout == expected_layout
+
+
+def test_split_modes_have_different_result(hlwm):
+    hlwm.create_clients(2)
+    layouts_so_far = set()
+    for mode in ['top', 'bottom', 'left', 'right', 'explode']:
+        hlwm.call(['load', '(clients grid:0)'])  # reset
+
+        hlwm.call(['split', mode])
+
+        layout = hlwm.call(['dump']).stdout
+        assert layout not in layouts_so_far
+
+        layouts_so_far.add(layout)
+
+
+def test_split_fraction(hlwm):
+    for mode in ['expl', 'auto', 'h', 'v']:
+        hlwm.call(['load', '(clients grid:0)'])  # reset
+
+        hlwm.call(['split', mode, '0.3'])
+
+        assert hlwm.attr.tags.focus.tiling.root.fraction() == '0.3'
 
 
 @pytest.mark.parametrize("align", ["horizontal", "vertical"])
@@ -983,13 +1129,19 @@ def test_focus_edge_shift_edge(hlwm, command):
 
     # we're on the leftmost frame
     layout_before = hlwm.call('dump').stdout
-    hlwm.call([command, 'left'])
+
+    proc = hlwm.unchecked_call([command, 'left'])
+    # we don't check stderr, because currently, the commands
+    # print 'no neighbour found' in any case
+    assert proc.returncode == 0
+
     assert hlwm.attr.tags.focus.tiling.focused_frame.index() == '00'
     assert layout_before == hlwm.call('dump').stdout
     assert hlwm.get_attr('monitors.focus.index') == '0'
 
     # focus_edge/shift_edge goes to the rightmost frame
-    hlwm.call([command, 'right'])
+    proc = hlwm.unchecked_call([command, 'right'])
+    assert proc.returncode == 0
     # we're still on the first monitor
     assert hlwm.get_attr('monitors.focus.index') == '0'
     # but in the right-most frame
@@ -1375,3 +1527,260 @@ def test_shift_no_neighbour_frame(hlwm, split):
 
     hlwm.call_xfail('shift up') \
         .expect_stderr('No neighbour found')
+
+
+@pytest.mark.parametrize("cross_monitor_bounds", [True, False])
+def test_shift_to_other_monitor_if_allowed_by_setting(hlwm, cross_monitor_bounds):
+    hlwm.attr.settings.focus_crosses_monitor_boundaries = hlwm.bool(cross_monitor_bounds)
+    hlwm.call('add othertag')
+    # monitor 1 right of monitor 0
+    hlwm.call('set_monitors 800x600+0+0 800x600+800+0')
+
+    hlwm.call('rule focus=on switchtag=off')
+    winid, _ = hlwm.create_client()
+    # put another window on the other tag
+    hlwm.call('rule tag=othertag')
+    hlwm.create_client()
+    # but the 'winid' is focused on monitor 0
+    assert hlwm.attr.clients.focus.winid() == winid
+    assert hlwm.attr.monitors.focus.index() == 0
+
+    command = ['shift', 'right']
+    if cross_monitor_bounds:
+        # 'winid' gets moved to the monitor on the right
+        hlwm.call(command)
+        assert hlwm.attr.monitors.focus.index() == 1
+    else:
+        # the setting forbids that the window leaves the tag
+        hlwm.call_xfail(command) \
+            .expect_stderr("No neighbour found")
+    # in any case 'winid' is still focused
+    assert hlwm.attr.clients.focus.winid() == winid
+
+
+@pytest.mark.parametrize("floating", ['off', 'tag', 'window'])
+def test_shift_to_other_monitor_floating(hlwm, floating):
+    hlwm.call('add othertag')
+    # monitor 1 left of monitor 0
+    hlwm.call('set_monitors 800x600+800+0 800x600+0+0')
+    snap_gap = 8
+    hlwm.attr.settings.snap_gap = snap_gap
+    winid, _ = hlwm.create_client(position=(snap_gap, snap_gap))
+    if floating == 'tag':
+        hlwm.attr.tags[0].floating = 'on'
+    elif floating == 'window':
+        hlwm.attr.clients[winid].floating = 'on'
+    else:
+        assert floating == 'off'
+    assert hlwm.attr.clients.focus.winid() == winid
+    assert hlwm.attr.monitors.focus.index() == 0
+
+    hlwm.call('shift left')
+
+    assert hlwm.attr.clients.focus.winid() == winid
+    assert hlwm.attr.monitors.focus.index() == 1
+
+
+@pytest.mark.parametrize("floating", [True, False])
+def test_shift_stays_on_monitor(hlwm, floating):
+    hlwm.call('add othertag')
+    # monitor 1 below monitor 0
+    hlwm.call('set_monitors 800x600+0+0 800x600+0+800')
+    if floating:
+        hlwm.attr.tags.focus.floating = 'on'
+    else:
+        # create empty frame at the bottom
+        hlwm.call('split bottom')
+
+    winid, _ = hlwm.create_client(position=(0, 0))
+    assert hlwm.attr.clients.focus.winid() == winid
+    assert hlwm.attr.monitors.focus.index() == 0
+
+    # the new client is far away from the bottom edge of monitor 0.
+    # so shifting it downwards makes it stay on monitor 0
+    hlwm.call('shift down')
+
+    assert hlwm.attr.clients.focus.winid() == winid
+    assert hlwm.attr.monitors.focus.index() == 0
+
+    # now the client is on the bottom corner of the monitor.
+    # so shifting it again will make it enter monitor 1
+    hlwm.call('shift down')
+
+    assert hlwm.attr.clients.focus.winid() == winid
+    assert hlwm.attr.monitors.focus.index() == 1
+
+
+def test_shift_no_monitor_in_direction(hlwm):
+    hlwm.call('add othertag')
+    # monitor 1 below monitor 0
+    hlwm.call('set_monitors 800x600+0+0 800x600+0+800')
+    winid, _ = hlwm.create_client()
+
+    for direction in ['left', 'right', 'up']:
+        hlwm.call_xfail(['shift', direction]) \
+            .expect_stderr('No neighbour found')
+        assert hlwm.attr.clients.focus.winid() == winid
+
+
+def test_focus_shift_completion(hlwm):
+    for cmd in ['shift', 'focus', 'shift_edge', 'focus_edge']:
+        directions = ['down', 'up', 'left', 'right']
+        flags = ['-i', '-e']
+        assert sorted(directions + flags) == hlwm.complete([cmd])
+
+        assert sorted(flags) == hlwm.complete([cmd, 'down'])
+
+        assert sorted(directions + ['-i']) == hlwm.complete([cmd, '-e'])
+
+        # actually, passing both -i and -e makes no sense,
+        # but ArgParse does not know that the flags exclude each other
+        hlwm.command_has_all_args([cmd, 'down', '-i', '-e'])
+
+
+def test_frame_leaf_selection_change(hlwm):
+    """test the attribute FrameLeaf::selection"""
+    clients = hlwm.create_clients(3)
+
+    def layout(idx):
+        return f"(clients vertical:{idx} {clients[0]} {clients[1]} {clients[2]})"
+
+    hlwm.call(['load', layout(0)])
+
+    for i in range(0, 3):
+        hlwm.attr.tags.focus.tiling.focused_frame.selection = i
+        assert hlwm.attr.clients.focus.winid() == clients[i]
+        assert hlwm.call('dump').stdout == layout(i)
+
+
+def test_frame_leaf_selection_if_empty(hlwm):
+    assert hlwm.attr.tags.focus.tiling.root.selection() == 0
+    hlwm.attr.tags.focus.tiling.root.selection = 0
+    assert hlwm.attr.tags.focus.tiling.root.selection() == 0
+
+    hlwm.call_xfail('attr tags.focus.tiling.root.selection 1') \
+        .expect_stderr('out of range')
+
+    hlwm.call_xfail('attr tags.focus.tiling.root.selection 2') \
+        .expect_stderr('out of range')
+
+    hlwm.call_xfail('attr tags.focus.tiling.root.selection -1') \
+        .expect_stderr('out of range')
+
+
+def test_frame_split_selection_change(hlwm):
+    """test the attribute FrameSplit::selection"""
+    clients = hlwm.create_clients(2)
+
+    def layout(idx):
+        return normalize_layout_string(f"""
+            (split horizontal:0.5:{idx}
+                (clients vertical:0 {clients[0]})
+                (clients vertical:0 {clients[1]}))
+        """)
+
+    hlwm.call(['load', layout(0)])
+
+    for i in [0, 1]:
+        hlwm.attr.tags.focus.tiling.root.selection = i
+        assert hlwm.attr.clients.focus.winid() == clients[i]
+        assert hlwm.call('dump').stdout == layout(i)
+
+
+def test_frame_selection_invalid_arg(hlwm):
+    hlwm.create_clients(6)  # 6 clients
+    hlwm.call('split explode')  # 2 frames, so 3 clients per frame
+    hlwm.call('dump')
+
+    # for frame splits, the selection can be 0 or 1
+    hlwm.call('attr tags.focus.tiling.root.selection 0')  # no failure
+    hlwm.call('attr tags.focus.tiling.root.selection 1')  # no failure
+    hlwm.call_xfail('attr tags.focus.tiling.root.selection -1') \
+        .expect_stderr('out of range')
+    hlwm.call_xfail('attr tags.focus.tiling.root.selection 2') \
+        .expect_stderr('out of range')
+
+    # for frame leafs, the selection only must be lower than the client count
+    hlwm.call('attr tags.focus.tiling.focused_frame.selection 0')  # no failure
+    hlwm.call('attr tags.focus.tiling.focused_frame.selection 1')  # no failure
+    hlwm.call('attr tags.focus.tiling.focused_frame.selection 2')  # no failure
+    hlwm.call_xfail('attr tags.focus.tiling.focused_frame.selection -1') \
+        .expect_stderr('out of range')
+    hlwm.call_xfail('attr tags.focus.tiling.focused_frame.selection 3') \
+        .expect_stderr('out of range')
+
+
+def test_frame_split_attribute_size_changes(hlwm, x11):
+    """
+    Test that changing the attributes 'fraction' or 'split_type' of FrameSplit
+    affects window sizes
+    """
+    # two clients side by side:
+    win1, winid1 = x11.create_client()
+    win2, winid2 = x11.create_client()
+    layout = f'(split horizontal:0.5:0 (clients max:0 {winid1}) (clients max:0 {winid2}))'
+    layout = normalize_layout_string(layout)
+    for attr, value in [('fraction', '0.6'), ('split_type', 'vertical')]:
+        hlwm.call(['load', layout])
+        x11.sync_with_hlwm()
+        width1 = win1.get_geometry().width
+        width2 = win2.get_geometry().width
+
+        hlwm.attr.tags.focus.tiling.root[attr] = value
+        assert hlwm.attr.tags.focus.tiling.root[attr]() == value
+        assert hlwm.call('dump').stdout.strip() != layout
+
+        x11.sync_with_hlwm()
+        # both updated attributes must lead to an increase of width of
+        # the first client:
+        assert width1 < win1.get_geometry().width
+        assert width2 != win2.get_geometry().width
+
+
+def test_frame_split_fraction_invalid_arg(hlwm):
+    hlwm.call(['load', '(split horizontal:0.5:0 (clients max:0) (clients max:0))'])
+
+    # test values that clamp to 0.1
+    for val in ['0.0', '0', '-0.1', '0.05', '0.1', '0.09', '-5', '-0.11']:
+        # reset the fraction
+        hlwm.attr.tags.focus.tiling.root.fraction = '0.5'
+        assert hlwm.attr.tags.focus.tiling.root.fraction() == '0.5'
+
+        hlwm.attr.tags.focus.tiling.root.fraction = val
+        assert hlwm.attr.tags.focus.tiling.root.fraction() == '0.1'
+
+    # test values that clamp to 0.9
+    for val in ['12', '1', '0.9', '0.95', '0.9', '0.91', '12312', '+23']:
+        # reset the fraction
+        hlwm.attr.tags.focus.tiling.root.fraction = '0.87'
+        assert hlwm.attr.tags.focus.tiling.root.fraction() == '0.87'
+
+        hlwm.attr.tags.focus.tiling.root.fraction = val
+        assert hlwm.attr.tags.focus.tiling.root.fraction() == '0.9'
+
+
+def test_frame_leaf_algorithm_change(hlwm, x11):
+    """
+    Test that changing the layout algorithm affects window sizes
+    """
+    # two clients below each other
+    win1, winid1 = x11.create_client()
+    win2, winid2 = x11.create_client()
+    hlwm.call('set_layout vertical')
+
+    geom1_before = win1.get_geometry()
+    geom2_before = win2.get_geometry()
+    assert geom1_before.width == geom2_before.width
+    assert geom1_before.height == geom2_before.height
+
+    # put them side by side
+    hlwm.attr.tags.focus.tiling.root.algorithm = 'horizontal'
+
+    x11.sync_with_hlwm()
+    geom1_now = win1.get_geometry()
+    geom2_now = win2.get_geometry()
+    # side by side implies: width decreases but height increases
+    assert geom1_before.width > geom1_now.width
+    assert geom1_before.height < geom1_now.height
+    assert geom1_now.width == geom2_now.width
+    assert geom1_now.height == geom2_now.height
