@@ -37,6 +37,16 @@ def test_object_tree(hlwm):
     assert len(t2) > len(t3)
 
 
+def test_object_tree_invalid_arg(hlwm):
+    hlwm.call_xfail('object_tree nonexistent') \
+        .expect_stderr('No such object nonexistent')
+
+
+def test_object_tree_completion(hlwm):
+    assert 'tags.' in hlwm.complete(['object_tree'], partial=True)
+    hlwm.command_has_all_args(['object_tree', 'tags'])
+
+
 def test_substitute(hlwm):
     expected_output = hlwm.get_attr('tags.count') + '\n'
 
@@ -45,9 +55,18 @@ def test_substitute(hlwm):
     assert call.stdout == expected_output
 
 
+def test_substitute_completion(hlwm):
+    assert 'tags.' in hlwm.complete(['substitute', 'X'], partial=True)
+    hlwm.command_has_all_args(['substitute', 'X', 'tags.count', 'true'])
+    assert 'floating' in hlwm.complete(['substitute', 'X', 'tags.count'])
+    assert 'X' in hlwm.complete(['substitute', 'X', 'tags.count', 'floating'])
+    # the nested command has all args already, so don't complete the 'X' here:
+    hlwm.command_has_all_args(['substitute', 'X', 'tags.count', 'floating', 'on'])
+
+
 @pytest.mark.parametrize('prefix', ['set_attr settings.',
                                     'attr settings.',
-                                    'cycle_value ',
+                                    'cycle_value settings.',
                                     'set '])
 def test_set_attr_completion(hlwm, prefix):
     assert hlwm.complete(prefix + "swap_monitors_to_get_tag") \
@@ -77,7 +96,7 @@ def test_set_attr_can_not_set_writable(hlwm):
 def test_substitute_missing_attribute__command_treated_as_attribute(hlwm):
     call = hlwm.call_xfail('substitute X echo X')
 
-    assert call.stderr == 'The root object has no attribute "echo"\n'
+    assert call.stderr == 'substitute: The root object has no attribute "echo"\n'
 
 
 def test_substitute_command_missing(hlwm):
@@ -118,13 +137,15 @@ def test_sprintf_nested(hlwm):
 def test_sprintf_too_few_attributes__command_treated_as_attribute(hlwm):
     call = hlwm.call_xfail('sprintf X %s/%s tags.count echo X')
 
-    assert call.stderr == 'The root object has no attribute "echo"\n'
+    assert call.stderr == 'sprintf: The root object has no attribute "echo"\n'
 
 
 def test_sprintf_too_few_attributes_in_total(hlwm):
-    call = hlwm.call_xfail('sprintf X %s/%s tags.count')
+    hlwm.call_xfail('sprintf X %s/%s tags.count') \
+        .expect_stderr('sprintf: not enough arguments\n')
 
-    assert call.stderr == 'sprintf: not enough arguments\n'
+    hlwm.call_xfail('sprintf X %s/%c tags.count') \
+        .expect_stderr('sprintf: not enough arguments\n')
 
 
 def test_sprintf_command_missing(hlwm):
@@ -139,6 +160,18 @@ def test_sprintf_double_percentage_escapes(hlwm):
     assert call.stdout == '%\n'
 
 
+def test_sprintf_invalid_format(hlwm):
+    hlwm.call_xfail('sprintf X %Z echo X') \
+        .expect_stderr('invalid format type %Z at position 1')
+
+    hlwm.call_xfail('sprintf X Z% echo X') \
+        .expect_stderr('dangling % at the end')
+
+    # invalid argument in the completion:
+    call = hlwm.unchecked_call('complete 4 sprintf X dangling%')
+    assert call.returncode != 0
+
+
 def test_sprintf_completion_1_placeholder(hlwm):
     assert hlwm.complete('sprintf T %s', partial=True) \
         == sorted(hlwm.complete('get_attr', partial=True))
@@ -183,7 +216,7 @@ def test_attribute_completion(hlwm):
     assert complete('monitors.fooob') == []
     assert complete('monitors.fooo.bar') == []
     assert len(complete('monitors.focus.')) >= 8
-    assert complete('t') == ['tags.', 'theme.', 'tmp.']
+    assert complete('t') == ['tags.', 'theme.', 'tmp.', 'types.']
     assert complete('') == [child + '.' for child in hlwm.list_children_via_attr('')]
 
 
@@ -222,6 +255,19 @@ def test_new_attr_missing_prefix(hlwm, attrtype, path):
         .expect_stderr('must start with "my_"')
 
 
+def test_new_attr_invalid_type(hlwm):
+    hlwm.call_xfail(['new_attr', 'nonexistent', 'my_attr']) \
+        .expect_stderr('unknown type "nonexistent"')
+
+
+def test_new_attr_completion(hlwm):
+    assert 'bool' in hlwm.complete('new_attr')
+    assert 'my_' in hlwm.complete('new_attr bool', partial=True)
+    assert 'tags.my_' in hlwm.complete('new_attr bool tags.', position=2, partial=True)
+    # no completion yet for initial values.
+    hlwm.command_has_all_args(['new_attr', 'bool', 'tags.my_attr', 'true'])
+
+
 @pytest.mark.parametrize('attrtypevalues', ATTRIBUTE_TYPE_EXAMPLE_VALUES.items())
 @pytest.mark.parametrize('path', ['my_foo', 'monitors.my_bar'])
 def test_new_attr_is_writable(hlwm, attrtypevalues, path):
@@ -337,6 +383,19 @@ def test_compare_invalid_operator(hlwm):
     hlwm.call_xfail('compare monitors.count -= 1') \
         .expect_stderr('Cannot.* "-=": Expecting one of: =, !=,')
 
+    hlwm.call_xfail('compare tags.focus.name gt 23') \
+        .expect_stderr('operator "gt" only.* numeric types')
+
+
+def test_compare_invalid_argument(hlwm):
+    hlwm.call_xfail('compare monitors.focus.lock_tag = notboolean') \
+        .expect_stderr('cannot parse "notboolean".*only.*are valid booleans')
+
+
+def test_compare_completion(hlwm):
+    assert '!=' in hlwm.complete('compare tags.count')
+    hlwm.command_has_all_args(['compare', 'tags.count', '=', '23'])
+
 
 def test_compare_fallback_string_equal(hlwm):
     hlwm.call('set_layout max')
@@ -385,7 +444,8 @@ def test_chain_return_code(hlwm):
 
     assert p1.returncode > 1
     assert p1.returncode == p2.returncode
-    assert p2.stderr[0:5] == 'line\n'
+    assert p2.stdout[0:5] == 'line\n'
+    assert p2.stderr.split(':')[1:] == p1.stderr.split(':')[1:]
 
 
 def test_chain_nested(hlwm):
@@ -399,13 +459,13 @@ def test_chain_nested(hlwm):
 def test_chain_and_1(hlwm):
     proc = hlwm.unchecked_call('and , echo foo , false , echo bar')
     assert proc.returncode == 1
-    assert proc.stderr == 'foo\n'
+    assert proc.stdout == 'foo\n'
 
 
 def test_chain_and_2(hlwm):
     proc = hlwm.unchecked_call('and , echo foo , true , echo bar , false , echo baz')
     assert proc.returncode == 1
-    assert proc.stderr == 'foo\nbar\n'
+    assert proc.stdout == 'foo\nbar\n'
 
 
 def test_chain_or(hlwm):
@@ -521,8 +581,10 @@ def test_mktemp_complete(hlwm):
 def test_negate_command(hlwm):
     assert hlwm.call('! false').stdout == ''
     assert hlwm.call('! ! echo f').stdout == 'f\n'
-    hlwm.call_xfail('! echo test') \
-        .expect_stderr('test')
+    proc = hlwm.unchecked_call('! echo test')
+    assert proc.returncode == 1
+    assert proc.stdout == 'test\n'
+    assert proc.stderr == ''
 
 
 def test_negate_complete_cmd(hlwm):
@@ -532,7 +594,7 @@ def test_negate_complete_cmd(hlwm):
 
 def test_negate_complete_arg(hlwm):
     assert 'left' in hlwm.complete('! focus')
-    assert [] == hlwm.complete('! true')
+    hlwm.command_has_all_args(['!', 'true'])
 
 
 def test_integer_out_of_range(hlwm):
@@ -548,37 +610,20 @@ def test_integer_out_of_range(hlwm):
                 .expect_stderr('out of range')
 
 
-def test_tag_status_invalid_monitor(hlwm):
-    hlwm.call_xfail('tag_status foobar') \
-        .expect_stderr('Monitor "foobar" not found!')
-
-
-def test_tag_status(hlwm, x11):
-    hlwm.call('add foobar')
-    hlwm.call('add baz')
-    hlwm.call('add qux')
-    hlwm.create_client()
-    hlwm.call('move baz')
-    winid, _ = hlwm.create_client()
-    hlwm.call('move qux')
-    x11.make_window_urgent(x11.window(winid))
-
-    assert hlwm.call('tag_status').stdout == "\t#default\t.foobar\t:baz\t!qux\t"
-
-
-def test_jumpto_invalid_client(hlwm):
-    hlwm.call_xfail('jumpto foobar') \
-        .expect_stderr('Could not find client "foobar".')
-
-
-def test_raise_winid_missing(hlwm):
+def test_raise_lower_winid_missing(hlwm):
     hlwm.call_xfail('raise') \
         .expect_stderr('raise: not enough arguments\n')
 
+    hlwm.call_xfail('lower') \
+        .expect_stderr('lower: not enough arguments\n')
 
-def test_raise_invalid_winid(hlwm):
+
+def test_raise_lower_invalid_winid(hlwm):
     hlwm.call_xfail('raise foobar') \
-        .expect_stderr('Could not find client "foobar".')
+        .expect_stderr('Invalid format, expecting')
+
+    hlwm.call_xfail('lower foobar') \
+        .expect_stderr('Invalid format, expecting')
 
 
 def test_argparse_too_few_range(hlwm):
@@ -621,11 +666,12 @@ def test_foreach_tag_merge(hlwm):
     # the second loop iteration for 'othertag'
     expected = [
         'tags.by-name.default',
-        'merge_tag: Tag "othertag" not found',
         'tags.by-name.othertag'
     ]
-    proc = hlwm.call('foreach T tags.by-name chain , merge_tag othertag , echo T')
-    assert proc.stdout.splitlines() == expected
+    error = 'merge_tag: Cannot parse argument "othertag": no such tag: othertag'
+    proc = hlwm.call('foreach T tags.by-name chain , merge_tag othertag , echo T',
+                     allowed_stderr=re.compile(error))
+    assert sorted(proc.stdout.splitlines()) == sorted(expected)
 
 
 def test_foreach_exit_code_success(hlwm):
@@ -745,6 +791,17 @@ def test_help_on_objects(hlwm, path='', depth=8):
         test_help_on_objects(hlwm, path=newpath, depth=depth - 1)
 
 
+def test_help_invalid_arg(hlwm):
+    hlwm.call_xfail('help too many args') \
+        .expect_stderr('too many arguments')
+
+    hlwm.call_xfail('help') \
+        .expect_stderr('not enough arguments')
+
+    hlwm.call_xfail('help certainly_an_invalid_arg') \
+        .expect_stderr("No help found for 'certainly_an_invalid_arg'")
+
+
 def test_watch_no_arguments(hlwm):
     hlwm.call_xfail('watch').expect_stderr(
         'Expected one argument, but got only 0 arguments.'
