@@ -1,5 +1,5 @@
 import pytest
-import re
+from herbstluftwm.types import Rectangle
 
 
 string_props = [
@@ -24,6 +24,7 @@ consequences = [
     'manage',
     'index',
     'floating',
+    'floating_geometry',
     'pseudotile',
     'ewmhrequests',
     'ewmhnotify',
@@ -56,13 +57,13 @@ def test_add_simple_rule_with_dashes(hlwm):
 
 
 def test_add_many_labeled_rules(hlwm):
-    # Add set of rules with every consequence and every valid combination of
+    # Add set of rules with every valid combination of
     # property and match operator appearing at least once:
 
-    # Make a single, long list of all consequences (with unique rhs values):
+    # Make a single, long list of all consequences which accept any string:
     consequences_str = \
         ' '.join(['{}=a{}b'.format(c, idx)
-                  for idx, c in enumerate(consequences, start=4117)])
+                  for idx, c in enumerate(['tag', 'monitor'], start=4117)])
 
     # Make three sets of long conditions lists: for numeric matches, string
     # equality and regexp equality:
@@ -97,14 +98,16 @@ def test_add_rule_with_misformatted_argument(hlwm, command):
 def test_cannot_add_rule_with_empty_label(hlwm, command):
     call = hlwm.call_xfail(f'{command} label= class=Foo tag=bar')
 
-    assert call.stderr == 'rule: Rule label cannot be empty\n'
+    commandname = command.split(' ')[0]
+    assert call.stderr == f'{commandname}: Rule label cannot be empty\n'
 
 
 @pytest.mark.parametrize('command', ['rule', 'apply_tmp_rule --all'])
 def test_cannot_use_tilde_operator_for_rule_label(hlwm, command):
     call = hlwm.call_xfail(f'{command} label~bla class=Foo tag=bar')
 
-    assert call.stderr == 'rule: Unknown rule label operation "~"\n'
+    commandname = command.split(' ')[0]
+    assert call.stderr == f'{commandname}: Unknown rule label operation "~"\n'
 
 
 @pytest.mark.parametrize('command', ['rule', 'apply_tmp_rule --all'])
@@ -126,26 +129,6 @@ def test_add_rule_maxage_condition_integer(hlwm, command):
     call.expect_stderr('rule: Cannot parse integer from "foo"')
 
 
-@pytest.mark.parametrize('rulearg,errormsg', [
-    ("fullscreen=foo", 'only.*are valid booleans'),
-    ("keymask=(", 'Parenthesis is not closed'),
-    ("floatplacement=bar", 'Expecting one of: center, '),
-])
-@pytest.mark.parametrize('command', ['apply_rules', 'apply_tmp_rule'])
-def test_rule_parse_error_printed_at_client(hlwm, command, rulearg, errormsg):
-    winid, _ = hlwm.create_client()
-    full_cmd = [command, winid]
-    if command == 'apply_rules':
-        hlwm.call(['rule', rulearg])
-    else:
-        full_cmd += [rulearg]
-
-    proc = hlwm.call(full_cmd)
-    # since the command does not fail, the error
-    # message appears on stdout
-    assert re.search(errormsg, proc.stdout)
-
-
 @pytest.mark.parametrize('method', ['-F', '--all'])
 def test_remove_all_rules(hlwm, method):
     hlwm.call('rule class=Foo tag=bar')
@@ -176,9 +159,8 @@ def test_remove_labeled_rule(hlwm):
 
 
 def test_remove_nonexistent_rule(hlwm):
-    call = hlwm.call_xfail('unrule nope')
-
-    assert call.stderr == 'Couldn\'t find any rules with label "nope"'
+    hlwm.call_xfail('unrule nope') \
+        .expect_stderr('Couldn\'t find any rules with label "nope"')
 
 
 def test_singleuse_rule_disappears_after_matching(hlwm):
@@ -276,7 +258,7 @@ def test_not_flag(hlwm):
 def test_condition_must_come_after_negation(hlwm, negation):
     call = hlwm.call_xfail(['rule', negation])
 
-    assert call.stderr == f'Expected another argument after "{negation}" flag\n'
+    assert call.stderr == f'rule: Expected another argument after "{negation}" flag\n'
 
 
 def test_condition_string_match(hlwm):
@@ -337,12 +319,36 @@ def test_condition_class(hlwm):
     assert hlwm.get_attr('clients', winid, 'tag') == 'tag2'
 
 
-def test_consequence_invalid_argument(hlwm):
-    # TODO: make this command fail at some point:
-    hlwm.call('rule focus=not-a-boolean')
+@pytest.mark.parametrize('rulearg,errormsg', [
+    ("fullscreen=foo", 'only.*are valid booleans'),
+    ("keymask=(", 'Parenthesis is not closed'),
+    ("floatplacement=bar", 'Expecting one of: center, '),
+    ("floating_geometry=4x5+1024+1024", 'Rectangle too small'),
+    ("floating_geometry=totallywrong", 'Rectangle too small'),  # TODO: make rectangle parsing fail
+])
+@pytest.mark.parametrize('command', ['rule', 'apply_tmp_rule'])
+def test_consequence_parse_error_correct_error_message(hlwm, command, rulearg, errormsg):
+    full_cmd = [command, rulearg]
+    if command == 'apply_tmp_rule':
+        winid, _ = hlwm.create_client()
+        full_cmd = [command, winid, rulearg]
 
-    # this must not crash:
-    hlwm.create_client()
+    hlwm.call_xfail(full_cmd) \
+        .expect_stderr(errormsg)
+
+
+def test_consequence_parse_error_general(hlwm):
+    """
+    Test that the rule parsing can fail for almost all consequences
+    and roughly check the error message
+    """
+    for c in consequences:
+        if c in ['tag', 'monitor', 'index', 'hook']:
+            continue
+        invalid_arg = 'invalid(arg'  # invalid for any type!
+        invalid_arg_escaped = r'invalid\(arg'
+        hlwm.call_xfail(['rule', f'{c}={invalid_arg}']) \
+            .expect_stderr(f'Invalid.*"{invalid_arg_escaped}".*to.*"{c}":')
 
 
 class RuleMode:
@@ -356,7 +362,7 @@ class RuleMode:
 def create_client(hlwm, rule_mode: RuleMode, rule):
     """create a client and the given rule. if client_before_rules, then
     first create the client, and later apply the rule to it via the
-    apply_rule command"""
+    apply_rules command"""
     if rule_mode == RuleMode.APPLY_RULES:
         winid, _ = hlwm.create_client()
         hlwm.call(['rule'] + rule)
@@ -631,8 +637,9 @@ def test_float_transient_for(hlwm, x11, transient_for):
     assert hlwm.get_attr(f'clients.{winid}.floating') == hlwm.bool(transient_for)
 
 
-def test_apply_rules_invalid_window(hlwm):
-    hlwm.call_xfail(['apply_rules', 'invalidWindowId']) \
+@pytest.mark.parametrize('command', ['apply_rules', 'apply_tmp_rule'])
+def test_apply_rules_invalid_window(hlwm, command):
+    hlwm.call_xfail([command, 'invalidWindowId']) \
         .expect_stderr(r'No such.*client')
 
 
@@ -741,16 +748,6 @@ def test_floatplacement_for_invisible_tag(hlwm, x11):
         == (500 / 2, 550 / 2)
 
 
-def test_floatplacement_parser(hlwm, hlwm_process, x11):
-    hlwm.call('rule floatplacement=notInTheList')
-    with hlwm_process.wait_stderr_match('Expecting one of: center, none'):
-        # here, it's important that we use a non-syncing way of creating
-        # a client, because the syncing would interfere with the
-        # above wait_stderr_match(). This is why we can't use
-        # hlwm.create_client() here
-        x11.create_client(sync_hlwm=False)
-
-
 def assert_that_rectangles_do_not_overlap(rectangles):
     """given a list of rectangles and a description (any string)
     verify that all the rectangles are disjoint. The rectangles
@@ -946,8 +943,85 @@ def test_apply_tmp_rule_focus(hlwm):
 
 
 def test_apply_tmp_rule_parse_error(hlwm, hlwm_process):
-    # FIXME: Here, we only check that it does not crash if we pass an
-    # unparseable argument.. In the future, this should be a proper error
-    # message for 'apply_tmp_rule'!
     hlwm.create_client()
-    hlwm.call('apply_tmp_rule --all focus=not-a-bool')
+    hlwm.call_xfail('apply_tmp_rule --all focus=not-a-bool') \
+        .expect_stderr('Invalid argument "not-a-bool" to consequence "focus"')
+
+
+def test_smart_placement_within_monitor(hlwm):
+    """
+    In the smart placement, the outer geometry of clients
+    must be used. So test that even with wide decorations,
+    the top left corner of the decoration is not off screen.
+    """
+    hlwm.attr.tags.focus.floating = 'on'
+    hlwm.call('rule floatplacement=smart')
+    bw = 25
+    hlwm.attr.theme.border_width = bw
+    hlwm.attr.settings.snap_gap = 5  # something smaller than bw
+    winid, _ = hlwm.create_client()
+
+    inner_geometry = hlwm.attr.clients[winid].content_geometry()
+
+    # assert that the top left corner of the decoration
+    # is still within the monitor
+    assert inner_geometry.x - bw >= 0
+    assert inner_geometry.y - bw >= 0
+
+
+@pytest.mark.parametrize('command', ['rule', 'apply_rules', 'apply_tmp_rule'])
+@pytest.mark.parametrize('visible_tag', [True, False])
+def test_floating_geometry(hlwm, x11, command, visible_tag):
+    if command != 'rule':
+        _, winid = x11.create_client()  # client first
+
+    geo = Rectangle(30, 40, 140, 170)
+    rule = ['floating=on', 'focus=on', 'floating_geometry=' + geo.to_user_str()]
+
+    if not visible_tag:
+        hlwm.call('add othertag')
+        rule += ['tag=othertag']
+
+    if command == 'rule':
+        hlwm.call(['rule'] + rule)
+        _, winid = x11.create_client()  # client second
+    elif command == 'apply_rules':
+        # apply fresh rules to long existing client
+        hlwm.call(['rule'] + rule)
+        hlwm.call(['apply_rules', winid])
+    else:
+        hlwm.call(['apply_tmp_rule', winid] + rule)
+
+    if not visible_tag:
+        hlwm.call('use othertag')
+
+    assert hlwm.attr.clients.focus.floating_geometry() == geo
+    assert hlwm.attr.clients.focus.content_geometry() == geo
+
+
+@pytest.mark.parametrize('place_center', [True, False])
+def test_floating_geometry_and_placement(hlwm, x11, place_center):
+    hlwm.attr.tags.focus.floating = True
+
+    rule_geo = Rectangle(30, 40, 140, 170)
+    rule = ['floating_geometry=' + rule_geo.to_user_str()]
+    if place_center:
+        rule += ['floatplacement=center']
+    else:
+        # also add another rule whose floatplacement is then overwritten:
+        hlwm.call('rule floatplacement=center')
+        rule += ['floatplacement=none']
+
+    # 'floatplacement' is only applied by 'rule', so no test
+    # for 'apply_tmp_rule' or 'apply_rules'
+    hlwm.call(['rule'] + rule)
+    _, winid = x11.create_client()
+
+    client_geo = hlwm.attr.clients[winid].floating_geometry()
+    monitor_geo = hlwm.attr.monitors.focus.geometry()
+
+    assert client_geo.size() == rule_geo.size()
+    if place_center:
+        assert client_geo.center() == monitor_geo.center()
+    else:
+        assert client_geo.topleft() == rule_geo.topleft()
