@@ -60,8 +60,8 @@ def test_raise_client_already_on_top(hlwm, floatingmode):
 
 def test_focused_tiling_client_stays_on_top_in_max_layout(hlwm):
     # in tiling mode, the focused window is always the top window within the
-    # tiling layer. Hence, trying to raise any other window must be undone
-    # immediately.
+    # tiling layer. Hence, trying to lower the focused window or to raise any
+    # other window must be undone immediately.
     #
     # the same holds for the 'selected' window within a max layout
     # hlwm.attr.settings.hide_covered_windows = 'off'
@@ -86,9 +86,10 @@ def test_focused_tiling_client_stays_on_top_in_max_layout(hlwm):
         assert stack.index(win[3]) < stack.index(win[4])
         assert stack.index(win[5]) < stack.index(win[6])
 
-    for winid in win:
-        hlwm.call(['raise', winid])
-        verify_stack()
+    for command in ['raise', 'lower']:
+        for winid in win:
+            hlwm.call([command, winid])
+            verify_stack()
 
 
 def test_raise_bottom_client(hlwm):
@@ -100,6 +101,55 @@ def test_raise_bottom_client(hlwm):
     assert helper_get_stack_as_list(hlwm, strip_focus_layer=True) == [c1, c2]
 
 
+def test_lower_topmost_client(hlwm):
+    hlwm.call('floating on')
+    hlwm.call('rule floating=on')
+    # new clients are placed on top, so put clients
+    # into stacking order by reversing the list:
+    clients = list(reversed(hlwm.create_clients(4)))
+    assert helper_get_stack_as_list(hlwm, strip_focus_layer=True) == clients
+
+    hlwm.call(['lower', clients[0]])
+
+    assert helper_get_stack_as_list(hlwm, strip_focus_layer=True) == clients[1:] + [clients[0]]
+
+
+@pytest.mark.parametrize('command', ['lower', 'raise'])
+def test_raise_lower_unmanaged_window(hlwm, x11, command):
+    hlwm.call('rule once manage=off')
+    _, winid = x11.create_client()
+
+    assert len(hlwm.list_children('clients')) == 0, \
+        "there must not be any managed client"
+
+    # we cannot test much, but at least it does not crash
+    # or throw any error:
+    hlwm.call([command, winid])
+
+    assert len(hlwm.list_children('clients')) == 0, \
+        "there must not be any managed client"
+
+
+def test_raise_lower_invalid_arg(hlwm):
+    for command in ['raise', 'lower']:
+        # both the error messages for WindowID and client should be shown:
+        hlwm.call_xfail([command, 'foobar']) \
+            .expect_stderr('Window id is not numeric') \
+            .expect_stderr('; or:') \
+            .expect_stderr("expecting 0xWINID or a description")
+
+        hlwm.call_xfail([command, '01two']) \
+            .expect_stderr('invalid characters at position 2')
+
+
+def test_raise_completion(hlwm):
+    assert 'urgent' in hlwm.complete('raise')
+
+    winid, _ = hlwm.create_client()
+
+    assert winid in hlwm.complete('raise')
+
+
 def create_two_monitors_with_client_each(hlwm):
     hlwm.call('add tag2')
     hlwm.call('set_attr tags.0.floating on')
@@ -191,10 +241,35 @@ def test_stack_tree(hlwm):
         - Client <windowid> "bash"
       - Frame Layer
         - Window <windowid>
+    - Desktop Windows
 '''
     assert strip_winids(stack.stdout) == expected_stack
 
 
+def test_stack_tree_desktop_windows(hlwm, x11):
+
+    _, winid = x11.create_client(wm_class=('TestDesktop', 'TestDeskClass'),
+                                 window_type='_NET_WM_WINDOW_TYPE_DESKTOP')
+
+    # Simplified tree style:
+    hlwm.call('set tree_style "     - -"')
+    framewindows = [x11.winid_str(w) for w in x11.get_hlwm_frames()]
+    expected_stack = f'''\
+  -
+    - Monitor 0 with tag "default"
+      - Focus-Layer
+      - Fullscreen-Layer
+      - Floating-Layer
+      - Tiling-Layer
+      - Frame Layer
+        - Window {framewindows[0]}
+    - Desktop Windows
+      - {winid} TestDesktop TestDeskClass
+'''
+    stack = hlwm.call('stack')
+    assert stack.stdout == expected_stack
+
+
 @pytest.mark.parametrize("focus_idx", range(0, 4))
 def test_tag_floating_state_on(hlwm, focus_idx):
     win = hlwm.create_clients(5)
@@ -249,7 +324,7 @@ def test_focused_on_other_monitor_above_fullscreen(hlwm, focus_other_monitor):
 
     if focus_other_monitor:
         hlwm.call(['jumpto', win_focused])
-        assert hlwm.attr.monitors.focus.index() == '1'
+        assert hlwm.attr.monitors.focus.index() == 1
         assert hlwm.attr.clients.focus.winid() == win_focused
         assert helper_get_stack_as_list(hlwm, strip_focus_layer=False) \
             == [win_focused, win_fullscreen]
